/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.core;

import java.sql.Connection;
import java.sql.SQLException;

/**
 * Attempts to connect to a database.
 * 
 * @author mbaumbach
 * 
 * @version 0.3.0
 */
public abstract class DatabaseConnection {

	/**
	 * Tests if able to get a connection to a database.
	 * 
	 * @param descriptor	The DatabaseDescriptor to use.
	 * 
	 * @return <code>true</code> if a connection could be established,
	 *         <code>false</code> otherwise.
	 */
	public boolean testConnect(DatabaseDescriptor descriptor) {
		boolean connected = false;

		try {
			connect(descriptor);
			connected = true;
		} catch (Exception e) {
			DatabaseLog.logError("Error connecting to database", e);
		}

		return connected;
	}

	/**
	 * Gets a connection to a database.
	 * 
	 * @param descriptor	The DatabaseDescriptor to obtain information from.
	 * 
	 * @return A java.sql.Connection to the database.
	 * 
	 * @throws SQLException
	 *             If an error occurs while connecting to the database
	 * @throws IllegalAccessException
	 *             If an error occurs while getting the driver class.
	 * @throws InstantiationException
	 *             If the driver class cannot be instantiated.
	 */
	public abstract Connection connect(DatabaseDescriptor descriptor)
		throws SQLException, InstantiationException, IllegalAccessException;

	/**
	 * Gets the default port for the specified database connection.
	 * 
	 * @return The default port, or <code>null</code> if no default is known.
	 */
	public abstract String getDefaultPort();

} // IDatabaseConnection
