/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.core;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

/**
 * Class provides a method for getting a connection to a database.
 * 
 * @author mbaumbach
 * 
 * @version 0.3.0
 */
public class DatabaseConnector {

	/**
	 * A reference to the only instance of this class.
	 */
	private static DatabaseConnector instance;

	/**
	 * Private constructor since this is a singleton. Use the getInstance()
	 * method to get an instance of this class.
	 */
	private DatabaseConnector() {
	}

	/**
	 * Gets an instance of this class.
	 * 
	 * @return A DatabaseConnector object.
	 */
	public static DatabaseConnector getInstance() {
		if (instance == null) {
			instance = new DatabaseConnector();
		}
		return instance;
	}

	/**
	 * Gets a connection to a database.
	 * 
	 * @param driver
	 *            The driver class to use.
	 * @param url
	 *            The URL to use to connect to the database.
	 * @param username
	 *            The username to connect with.
	 * @param password
	 *            The password to connect with.
	 * 
	 * @return A java.sql.Connection to the database.
	 * 
	 * @throws SQLException
	 *             If an error occurs while connecting to the database
	 * @throws IllegalAccessException
	 *             If an error occurs while getting the driver class.
	 * @throws InstantiationException
	 *             If the driver class cannot be instantiated.
	 */
	public Connection getConnection(Class driver, String url, String username,
			String password) throws SQLException, InstantiationException,
			IllegalAccessException {
		driver.newInstance();
		return DriverManager.getConnection(url, username, password);
	}

} // DatabaseConnection
