/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.core;

/**
 * Describes a database in a YML file.
 */
public class DatabaseDescriptor {

	private static final String DEFAULT_HOST = "localhost"; 
	
	private YMLFile parent;
	
	private String descriptorName;

	private String adapter;

	private String database;

	private String host;

	private String username;

	private String password;

	private String port;

	private String sslCert;

	private String sslCapath;

	private String sslCipher;

	private String socket;

	private String sslKey;

	private String schemaOrder;

	private String dbFile;

	public DatabaseDescriptor(String name) {
		this.descriptorName = name;
		this.adapter = "";
		this.database = "";
		this.host = DEFAULT_HOST;
		this.username = "";
		this.password = "";
		this.port = "";
		this.socket = "";
		this.sslCert = "";
		this.sslCapath = "";
		this.sslKey = "";
		this.schemaOrder = "";
		this.dbFile = "";
	}
	
	/**
	 * Creates a new DatabaseDescriptor.
	 * @deprecated
	 */
	public DatabaseDescriptor(YMLFile parent) {
		this.parent = parent;
		this.adapter = "";
		this.database = "";
		this.host = DEFAULT_HOST;
		this.username = "";
		this.password = "";
		this.port = "";
		this.socket = "";
		this.sslCert = "";
		this.sslCapath = "";
		this.sslKey = "";
		this.schemaOrder = "";
		this.dbFile = "";
	}
	
	/**
	 * Copy constructor.
	 * 
	 * @param parent
	 * @param desc
	 * @deprecated
	 */
	public DatabaseDescriptor(YMLFile parent, DatabaseDescriptor desc) {
		this.parent = parent;
		this.adapter = desc.adapter;
		this.database = desc.database;
		this.host = desc.host;
		this.username = desc.username;
		this.password = desc.password;
		this.port = desc.port;
		this.socket = desc.socket;
		this.sslCapath = desc.sslCapath;
		this.sslCert = desc.sslCert;
		this.sslCipher = desc.sslCipher;
		this.sslKey = desc.sslKey;
		this.schemaOrder = desc.schemaOrder;
		this.dbFile = desc.dbFile;
	}

	/**
	 * Mark this object as changed and notify the parent YMLfile.
	 * @deprecated
	 */
	public void changed() {
		parent.fireChangeEvent();
	}
	
	public String getName() {
		return descriptorName;
	}

	/**
	 * Gets the adapter.
	 * 
	 * @return Returns the adapter.
	 */
	public String getAdapter() {
		return adapter;
	}

	/**
	 * Sets the adapter to adapter.
	 * 
	 * @param adapter
	 *            The adapter to set.
	 */
	public void setAdapter(String adapter) {
		this.adapter = adapter;
	}

	/**
	 * Gets the database.
	 * 
	 * @return Returns the database.
	 */
	public String getDatabase() {
		return database;
	}

	/**
	 * Sets the database to database.
	 * 
	 * @param database
	 *            The database to set.
	 */
	public void setDatabase(String database) {
		this.database = database;
	}

	/**
	 * Gets the host.
	 * 
	 * @return Returns the host.
	 */
	public String getHost() {
		if ( host == null || host.trim().equals("") ) {
			host = DEFAULT_HOST;
		}
		return host;
	}

	/**
	 * Sets the host to host.
	 * 
	 * @param host
	 *            The host to set.
	 */
	public void setHost(String host) {
		this.host = host;
	}

	/**
	 * Gets the password.
	 * 
	 * @return Returns the password.
	 */
	public String getPassword() {
		return password;
	}

	/**
	 * Sets the password to password.
	 * 
	 * @param password
	 *            The password to set.
	 */
	public void setPassword(String password) {
		this.password = password;
	}

	/**
	 * Gets the username.
	 * 
	 * @return Returns the username.
	 */
	public String getUsername() {
		return username;
	}

	/**
	 * Sets the username to username.
	 * 
	 * @param username
	 *            The username to set.
	 */
	public void setUsername(String username) {
		this.username = username;
	}

	/**
	 * @return Returns the dbfile.
	 */
	public String getDBFile() {
		return dbFile;
	}

	/**
	 * @param dbfile
	 *            The dbfile to set.
	 */
	public void setDBFile(String dbfile) {
		this.dbFile = dbfile;
	}

	/**
	 * @return Returns the port.
	 */
	public String getPort() {
		return port;
	}

	/**
	 * @param port
	 *            The port to set.
	 */
	public void setPort(String port) {
		this.port = port;
	}

	/**
	 * @return Returns the schema_order.
	 */
	public String getSchemaOrder() {
		return schemaOrder;
	}

	/**
	 * @param schema_order
	 *            The schema_order to set.
	 */
	public void setSchemaOrder(String schema_order) {
		this.schemaOrder = schema_order;
	}

	/**
	 * @return Returns the socket.
	 */
	public String getSocket() {
		return socket;
	}

	/**
	 * @param socket
	 *            The socket to set.
	 */
	public void setSocket(String socket) {
		this.socket = socket;
	}

	/**
	 * @return Returns the sslcapath.
	 */
	public String getSSLCapath() {
		return sslCapath;
	}

	/**
	 * @param sslcapath
	 *            The sslcapath to set.
	 */
	public void setSSLCapath(String sslcapath) {
		this.sslCapath = sslcapath;
	}

	/**
	 * @return Returns the sslcert.
	 */
	public String getSSLCert() {
		return sslCert;
	}

	/**
	 * @param sslcert
	 *            The sslcert to set.
	 */
	public void setSSLCert(String sslcert) {
		this.sslCert = sslcert;
	}

	/**
	 * @return Returns the sslcipher.
	 */
	public String getSSLCipher() {
		return sslCipher;
	}

	/**
	 * @param sslcipher
	 *            The sslcipher to set.
	 */
	public void setSSLCipher(String sslcipher) {
		this.sslCipher = sslcipher;
	}

	/**
	 * @return Returns the sslkey.
	 */
	public String getSSLKey() {
		return sslKey;
	}

	/**
	 * @param sslkey
	 *            The sslkey to set.
	 */
	public void setSSLKey(String sslkey) {
		this.sslKey = sslkey;
	}

}