/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.core;

import java.util.Collection;

/**
 * Represents a result of an SQL query. Provides easy access to data returned as
 * well as the column names.
 * 
 * @author mbaumbach
 * 
 * @version 0.3.0
 */
public class QueryResult {

	/**
	 * A Collection of the names of the columns from the query's result.
	 */
	private Collection columnNames;

	/**
	 * A Collection of the rows return from the query's result.
	 */
	private Collection rows;

	/**
	 * Creates a new QueryResult object.
	 * 
	 * @param columnNames
	 *            A Collection of column names as Strings.
	 * @param rows
	 *            A Collection of rows as Collections of Objects.
	 */
	public QueryResult(Collection columnNames, Collection rows) {
		this.columnNames = columnNames;
		this.rows = rows;
	}

	/**
	 * @return Returns the columnNames.
	 */
	public Collection getColumnNames() {
		return columnNames;
	}

	/**
	 * @return Returns the rows.
	 */
	public Collection getRows() {
		return rows;
	}

	/**
	 * Gets the number of rows returned by the query.
	 * 
	 * @return The number of rows returned.
	 */
	public int getRowsReturned() {
		return rows.size();
	}

	/**
	 * Gets the number of columns in this result set.
	 * 
	 * @return The number of columns in the result set.
	 */
	public int getNumberOfColumns() {
		return columnNames.size();
	}

} // QueryResult
