/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.core.ibmdb2;

import java.sql.Connection;
import java.sql.SQLException;

import org.radrails.db.core.DatabaseConnection;
import org.radrails.db.core.DatabaseConnector;
import org.radrails.db.core.DatabaseDescriptor;

/**
 * Verifies a connection can be made with the database.
 * 
 * @author mbaumbach
 * 
 * @version 0.2.0
 */
public class IBMDB2DatabaseConnection extends DatabaseConnection {

	public static final String DB2_LOCALHOST = "localhost";

	public static final String DB2_DEFAULT_PORT = "50000";

	public static final String JDBC_DB2_URL = "jdbc:db2://";

	public static final String CONNECTION_URL = JDBC_DB2_URL
			+ DB2_LOCALHOST + ":" + DB2_DEFAULT_PORT;

	/**
	 * @see org.radrails.db.core.DatabaseConnection#getDefaultPort()
	 */
	public String getDefaultPort() {
		return IBMDB2DatabaseConnection.DB2_DEFAULT_PORT;
	}

	/**
	 * @see org.radrails.db.core.DatabaseConnection#connect(org.radrails.db.core.DatabaseDescriptor)
	 */
	public Connection connect(DatabaseDescriptor desc) throws SQLException,
			InstantiationException, IllegalAccessException {
		// If no port was specified, use the default port
		String port = desc.getPort();
		String host = desc.getHost();
		String database = desc.getDatabase();
		String username = desc.getUsername();
		String password = desc.getPassword();
		if (port == null || port.equals("") ) {
			port = getDefaultPort();
		}

		Class driver = com.ibm.db2.jcc.DB2Driver.class;
		String url = JDBC_DB2_URL + host + ":" + port + "/" + database;
		return DatabaseConnector.getInstance().getConnection(driver, url,
				username, password);
	}

} // MySQLDatabaseConnection
