/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.core.mysql;

import java.sql.Connection;
import java.sql.SQLException;

import org.radrails.db.core.DatabaseConnection;
import org.radrails.db.core.DatabaseConnector;
import org.radrails.db.core.DatabaseDescriptor;

/**
 * Verifies a connection can be made with the database.
 * 
 * @author mbaumbach
 * 
 * @version 0.2.0
 */
public class MySQLDatabaseConnection extends DatabaseConnection {

	public static final String MYSQL_LOCALHOST = "localhost";

	public static final String MYSQL_DEFAULT_PORT = "3306";

	public static final String JDBC_MYSQL_URL = "jdbc:mysql://";
	/**
	 * @see org.radrails.db.core.DatabaseConnection#getDefaultPort()
	 */
	public String getDefaultPort() {
		return MySQLDatabaseConnection.MYSQL_DEFAULT_PORT;
	}

	/**
	 * @see org.radrails.db.core.DatabaseConnection#connect(org.radrails.db.core.DatabaseDescriptor)
	 */
	public Connection connect(DatabaseDescriptor desc) throws SQLException,
			InstantiationException, IllegalAccessException {
		// If no port was specified, use the default port
		String port = desc.getPort();
		String host = desc.getHost();
		String database = desc.getDatabase();
		String username = desc.getUsername();
		String password = desc.getPassword();
		if (port == null || port.equals("") ) {
			port = getDefaultPort();
		}

		Class driver = com.mysql.jdbc.Driver.class;
		String url = JDBC_MYSQL_URL + host + ":" + port + "/" + database + "?zeroDateTimeBehavior=convertToNull";
		return DatabaseConnector.getInstance().getConnection(driver, url,
				username, password);
	}

} // MySQLDatabaseConnection
