/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.core.oracle;

import java.sql.Connection;
import java.sql.SQLException;

import org.radrails.db.core.DatabaseConnection;
import org.radrails.db.core.DatabaseConnector;
import org.radrails.db.core.DatabaseDescriptor;

/**
 * Verifies a connection can be made with the database.
 * 
 * @author mbaumbach
 * 
 * @version 0.2.0
 */
public class OracleDatabaseConnection extends DatabaseConnection {

	public static final String ORACLE_LOCALHOST = "localhost";

	public static final String ORACLE_DEFAULT_PORT = "1521";

	public static final String JDBC_ORACLE_URL = "jdbc:oracle:thin:@";

	public static final String CONNECTION_URL = JDBC_ORACLE_URL
			+ ORACLE_LOCALHOST + ":" + ORACLE_DEFAULT_PORT;

	/**
	 * @see org.radrails.db.core.DatabaseConnection#getDefaultPort()
	 */
	public String getDefaultPort() {
		return OracleDatabaseConnection.ORACLE_DEFAULT_PORT;
	}

	/**
	 * @see org.radrails.db.core.DatabaseConnection#connect(org.radrails.db.core.DatabaseDescriptor)
	 */
	public Connection connect(DatabaseDescriptor desc) throws SQLException,
			InstantiationException, IllegalAccessException {
		// If no port was specified, use the default port
		String port = desc.getPort();
		String host = desc.getHost();
		String username = desc.getUsername();
		String password = desc.getPassword();
		String database = desc.getDatabase();
		if (port == null || port.equals("null") || port.equals("") ) {
			port = getDefaultPort();
		}
		if ( host == null || host.equals("null") || host.equals("") ) {
			host = ORACLE_LOCALHOST;
		}
		if ( database != null && !database.equals("") ) {
			database = ":" + database;
		} else {
			// This will handle when the user enters in a host of something like: http://127.0.0.1/database
			String[] hostSplit = host.split("/", 2);
			host = hostSplit[0];
			if ( hostSplit.length > 1 ) {
				database = ":" + hostSplit[1];
			}
		}

		// Setup the oracle data source and connect to it
		Class driver = oracle.jdbc.driver.OracleDriver.class;
		String url = JDBC_ORACLE_URL + host + ":" + port + database;
		
		return DatabaseConnector.getInstance().getConnection(driver, url, username, password);
	}

} // OracleDatabaseConnection
