/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.core.postgresql;

import java.sql.Connection;
import java.sql.SQLException;

import org.radrails.db.core.DatabaseConnection;
import org.radrails.db.core.DatabaseConnector;
import org.radrails.db.core.DatabaseDescriptor;

/**
 * Verifies a connection can be made with the database.
 * 
 * @author mbaumbach
 * 
 * @version 0.3.0
 */
public class PostgreSQLDatabaseConnection extends DatabaseConnection {

	public static final String POSTGRESQL_LOCALHOST = "localhost";

	public static final String POSTGRESQL_DEFAULT_PORT = "5432";

	public static final String JDBC_POSTGRESQL_URL = "jdbc:postgresql://";

	public static final String CONNECTION_URL = JDBC_POSTGRESQL_URL
			+ POSTGRESQL_LOCALHOST + ":" + POSTGRESQL_DEFAULT_PORT;

	/**
	 * @see org.radrails.db.core.DatabaseConnection#connect(org.radrails.db.core.DatabaseDescriptor)
	 */
	public Connection connect(DatabaseDescriptor desc) throws SQLException,
			InstantiationException, IllegalAccessException {
		String port = desc.getPort();
		String host = desc.getHost();
		String database = desc.getDatabase();
		String username = desc.getUsername();
		String password = desc.getPassword();
		// If no port was specified, use the default port
		if (port == null || port.equals("") ) {
			port = getDefaultPort();
		}

		Class driver = org.postgresql.Driver.class;
		String url = JDBC_POSTGRESQL_URL + host + ":" + port + "/" + database;

		return DatabaseConnector.getInstance().getConnection(driver, url,
				username, password);
	}

	/**
	 * @see org.radrails.db.core.DatabaseConnection#getDefaultPort()
	 */
	public String getDefaultPort() {
		return PostgreSQLDatabaseConnection.POSTGRESQL_DEFAULT_PORT;
	}

} // MySQLDatabaseConnection
