/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.internal.ui;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

/**
 * Creates a ContentProvider for the DataNavigator.
 *
 * @author	mbaumbach
 *
 * @version	0.3.1
 */
public class DataContentProvider implements
		ITreeContentProvider {

	private Collection projects;
	
	/**
	 * Creates a new DataContentProvider.
	 */
	public DataContentProvider(){
		projects = new ArrayList();
	}
	
	/**
	 * Adds a project to the content provider.
	 * 
	 * @param	p	The project to add.
	 */
	public void add(Project p){
		projects.add(p);		
	}
	
	/**
	 * Removes a project from the content provider.
	 * 
	 * @param	projectName	The project's name to remove.
	 */
	public void remove(String projectName){
		Iterator i = projects.iterator();
		while ( i.hasNext() ){
			Project project = (Project)i.next();
			if ( projectName.equals(project.getName()) ){
				i.remove();
			}
		}
	}
	
	/**
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
	 */
	public Object[] getChildren(Object parentElement) {
		IDatabaseElement databaseElement = (IDatabaseElement)parentElement;
		return databaseElement.getChildren();
	}

	/**
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
	 */
	public Object getParent(Object element) {
		IDatabaseElement databaseElement = (IDatabaseElement)element;
		return databaseElement.getParent();
	}

	/**
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
	 */
	public boolean hasChildren(Object element) {
		IDatabaseElement databaseElement = (IDatabaseElement)element;
		return databaseElement.hasChildren();
	}

	/**
	 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
	 */
	public Object[] getElements(Object inputElement) {
		return projects.toArray();
	}

	/**
	 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
	 */
	public void dispose() {
		// Nothing to dispose
	}

	/**
	 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
	 */
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		// Input does not matter at the moment
	}

} // DataContentProvider