/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.internal.ui;

import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.swt.graphics.Image;

/**
 * The LabelProvider for the DataNavigator.
 *
 * @author	mbaumbach
 *
 * @version	0.3.1
 */
public class DataLabelProvider implements ILabelProvider {

	/**
	 * @see org.eclipse.jface.viewers.ILabelProvider#getImage(java.lang.Object)
	 */
	public Image getImage(Object element) {
		if ( element instanceof Project ){
			return DbUIPlugin.getImageDescriptor("icons/navigator_project.gif").createImage();
		}else if ( element instanceof Database ){
			 return DbUIPlugin.getImageDescriptor("icons/database.gif").createImage();
		}else if ( element instanceof Table ){
			 return DbUIPlugin.getImageDescriptor("icons/table.gif").createImage();
		}else if ( element instanceof Column ){
			Column col = (Column) element;
			if(col.isPrimaryKey()) {
				return DbUIPlugin.getImageDescriptor("icons/pk_column.gif").createImage();
			}
			else if(col.isForeignKey()) {
				return DbUIPlugin.getImageDescriptor("icons/fk_column.gif").createImage();
			}
			else {
			 return DbUIPlugin.getImageDescriptor("icons/column.gif").createImage();
			}
		}
		return null;
	}

	/**
	 * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
	 */
	public String getText(Object element) {
		IDatabaseElement e = (IDatabaseElement)element;
		return e.getName();
	}

	/**
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#addListener(org.eclipse.jface.viewers.ILabelProviderListener)
	 */
	public void addListener(ILabelProviderListener listener) {}

	/**
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
	 */
	public void dispose() {}

	/**
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#isLabelProperty(java.lang.Object, java.lang.String)
	 */
	public boolean isLabelProperty(Object element, String property) {
		return false;
	}

	/**
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#removeListener(org.eclipse.jface.viewers.ILabelProviderListener)
	 */
	public void removeListener(ILabelProviderListener listener) {}

} // DataLabelProvider
