/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.internal.ui;

import java.util.ArrayList;
import java.util.List;

/**
 * Represents a Database in the DataNavigator.
 *
 * @author	mbaumbach
 *
 * @version	0.3.1
 */
public class Database implements IDatabaseElement {

	private String name;
	private List tables;
	private Project parent;
	
	/**
	 * Creates a new Database.
	 * 
	 * @param	databaseName	The name of the database.
	 * @param	parent	The parent of the database.
	 */
	public Database(String databaseName, Project parent){
		this.name = databaseName;
		this.tables = new ArrayList();
		this.parent = parent;
	}
	
	/**
	 * @see org.radrails.db.internal.ui.IDatabaseElement#getParent()
	 */
	public IDatabaseElement getParent() {
		return parent;
	}
	
	/**
	 * Adds a table to this database.
	 * 
	 * @param	table	The table to add.
	 */
	public void addTable(Table table){
		tables.add(table);
	}
	
	/**
	 * Removes a table from this database.
	 * 
	 * @param	table	The table to remove.
	 */
	public void removeTable(Table table){
		tables.add(table);
	}

	/**
	 * @see org.radrails.db.internal.ui.IDatabaseElement#getChildren()
	 */
	public Object[] getChildren() {
		return tables.toArray();
	}

	/**
	 * @see org.radrails.db.internal.ui.IDatabaseElement#hasChildren()
	 */
	public boolean hasChildren() {
		return !tables.isEmpty();
	}

	/**
	 * @see org.radrails.db.internal.ui.IDatabaseElement#getName()
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * Clears this database of all tables.
	 */
	public void clear(){
		this.tables.clear();
	}
	
} // Database