/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.internal.ui;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.jface.resource.StringConverter;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;

/**
 * Color provider for the db ui plugin.
 * 
 * @author mkent
 * 
 */
public class DbUIColorProvider {

	private static DbUIColorProvider instance;

	private Map colorMap;

	private DbUIColorProvider() {
		colorMap = new HashMap();
	}

	/**
	 * @return the singleton instance of the color provider
	 */
	public static DbUIColorProvider getInstance() {
		if (instance == null) {
			instance = new DbUIColorProvider();
		}
		return instance;
	}

	/**
	 * Dispose the color resources held by the provider.
	 */
	public void dispose() {
		Iterator i = colorMap.values().iterator();
		while (i.hasNext())
			((Color) i.next()).dispose();
	}

	/**
	 * Creates a <code>Color</code> from the given <code>RGB</code> value.
	 * 
	 * @param rgb
	 *            the <code>RGB</code> value of the color
	 * @return a <code>Color</code> object
	 */
	public Color getColor(RGB rgb) {
		Color color = (Color) colorMap.get(rgb);
		if (color == null) {
			color = new Color(Display.getCurrent(), rgb);
			colorMap.put(rgb, color);
		}
		return color;
	}

	/**
	 * Creates a <code>Color</code> from the given string RGB value.
	 * 
	 * @param rgb
	 *            the string RGB value
	 * @return a <code>Color</code> object
	 */
	public Color getColor(String rgb) {
		return getColor(StringConverter.asRGB(rgb));
	}

	/**
	 * Creates a <code>Color</code> from a preference located in the
	 * <code>RailsUIPlugin</code> preference store.
	 * 
	 * @param prefId
	 *            the identifier of the preference
	 * @return a <code>Color</code> object
	 */
	public Color getColorFromPreference(String prefId) {
		RGB rgb = PreferenceConverter.getColor(DbUIPlugin.getInstance()
				.getPreferenceStore(), prefId);
		return getColor(rgb);
	}
}
