/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.internal.ui;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;
import org.radrails.db.core.DatabaseLog;
import org.radrails.db.internal.ui.editors.sql.SQLPartitionScanner;
import org.radrails.db.internal.ui.editors.sql.util.ColorProvider;

/**
 * The main plugin class to be used in the desktop.
 */
public class DbUIPlugin extends AbstractUIPlugin {

	//The shared instance.
	private static DbUIPlugin plugin;

	//Resource bundle.
	private ResourceBundle resourceBundle;

	ColorProvider colorProvider;

	private SQLPartitionScanner fPartitionScanner;

	public final static String SQL_PARTITIONING = "__sql_partitioning"; //$NON-NLS-1$

	/**
	 * The constructor.
	 */
	public DbUIPlugin() {
		super();
		plugin = this;
		try {
			resourceBundle = ResourceBundle
					.getBundle("org.radrails.db.internal.ui.editors.sql.SQLEditorPluginResources");
		} catch (MissingResourceException x) {
			resourceBundle = null;
			DatabaseLog.logError("Could not find resource bundle for Database UI plugin", x);
		}
		colorProvider = new ColorProvider();

	}

	/**
	 * This method is called upon plug-in activation
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
	}

	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
		plugin = null;
	}

	/**
	 * Returns the shared instance.
	 */
	public static DbUIPlugin getInstance() {
		return plugin;
	}

	/**
	 * Returns an image descriptor for the image file at the given
	 * plug-in relative path.
	 *
	 * @param path the path
	 * 
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return AbstractUIPlugin.imageDescriptorFromPlugin("org.radrails.db.ui",
				path);
	}

	/**
	 * Returns the colorProvider.
	 * 
	 * @return ColorProvider
	 */
	public ColorProvider getColorProvider() {
		return colorProvider;
	}

	/**
	 * Return a scanner for creating java partitions.
	 */
	public SQLPartitionScanner getSQLPartitionScanner() {
		if (fPartitionScanner == null)
			fPartitionScanner = new SQLPartitionScanner();
		return fPartitionScanner;
	}

	/**
	 * Returns the workspace instance.
	 */
	public static IWorkspace getWorkspace() {
		return ResourcesPlugin.getWorkspace();
	}

	/**
	 * Returns the string from the plugin's resource bundle, or 'key' if not
	 * found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle = DbUIPlugin.getInstance()
				.getResourceBundle();
		try {
			return bundle.getString(key);
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}

	/**
	 * Disposes the color provider.
	 */
	public void disposeColorProvider() {
		colorProvider.dispose();
	}
	
} // UiPlugin