/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.internal.ui;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.core.resources.IProject;
import org.radrails.db.core.ProjectDatabaseManager;

/**
 * Represents a Project in the DataNavigator.
 *
 * @author	mbaumbach
 *
 * @version	0.3.1
 */
public class Project implements IDatabaseElement {
	
	private IProject iProject;
	private Collection databases;
	private ProjectDatabaseManager pdm;
	
	/**
	 * Creates a new project.
	 * 
	 * @param	pdm	The ProjectDatabaseManager this project uses.
	 */
	public Project(ProjectDatabaseManager pdm){
		this.iProject = pdm.getIProject();
		this.databases = new ArrayList();
		this.pdm = pdm;
	}
	
	/**
	 * Adds the specified database to this project.
	 * 
	 * @param	database	The database to add.
	 */
	public void addDatabase(Database database){
		this.databases.add(database);
	}
	
	/**
	 * Gets all database belonging to this project.
	 * 
	 * @return	A Collection of the Databases.
	 */
	public Collection getDatabases(){
		return this.databases;
	}
	
	/**
	 * Clears all database belonging to this project.
	 */
	public void clearDatabases(){
		this.databases.clear();
	}
	
	/**
	 * @see org.radrails.db.internal.ui.IDatabaseElement#getName()
	 */
	public String getName() {
		return iProject.getName();
	}
	
	/**
	 * Gets the IProject this Project represents.
	 * 
	 * @return	The IProject this project represents.
	 */
	public IProject getProject(){
		return iProject;
	}
	
	/**
	 * Gets the ProjectDatabaseManager this project uses.
	 * 
	 * @return	The ProjectDatabaseManager this project uses.
	 */
	public ProjectDatabaseManager getProjectDatabaseManager(){
		return pdm;
	}

	/**
	 * @see org.radrails.db.internal.ui.IDatabaseElement#getParent()
	 */
	public IDatabaseElement getParent() {
		return null;
	}

	/**
	 * @see org.radrails.db.internal.ui.IDatabaseElement#getChildren()
	 */
	public Object[] getChildren() {
		return databases.toArray();
	}

	/**
	 * @see org.radrails.db.internal.ui.IDatabaseElement#hasChildren()
	 */
	public boolean hasChildren() {
		return !databases.isEmpty();
	}

} // Project