/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.internal.ui;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.ViewPart;
import org.radrails.db.core.DatabaseLog;

/**
 * The view for Queries.
 *
 * @author	mbaumbach
 *
 * @version	0.3.1
 */
public class QueryView extends ViewPart {

	/**
	 * @see org.eclipse.ui.IWorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {
		createQueryComposite(parent);
	}

	/**
	 * Creates a composite of Queries.
	 * 
	 * @param	parent	The composite this composite belongs to.
	 */
	private void createQueryComposite(Composite parent) {
		Composite composite = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		composite.setLayout(layout);
		GridData compositeData = new GridData(GridData.FILL_BOTH);
		composite.setLayoutData(compositeData);
		final Text query = new Text(composite, SWT.MULTI | SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL);
		GridData queryData = new GridData(GridData.FILL_BOTH);
		query.setLayoutData(queryData);
		
		Button execute = new Button(composite, SWT.PUSH);
		execute.setText("Execute");
		GridData executeData = new GridData(GridData.FILL_VERTICAL);
		execute.setLayoutData(executeData);
		execute.addSelectionListener(new SelectionListener() {

			/**
			 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			public void widgetSelected(SelectionEvent e) {
				openEditor(query.getText());
			}

			/**
			 * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			public void widgetDefaultSelected(SelectionEvent e) {}

		});

	}

	/**
	 * Opens the editor containing this view.
	 * 
	 * @param	queryString	The query string the view will represent.
	 */
	private void openEditor(String queryString) {
		try {
			StructuredSelection selected = (StructuredSelection)DataNavigator.getTreeViewer().getSelection();
			Database database = findDatabase((IDatabaseElement)selected.getFirstElement());
			if ( (database != null) && database.hasChildren() && (queryString.length() > 0)){
				IEditorInput editorInput = new DataEditorInput(queryString,database);
				getViewSite().getPage().openEditor(editorInput, "org.radrails.db.ui.DataEditor");
			}else if (database == null){
				MessageDialog.openWarning(getViewSite().getShell(), "Database Query","Which database should I connect to?  Please select the desired database in the navigator on the left then try to execute the query.");
			}
		} catch (PartInitException e) {
			DatabaseLog.logError("Could not open query editor", e);
		}
	}
	
	/**
	 * @see org.eclipse.ui.IWorkbenchPart#setFocus()
	 */
	public void setFocus() {}
	
	/**
	 * Finds the Database from a IDatabaseElement by recursing
	 * up the tree.
	 * 
	 * @param	element	The element to start looking at.
	 * 
	 * @return	The Database this element belongs in, or
	 * 			<code>null</code> if one could not be found.
	 */
	private Database findDatabase(IDatabaseElement element){
		Database database = null;
		if ( element instanceof Database ){
			database = (Database)element;
		}else if (element == null){
			return null;
		}else{
			return findDatabase(element.getParent());
		}
		return database;
	}

} // QueryView