/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.internal.ui;

import java.util.ArrayList;
import java.util.List;

/**
 * Represents a Table in the DataNavigator.
 *
 * @author	mbaumbach
 *
 * @version	0.3.1
 */
public class Table implements IDatabaseElement {
	
	private String name;
	private Database database;
	private List columns;
	
	/**
	 * Creates a new Table.
	 * 
	 * @param	tableName	The name of the table.
	 * @param	database	The database this table belongs in.
	 */
	public Table(String tableName, Database database){
		this.name = tableName;
		this.database = database;
		this.columns = new ArrayList();
	}
	
	/**
	 * Adds a column to this table.
	 * 
	 * @param	column	The column to add.
	 */
	public void addColumn(Column column){
		this.columns.add(column);
	}
	
	/**
	 * Removes a column from this table.
	 * 
	 * @param	column	The column to remove.
	 */
	public void removeColumn(Column column){
		this.columns.remove(column);
	}
	
	/**
	 * @see org.radrails.db.internal.ui.IDatabaseElement#getParent()
	 */
	public IDatabaseElement getParent() {
		return database;
	}

	/**
	 * @see org.radrails.db.internal.ui.IDatabaseElement#getChildren()
	 */
	public Object[] getChildren() {
		return columns.toArray();
	}

	/**
	 * @see org.radrails.db.internal.ui.IDatabaseElement#hasChildren()
	 */
	public boolean hasChildren() {
		return !columns.isEmpty();
	}

	/**
	 * @see org.radrails.db.internal.ui.IDatabaseElement#getName()
	 */
	public String getName() {
		return name;
	}

} // Table