/*******************************************************************************
 * Copyright (c) 2006 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.internal.ui.editors.dbtable;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * @author mkent
 *
 */
public class DbTableContentModel {

	private Collection fChangeListeners;
	
	private Collection fRows;
	
	private Map fDeltas;
	
	public DbTableContentModel() {
		fChangeListeners = new ArrayList();
		fDeltas = new HashMap();
	}
	
	public void addChangeListener(IDbTableChangeListener listener) {
		fChangeListeners.add(listener);
	}
	
	public void removeChangeListener(IDbTableChangeListener listener) {
		fChangeListeners.remove(listener);
	}
	
	public void rowAdded(Row r) {
		fRows.add(r);
		fDeltas.put(r, new RowDelta(r, RowDelta.ADD));
		
		Iterator i = fChangeListeners.iterator();
		while(i.hasNext()) {
			IDbTableChangeListener listener = (IDbTableChangeListener) i.next();
			listener.rowAdded(r);
		}
	}
	
	public void rowRemoved(Row r) {
		fRows.remove(r);
		RowDelta del = (RowDelta) fDeltas.get(r);
		if((del != null) && del.getRow().isNewRow()) {
			fDeltas.remove(r);
		}
		else {
			fDeltas.put(r, new RowDelta(r, RowDelta.REMOVE));
		}
		
		Iterator i = fChangeListeners.iterator();
		while(i.hasNext()) {
			IDbTableChangeListener listener = (IDbTableChangeListener) i.next();
			listener.rowRemoved(r);
		}
	}
	
	public void rowsRemoved(Collection rows) {
		fRows.removeAll(rows);
		Iterator i = rows.iterator();
		while(i.hasNext()) {
			Row r = (Row) i.next();
			RowDelta del = (RowDelta) fDeltas.get(r);
			if((del != null) && del.getRow().isNewRow()) {
				fDeltas.remove(r);
			}
			else {
				fDeltas.put(r, new RowDelta(r, RowDelta.REMOVE));
			}
		}
		
		Iterator j = fChangeListeners.iterator();
		while(j.hasNext()) {
			IDbTableChangeListener listener = (IDbTableChangeListener) j.next();
			listener.rowsRemoved(rows);
		}
	}
	
	public void rowUpdated(Row r) {
		RowDelta del = (RowDelta) fDeltas.get(r);
		if((del != null) && del.getRow().isNewRow()) {
			fDeltas.put(r, new RowDelta(r, RowDelta.ADD));
		}
		else {
			fDeltas.put(r, new RowDelta(r, RowDelta.UPDATE));
		}
		
		Iterator i = fChangeListeners.iterator();
		while(i.hasNext()) {
			IDbTableChangeListener listener = (IDbTableChangeListener) i.next();
			listener.rowUpdated(r);
		}
	}
	
	public void setRows(Collection rows) {
		fRows = rows;
	}
	
	public Collection getRows() {
		return fRows;
	}
	
	public void clearDeltas() {
		fDeltas.clear();
	}
	
	public Collection getDeltas() {
		return fDeltas.values();
	}
}
