/*******************************************************************************
 * Copyright (c) 2006 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.internal.ui.editors.dbtable;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.EditorPart;
import org.radrails.db.core.DatabaseLog;
import org.radrails.db.core.QueryAdministrator;
import org.radrails.db.internal.ui.Database;
import org.radrails.db.internal.ui.Project;

/**
 * @author mkent
 * 
 */
public class DbTableEditor extends EditorPart {

	private TableViewer fTableViewer;

	private Table fTableWidget;

	private Database fDatabase;

	private String fTableName;

	private DbTableContentModel fContent;

	private String[] fColumnTitles;
	
	private String[] fColumnNames;
	
	private Row fBlankRow;

	private boolean fDirty;

	public void doSave(IProgressMonitor monitor) {
		try {
			commitDeltas();
			fetchData();
			fTableViewer.setInput(fContent);
			setDirty(false);
		} catch (Exception e) {
			MessageDialog.openError(getSite().getShell(), "Save failed", e
					.getMessage());
			DatabaseLog.logError("Table editor save failed", e);
		}
	}

	private void commitDeltas() throws Exception {
		Iterator i = fContent.getDeltas().iterator();
		while (i.hasNext()) {
			RowDelta del = (RowDelta) i.next();
			switch (del.getType()) {
			case RowDelta.ADD:
				runQuery(DbTableQueryHelper.insertRow(fTableName, del.getRow()));
				break;
			case RowDelta.REMOVE:
				runQuery(DbTableQueryHelper.deleteRow(fTableName, del.getRow()));
				break;
			case RowDelta.UPDATE:
				runQuery(DbTableQueryHelper.updateRow(fTableName, del.getRow()));
				break;
			}
		}
	}



	private void runQuery(String query) throws Exception {
		Project project = (Project) fDatabase.getParent();
		Connection connection = project.getProjectDatabaseManager()
				.getConnection(fDatabase.getName());
		QueryAdministrator admin = new QueryAdministrator(connection);
		admin.execute(query, 1000);
	}

	public void doSaveAs() {

	}

	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		setSite(site);
		setInput(input);
		DbTableEditorInput dbInput = (DbTableEditorInput) input;
		fDatabase = dbInput.getDatabase();
		fTableName = dbInput.getTableName();
		fDirty = false;
		setPartName(fTableName);
	}

	public boolean isDirty() {
		return fDirty;
	}

	public boolean isSaveAsAllowed() {
		return false;
	}

	public void createPartControl(Composite parent) {
		GridLayout layout = new GridLayout();
		parent.setLayout(layout);
		createTableViewer(parent);
		Composite buttonBarComp = new Composite(parent, SWT.NULL);
		buttonBarComp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		buttonBarComp.setLayout(new GridLayout(2, false));
		createButtonBar(buttonBarComp);

		fetchData();
		configureTableViewer();
		fTableViewer.setInput(fContent);
	}

	private void createTableViewer(Composite parent) {
		int styles = SWT.BORDER | SWT.H_SCROLL | SWT.FULL_SELECTION | SWT.MULTI
				| SWT.VIRTUAL;
		fTableWidget = new Table(parent, styles);
		fTableWidget.setLayoutData(new GridData(GridData.FILL_BOTH));

		fTableViewer = new TableViewer(fTableWidget);
		fTableViewer.setContentProvider(new DbTableContentProvider());
		fTableViewer.setLabelProvider(new DbTableLabelProvider());
		fTableWidget.setLinesVisible(true);
		fTableWidget.setHeaderVisible(true);
	}

	private void configureTableViewer() {
		createTableColumns(fColumnTitles);
		fTableViewer.setColumnProperties(fColumnNames);
		CellEditor[] cellEditors = new CellEditor[fTableWidget.getColumnCount()];
		for (int i = 0; i < fTableWidget.getColumnCount(); i++) {
			cellEditors[i] = new TextCellEditor(fTableWidget);
		}
		fTableViewer.setCellEditors(cellEditors);
		fTableViewer.setCellModifier(new DbTableCellModifier());
	}

	private void createButtonBar(Composite parent) {
		Button addButton = new Button(parent, SWT.PUSH);
		addButton.setText("Add");
		addButton.addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent e) {

			}

			public void widgetSelected(SelectionEvent e) {
				fContent.rowAdded(copyRow(fBlankRow));
			}

		});

		Button removeButton = new Button(parent, SWT.PUSH);
		removeButton.setText("Remove");
		removeButton.addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent e) {

			}

			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection selection = (IStructuredSelection) fTableViewer
						.getSelection();
				if(selection.size() == 1) {
					fContent.rowRemoved((Row) selection.getFirstElement());
				}
				else if(selection.size() > 1) {
					fContent.rowsRemoved(selection.toList());
				}
			}

		});

	}

	private Row copyRow(Row r) {
		// stupid clone method doesn't work
		Row cr = new Row(fContent, true);
		Iterator i = fBlankRow.getFields().iterator();
		while (i.hasNext()) {
			Field f = (Field) i.next();
			Field cf = new Field(cr, f.getColumnName(), f.getOriginalValue(), f
					.isPk(), f.getType());
			cr.addField(cf);
		}
		return cr;
	}

	public void setFocus() {

	}

	private void fetchData() {
		String query = "SELECT * FROM " + fTableName;
		try {
			Project project = (Project) fDatabase.getParent();
			Connection connection = project.getProjectDatabaseManager()
					.getConnection(fDatabase.getName());
			QueryAdministrator admin = new QueryAdministrator(connection);
			// TODO limit to 1000 rows for now
			ResultSet queryResult = admin.executeRaw(query, 1000);

			fContent = new DbTableContentModel();
			Collection rows = convertToRows(admin, queryResult);
			
			// create a blank row for use with the add button
			fBlankRow = new Row(fContent, true);
			
			Collection colTitles = new ArrayList();
			Collection colNames = new ArrayList();
			ResultSet rs = connection.getMetaData().getColumns(null, null, fTableName, null);
			while(rs.next()) {
				String colName = rs.getString(4);
				int type = rs.getInt(5);
				String typeName = rs.getString(6);
				
				colTitles.add(colName + " (" + typeName + ")");
				colNames.add(colName);
				Field fd = new Field(fBlankRow, colName, "<value>", admin.isPrimaryKey(colName, fTableName), type);
				fBlankRow.addField(fd);
			}
			fColumnTitles = (String[]) colTitles.toArray(new String[0]);
			fColumnNames = (String[]) colNames.toArray(new String[0]);

			fContent.setRows(rows);
		} catch (Exception e) {
			MessageDialog.openError(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), "Database Error", e.getMessage());
			DatabaseLog.logError("Could not execute statement", e);
		}
	}

	private Collection convertToRows(QueryAdministrator admin,
			ResultSet result) throws SQLException {
		Collection rows = new ArrayList();

		if(result != null) {
			ResultSetMetaData rsmd = result.getMetaData();
			int cols = rsmd.getColumnCount();
			
			while(result.next()) {
				Row row = new Row(fContent, false);
				
				for(int i = 1; i <= cols; i++) {
					Object val = result.getObject(i);
					String strVal = (val == null) ? "" : val.toString();
					String colName = rsmd.getColumnName(i);
					int type = rsmd.getColumnType(i);
					
					Field f = new Field(row, colName, strVal, admin.isPrimaryKey(colName, fTableName), type);
					row.addField(f);
				}
				rows.add(row);
			}
		}

		return rows;
	}

	private void createTableColumns(String[] cols) {
		for(int i = 0; i < cols.length; i++) {
			String columnName = cols[i];
			TableColumn tableColumn = new TableColumn(fTableWidget, SWT.LEFT);
			tableColumn.setText(columnName);
			tableColumn.setWidth(150);
		}
	}

	private void setDirty(boolean val) {
		fDirty = val;
		firePropertyChange(PROP_DIRTY);
	}

	private class DbTableContentProvider implements IStructuredContentProvider,
			IDbTableChangeListener {

		public Object[] getElements(Object inputElement) {
			return fContent.getRows().toArray();
		}

		public void dispose() {

		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			if (oldInput != null) {
				DbTableContentModel oldContent = (DbTableContentModel) oldInput;
				oldContent.removeChangeListener(this);
			}
			if (newInput != null) {
				DbTableContentModel newContent = (DbTableContentModel) newInput;
				newContent.addChangeListener(this);
			}
		}

		public void rowAdded(Row r) {
			setDirty(true);
			fTableViewer.add(r);
		}

		public void rowRemoved(Row r) {
			setDirty(true);
			fTableWidget.deselectAll();
			fTableViewer.remove(r);
		}
		
		public void rowsRemoved(Collection rows) {
			setDirty(true);
			fTableWidget.deselectAll();
			fTableViewer.remove(rows.toArray());
		}

		public void rowUpdated(Row r) {
			setDirty(true);
			fTableViewer.update(r, null);
		}

	}

}
