/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.internal.ui.editors.sql;

import org.eclipse.jface.text.DocumentCommand;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.source.IOverviewRuler;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.jface.text.source.projection.ProjectionViewer;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.widgets.Composite;
import org.radrails.db.internal.ui.DbUIPlugin;
import org.radrails.db.internal.ui.editors.preferences.IPreferenceConstants;

public class SQLEditorSourceViewer extends ProjectionViewer {

	private boolean isTabReplacing = false;

	private boolean fIgnoreTextConverters = false;

	private TabExpander tabExpander;

	private IPropertyChangeListener propListener = new IPropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent event) {
			if (event.getProperty().equals(
					IPreferenceConstants.SQL_FORMAT_USE_TAB)) {
				isTabReplacing = !DbUIPlugin.getInstance().getPreferenceStore()
						.getBoolean(IPreferenceConstants.SQL_FORMAT_USE_TAB);
			} else if (event.getProperty().equals(
					IPreferenceConstants.SQL_INDENTATION)) {
				int length = DbUIPlugin.getInstance().getPreferenceStore()
						.getInt(IPreferenceConstants.SQL_INDENTATION);
				tabExpander = new TabExpander(length);
			}
		}
	};

	public SQLEditorSourceViewer(Composite composite,
			IVerticalRuler verticalRuler, IOverviewRuler overviewRuler,
			boolean overviewRulerVisible, int styles) {
		super(composite, verticalRuler, overviewRuler, overviewRulerVisible,
				styles);
		initializeTabReplace();
		DbUIPlugin.getInstance().getPreferenceStore()
				.addPropertyChangeListener(propListener);
	}

	public void doOperation(int operation) {
		if (getTextWidget() == null || !redraws()) {
			return;
		}

		switch (operation) {
		case UNDO:
			fIgnoreTextConverters = true;
			break;
		case REDO:
			fIgnoreTextConverters = true;
			break;
		}

		super.doOperation(operation);
	}

	protected void customizeDocumentCommand(DocumentCommand command) {
		super.customizeDocumentCommand(command);
		if (!fIgnoreTextConverters) {
			convertTabs(command, getDocument());
		}
		fIgnoreTextConverters = false;
	}

	void initializeTabReplace() {
		this.isTabReplacing = !DbUIPlugin.getInstance().getPreferenceStore()
				.getBoolean(IPreferenceConstants.SQL_FORMAT_USE_TAB);
		if (this.isTabReplacing) {
			int length = DbUIPlugin.getInstance().getPreferenceStore().getInt(
					IPreferenceConstants.SQL_INDENTATION);
			tabExpander = new TabExpander(length);
		}
	}

	protected void convertTabs(DocumentCommand command, IDocument document) {
		if (!isTabReplacing)
			return;

		if (command.text.equals("\t"))
			tabExpander.expandTab(command, document);
	}

	public boolean isTabReplacing() {
		return isTabReplacing;
	}

	public String getIndentString() {
		return tabExpander.getFullIndent();
	}
}
