/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/
package org.radrails.db.internal.ui.editors.sql.util;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author mkent
 * 
 */
public class IndentCodeFormatter {

	private static final String NEW_LINE = System.getProperty("line.separator");

	private static final String SPACE = " ";

	private static final String TAB = "\t";

	private static IndentCodeFormatter instance;

	private IndentCodeFormatter() {
	}

	public static IndentCodeFormatter getInstance() {
		if (instance == null) {
			instance = new IndentCodeFormatter();
		}
		return instance;
	}

	/**
	 * @param unformatted
	 * @return
	 */
	public String format(String unformatted, int spacesPerTab, boolean useTabs) {
		StringBuffer formatted = new StringBuffer();
		String[] lines = unformatted.split(NEW_LINE);

		StringBuffer sb = new StringBuffer();
		for (int j = 0; j < spacesPerTab; j++) {
			sb.append(SPACE);
		}
		String spacesForTab = sb.toString();

		// Compile a regex to find the indent whitespace
		Pattern indentPat = Pattern.compile("^[\\t|\\s]*");
		for (int i = 0; i < lines.length; i++) {
			Matcher indentMat = indentPat.matcher(lines[i]);

			// If the current line has an indent
			if (indentMat.find()) {
				String indent = indentMat.group();
				int indentEndPos = indentMat.end();

				// Replace all tabs in the indent with spaces
				if(useTabs) {
					indent = indent.replaceAll(spacesForTab, TAB);
				}
				// Replace all spaces in the indent with tabs
				else {
					indent = indent.replaceAll(TAB, spacesForTab);
				}
				lines[i] = indent + lines[i].substring(indentEndPos);
				formatted.append(lines[i]);

				// Append a newline to the end of the line unless it's the
				// last
				if (i != (lines.length - 1)) {
					formatted.append(NEW_LINE);
				}
			}
		}

		return formatted.toString();
	}
}
