/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.internal.ui.editors.sql.util;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.PlatformUI;
import org.radrails.db.core.DatabaseLog;
import org.radrails.db.core.IDatabaseConstants;

/**
 * Handles the SQL execute action for the test database.
 * 
 * @author mbaumbach
 * 
 * @version 0.3.0
 */
public class SQLExecuteTestAction implements IActionDelegate {

	private ISelection selection;

	/**
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	public void run(IAction action) {
		if (selection instanceof StructuredSelection) {
			StructuredSelection sSelection = (StructuredSelection) selection;
			if (sSelection.getFirstElement() instanceof IFile) {
				IFile file = (IFile) sSelection.getFirstElement();
				try {
					SQLExecuteHelper.getInstance().runSQLFile(file,
							IDatabaseConstants.ENV_TEST);
					MessageDialog.openInformation(PlatformUI.getWorkbench()
							.getDisplay().getActiveShell(), "Database Query",
							"The query was executed successfully.");
				} catch (Exception e) {
					MessageDialog.openError(PlatformUI.getWorkbench()
							.getDisplay().getActiveShell(), "Database Error", e
							.getMessage());
					DatabaseLog.logError("Exception running query", e);
				}
			}
		}
	}

	/**
	 * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction,
	 *      org.eclipse.jface.viewers.ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		this.selection = selection;
	}

} // SQLExecuteDevelopmentAction
