/*******************************************************************************
 * Copyright (c) 2006 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.internal.core.launching;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.model.ILaunchConfigurationDelegate;
import org.eclipse.debug.core.model.IProcess;
import org.radrails.rails.internal.core.RailsRuntime;

/**
 * @author matt
 *
 */
public class RailsLaunchConfigDelegate implements
		ILaunchConfigurationDelegate {

	public void launch(ILaunchConfiguration config, String mode,
			ILaunch launch, IProgressMonitor monitor) throws CoreException {

		String workingDirStr = config.getAttribute(IRailsLaunchConfigAttributes.WORKING_DIR, "");
		File workingDir = new File(workingDirStr);
		String programArgsStr = config.getAttribute(IRailsLaunchConfigAttributes.PROGRAM_ARGS, "");
		String[] programArgs = programArgsStr.split("\\s+");
		String procLabel = config.getAttribute(IRailsLaunchConfigAttributes.PROCESS_LABEL, "");
		
		// perhaps this will enable backtracing via RDT?
		Map atts = new HashMap();
        atts.put(IProcess.ATTR_PROCESS_TYPE, "ruby");
        
        String scriptType = config.getAttribute(IRailsLaunchConfigAttributes.SCRIPT_TYPE, "");
        Process proc = null;
        
        if(monitor != null) {
        	monitor.worked(1);
        }
        if(scriptType.equals(IRailsLaunchConfigAttributes.SCRIPT_RUBY)) {
        	proc = RailsRuntime.rubyExec(programArgs, workingDir);
        }
        else if(scriptType.equals(IRailsLaunchConfigAttributes.SCRIPT_RAKE)) {
        	proc = RailsRuntime.rakeExec(programArgs, workingDir);
        }
        else if(scriptType.equals(IRailsLaunchConfigAttributes.SCRIPT_RAILS)) {
        	proc = RailsRuntime.railsExec(programArgs, workingDir);
        }
        if(monitor != null) {
        	monitor.worked(1);
        }
        IProcess iProc = DebugPlugin.newProcess(launch, proc, procLabel, atts);
        String refresh = config.getAttribute(IRailsLaunchConfigAttributes.REQUIRES_REFRESH, "");
        if(refresh.equals("true")) {
        	String projName = config.getAttribute(IRailsLaunchConfigAttributes.PROJECT_NAME, "");
        	iProc.setAttribute(IRailsLaunchConfigAttributes.REQUIRES_REFRESH, "true");
        	iProc.setAttribute(IRailsLaunchConfigAttributes.PROJECT_NAME, projName);
        }
        if(monitor != null) {
        	monitor.worked(1);
        	monitor.done();
        }
	}

}
