/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/
package org.radrails.rails.core.yml;

/**
 * Clients who implement this interface will be notified of content in the YML
 * document. The order of the events mirrors the order of the content in the
 * document.
 * 
 * @author mkent
 * 
 */
public interface YMLContentHandler {

	/**
	 * The beginning of the document.
	 */
	public void startDocument();

	/**
	 * The end of the document.
	 */
	public void endDocument();

	/**
	 * A node header.
	 * 
	 * @param name
	 *            the name of the node
	 */
	public void node(String name);

	/**
	 * An alias definition for a node.
	 * 
	 * @param name
	 *            the name of the alias
	 */
	public void alias(String name);

	/**
	 * A content mapping from a node.
	 * 
	 * @param key
	 *            the mapping key
	 * @param value
	 *            the mapping value
	 */
	public void mapping(String key, String value);
}
