/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.generators;

import java.io.File;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.runtime.CoreException;
import org.radrails.rails.core.RailsLog;

public class ProjectGeneratorsLocator extends GeneratorsLocator implements
		IResourceChangeListener  {

	private static final String GENERATORS_DIRECTORY = "generators";

	private IProject project;

	public ProjectGeneratorsLocator(IProject project) {
		super();
		this.project = project;
	}

	public void locateGenerators() {
		
		String[] generatorLocations = { "lib", "vendor" };
		generators.clear();
		
		for (int i = 0; i < generatorLocations.length; i++) {
			IFolder folder = project.getFolder(generatorLocations[i]
					+ File.separator + GENERATORS_DIRECTORY);
			populateGeneratorsAtPath(folder);
		}
		populateGeneratorsFromPlugins(project.getFolder("vendor"
				+ File.separator + "plugins"));
	}

	private void populateGeneratorsFromPlugins(IFolder pluginsDir) {
		try {
			IResource[] members = pluginsDir.members();

			for (int i = 0; i < members.length; i++) {
				// Search for generators under each plugin's directory
				if (members[i].getType() == IResource.FOLDER) {
					IFolder generatorsFolder = ((IFolder) members[i])
							.getFolder(GENERATORS_DIRECTORY);
					populateGeneratorsAtPath(generatorsFolder);
				}
			}

		} catch (CoreException e) {
			RailsLog.logError("Could not fetch members of "
					+ pluginsDir.getName(), e);
		}
	}

	private void populateGeneratorsAtPath(IFolder folder) {

		try {
			if (folder != null && folder.getName().equals(GENERATORS_DIRECTORY)) {
				IResource[] genFolderMembers = folder.members();
				for (int i = 0; i < genFolderMembers.length; i++) {
					if (genFolderMembers[i].getType() == IResource.FOLDER) {
						IFolder generatorFolder = (IFolder) genFolderMembers[i];
						IFile generatorFile = generatorFolder
								.getFile(generatorFolder.getName()
										+ "_generator.rb");
						if (generatorFile.exists()) {
							Generator gen = new Generator(generatorFile
									.toString());
							generators.add(gen);
						}
					}
				}
			}

		} catch (CoreException e) {
			RailsLog.logError("Could not fetch members of " + folder.getName(),
					e);
		}
	}

	public void resourceChanged(IResourceChangeEvent event) {
		RailsLog.logInfo("resource changed "
				+ event.getDelta().getFullPath().toString(), null);

	}

}
