/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.preference.IPersistentPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;
import org.radrails.rails.internal.ui.autotest.AutotestManager;

/**
 * Plugin class for the rails.ui plugin.
 * 
 * @author mkent
 * 
 */
public class RailsUIPlugin extends AbstractUIPlugin {

	private static RailsUIPlugin instance;

	public final static String RHTML_PARTITIONING = "__rhtml_partitioning";

	private AutotestManager fAutotestManager;

	/**
	 * Default constructor.
	 */
	public RailsUIPlugin() {
		super();
		instance = this;
	}

	/**
	 * Returns the singleton instance of the plugin.
	 * 
	 * @return - the plugin
	 */
	public static RailsUIPlugin getInstance() {
		return instance;
	}

	/**
	 * Returns an image descriptor for the image file at the given plug-in
	 * relative path.
	 * 
	 * @param path
	 *            the path
	 * 
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return AbstractUIPlugin.imageDescriptorFromPlugin(
				"org.radrails.rails.ui", path);
	}

	public void start(BundleContext context) throws Exception {
		super.start(context);
		fAutotestManager = new AutotestManager();
		ResourcesPlugin.getWorkspace().addResourceChangeListener(
				fAutotestManager);
		getPreferenceStore().addPropertyChangeListener(fAutotestManager);
	}

	public void stop(BundleContext context) throws Exception {
		super.stop(context);
		ResourcesPlugin.getWorkspace().removeResourceChangeListener(
				fAutotestManager);
		
		// save pref store - for autotest
		IPersistentPreferenceStore ppStore = (IPersistentPreferenceStore) getPreferenceStore();
		ppStore.save();
	}
	
	public AutotestManager getAutotestManager() {
		return fAutotestManager;
	}

}
