/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.actions;

import java.io.File;

import org.eclipse.core.resources.IProject;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.radrails.rails.internal.core.RailsPlugin;
import org.radrails.rails.internal.ui.IRailsUIConstants;
import org.radrails.rails.internal.ui.RailsUILog;
import org.radrails.rails.internal.ui.terminal.TerminalSessionInput;

/**
 * Toolbar action that opens a terminal session.
 * 
 * @author mkent
 * @deprecated
 */
public class OpenTerminalActionDelegate implements
		IWorkbenchWindowActionDelegate {

	private IWorkbenchWindow window;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#dispose()
	 */
	public void dispose() {
		// Do nothing
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#init(org.eclipse.ui.IWorkbenchWindow)
	 */
	public void init(IWorkbenchWindow window) {
		this.window = window;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	public void run(IAction action) {
		
		IProject project = RailsPlugin.getInstance().getSelectedRailsOrRubyProject();
		
		if ( project != null) {
			File projectRoot = project.getLocation().toFile();

			// Open a new terminal window for the selected project
			try {
				IEditorInput editorInput = new TerminalSessionInput(
						projectRoot, null);
				PlatformUI.getWorkbench().getActiveWorkbenchWindow()
						.getActivePage().openEditor(editorInput,
								"org.radrails.rails.ui.TerminalSession");
			} catch (PartInitException e) {
				RailsUILog.logError("Error opening terminal session", e);
			}
		} else {
			MessageDialog.openError(window.getShell(),
					"Error opening terminal",
					IRailsUIConstants.SELECT_RAILS_PROJECT_STRING);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction,
	 *      org.eclipse.jface.viewers.ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		// Do nothing
	}

}
