/*******************************************************************************
 * Copyright (c) 2006 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.actions;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.progress.UIJob;
import org.radrails.rails.internal.core.RailsPlugin;
import org.radrails.rails.internal.ui.IRailsUIConstants;
import org.radrails.rails.ui.test.TestLauncher;

/**
 * Action to run all tests (unit, functional and integration) for a Rails
 * project.
 * 
 * @author Kyle
 * @deprecated tests now launched from RunTestsPulldownDelegate
 * 
 */
public class RunTestsActionDelegate implements IWorkbenchWindowActionDelegate {

	private IWorkbenchWindow window;

	public void dispose() {

	}

	public void init(IWorkbenchWindow window) {
		this.window = window;
	}

	public void run(IAction action) {
		
//		 Create a new job
		Job j = new UIJob("All tests") {
			public IStatus runInUIThread(IProgressMonitor monitor) {
				monitor.beginTask("Running all tests", 3);
				IProject project = RailsPlugin.getInstance()
					.getSelectedRailsOrRubyProject();
				monitor.worked(1);
				if (project != null) {
					TestLauncher t = new TestLauncher();
					monitor.worked(1);
					t.goLaunch(project, "run", "run_tests.rb");
				} else {
					MessageDialog.openError(window.getShell(), "Error running tests",
							IRailsUIConstants.SELECT_RAILS_PROJECT_STRING);
				}

				// Wait for the plugin install to complete and refresh the
				// project
				monitor.worked(1);
				monitor.done();
				return Status.OK_STATUS;
			}
		};
		j.setUser(true);
		j.schedule();
	}

	public void selectionChanged(IAction action, ISelection selection) {

	}

}
