/*******************************************************************************
 * Copyright (c) 2006 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.actions.mvc;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.FileEditorInput;
import org.radrails.rails.internal.ui.RailsUILog;

/**
 * Keybind to switch between models/controllers and their tests
 * 
 * @author Kyle
 */
public class TestAction implements IWorkbenchWindowActionDelegate {

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#dispose()
	 */
	public void dispose() {
		// Do nothing
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#init(org.eclipse.ui.IWorkbenchWindow)
	 */
	public void init(IWorkbenchWindow window) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	public void run(IAction action) {
		IEditorPart part = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				.getActivePage().getActiveEditor();
		IFile original = ((FileEditorInput) part.getEditorInput()).getFile();
		IProject project = original.getProject();
		String filename = original.getProjectRelativePath().toString();
		
		if (attemptToOpenMvcRelatedFile(project, filename, "app/controllers", "", "test/functional", "_test")) return;
		if (attemptToOpenMvcRelatedFile(project, filename, "app/models", "", "test/unit", "_test")) return;
		if (attemptToOpenMvcRelatedFile(project, filename, "test/functional", "_test", "app/controllers", "")) return;
		if (attemptToOpenMvcRelatedFile(project, filename, "test/unit", "_test", "app/models", "")) return;
	}

	private boolean attemptToOpenMvcRelatedFile(IProject project, String filename, String srcDir, String srcSuffix, String destDir, String destSuffix) {
		Matcher m = Pattern.compile("^((?:.+?/)?)"+srcDir+"/(.+)"+srcSuffix+"\\.rb$").matcher(filename);
		if (!m.matches())
			return false;
		
		IPath thePath = project.getProjectRelativePath().append(project.getName() + (m.group(1).length()==0 ? "" : "/") + m.group(1) + "/" + destDir + "/" + m.group(2) + destSuffix + ".rb");
		IFile fileHandle = ResourcesPlugin.getWorkspace().getRoot().getFile(thePath);
		IEditorInput editorInput = new FileEditorInput(fileHandle);
		try{		
			if (fileHandle.exists()) {
				PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().openEditor(editorInput,"org.rubypeople.rdt.ui.EditorRubyFile");
				return true;
			}
		} catch (PartInitException e) {
			RailsUILog.logError("Error creating editor", e);
		}
		return false;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction,
	 *      org.eclipse.jface.viewers.ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		// Do nothing
	}

}
