/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.actions.mvc;

import java.io.ByteArrayInputStream;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.FileEditorInput;
import org.radrails.rails.internal.ui.RailsUILog;
import org.rubypeople.rdt.core.IParent;
import org.rubypeople.rdt.core.IRubyElement;
import org.rubypeople.rdt.core.IRubyScript;
import org.rubypeople.rdt.core.RubyModelException;
import org.rubypeople.rdt.internal.core.RubyMethod;
import org.rubypeople.rdt.internal.ui.rubyeditor.RubyDocumentProvider;
import org.rubypeople.rdt.internal.ui.rubyeditor.RubyEditor;

/**
 * Toggle action that switches between View and Mailer/Controller files.
 * 
 * @author Kyle
 * 
 */
public class ViewAction implements IWorkbenchWindowActionDelegate {

	private IWorkbenchWindow window;

	public static final String[] VIEW_TYPES = { "rhtml", "rjs", "rxml" };

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#dispose()
	 */
	public void dispose() {
		// Do nothing
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#init(org.eclipse.ui.IWorkbenchWindow)
	 */
	public void init(IWorkbenchWindow window) {
		this.window = window;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	public void run(IAction action) {
		IEditorPart part = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				.getActivePage().getActiveEditor();
		IFile original = ((FileEditorInput) part.getEditorInput()).getFile();
		IProject project = original.getProject();
		String path = original.getProjectRelativePath().toString();
		try {
			if (attemptToOpenMvcView(project, path, part, "app/controllers", "_controller")) return;
			if (attemptToOpenMvcView(project, path, part, "app/models", "")) return;
			if (attemptToOpenMvcModelOrController(project, path, original)) return;

		} catch (RubyModelException e) {
			RailsUILog.logError("Error accessing ruby model", e);
		} catch (PartInitException e) {
			RailsUILog.logError("Error creating editor", e);
		} catch (CoreException e) {
			RailsUILog.logError("Error running action", e);
		}
	}

	private boolean attemptToOpenMvcView(IProject project, String path, IEditorPart part, String srcDir, String srcSuffix) throws RubyModelException, CoreException, PartInitException {
		Matcher m = Pattern.compile("^((?:.+?/)?)"+srcDir+"/(.+)"+srcSuffix+"\\.rb$").matcher(path);
		if (!m.matches())
			return false;

		String appRootDir = (m.group(1).length()==0 ? "" : "/") + m.group(1);
		String viewName = m.group(2);

		RubyEditor editor = (RubyEditor) part;
		RubyDocumentProvider provider = (RubyDocumentProvider) editor.getDocumentProvider();
		IRubyScript script = (IRubyScript) provider.getWorkingCopy(editor.getEditorInput());
		TextSelection ts = (TextSelection) editor.getSelectionProvider().getSelection();
		String actionName = findAction(script, ts.getOffset());
		if (actionName != null) {
			IFile fileHandle = findViewFile(project, appRootDir, viewName, actionName);
			IEditorInput editorInput = new FileEditorInput(fileHandle);
			if (!fileHandle.exists()) {
				// new dialog
				String filename = ViewSelectionDialog.openConfirm(window.getShell(), actionName);

				if (filename != null) {
					IPath filePath = project.getProjectRelativePath().append(project.getName() + appRootDir + "/app/views/" + viewName + "/" + filename);
					IFile newFileHandle = ResourcesPlugin.getWorkspace().getRoot().getFile(filePath);
					newFileHandle.create(new ByteArrayInputStream(("").getBytes()), false /* force */, null);
					IEditorInput newEditorInput = new FileEditorInput(newFileHandle);
					if (newFileHandle.getFileExtension().equals("rhtml")) {
						PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().openEditor(newEditorInput,"org.radrails.rails.ui.editors.RHTML");
					} else {
						PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().openEditor(newEditorInput,"org.rubypeople.rdt.ui.EditorRubyFile");
					}
				}
			} else {
				if (fileHandle.getFileExtension().equals("rhtml")) {
					PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().openEditor(editorInput,"org.radrails.rails.ui.editors.RHTML");
				} else {
					PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().openEditor(editorInput,"org.rubypeople.rdt.ui.EditorRubyFile");
				}
			}
		}
		return true;
	}

	private boolean attemptToOpenMvcModelOrController(IProject project, String path, IFile original) throws PartInitException {
		String appRootDir = "";
		Matcher m = Pattern.compile("^(.+?/)(app/views/.+)$").matcher(path);
		if (m.matches()) {
			appRootDir = "/" + m.group(1);
			path = m.group(2);
		}
		
		if (path.startsWith("app/views")) {
			IPath tPath = original.getProjectRelativePath();
			String[] segments = tPath.segments();

			String patternStr = "app/views/(\\S+?)/" + segments[segments.length - 1];
			Pattern pattern = Pattern.compile(patternStr);
			Matcher matcher = pattern.matcher("");

			// Set the input
			matcher.reset(path);

			// Get tagname and contents of tag
			matcher.find(); // true
			String controllername = matcher.group(1);

			IPath thepath = project.getProjectRelativePath().append(project.getName() + appRootDir + "/app/controllers/" + controllername + "_controller.rb");
			IFile fileHandle = ResourcesPlugin.getWorkspace().getRoot().getFile(thepath);
			IPath mailerpath = project.getProjectRelativePath().append(project.getName() + appRootDir + "/app/models/" + controllername + ".rb");
			IFile fileHandleMailer = ResourcesPlugin.getWorkspace().getRoot().getFile(mailerpath);

			if (fileHandle.exists()) {
				IEditorInput editorInput = new FileEditorInput(fileHandle);
				PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().openEditor(editorInput,"org.rubypeople.rdt.ui.EditorRubyFile");
				return true;
			} else if (fileHandleMailer.exists()) {
				IEditorInput editorInput = new FileEditorInput(fileHandleMailer);
				PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().openEditor(editorInput,"org.rubypeople.rdt.ui.EditorRubyFile");
				return true;
			}
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction,
	 *      org.eclipse.jface.viewers.ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		// Do nothing
	}

	private IFile findViewFile(IProject project, String appRootDir, String viewName, String action) {
		IFile fileHandle = null;
		for (int i = 0; i < VIEW_TYPES.length; i++) {
			IPath thePath = project.getProjectRelativePath().append(
					project.getName() + appRootDir + "/app/views/" + viewName + "/"
							+ action + "." + VIEW_TYPES[i]);
			fileHandle = ResourcesPlugin.getWorkspace().getRoot().getFile(
					thePath);
			if (fileHandle.exists()) {
				break;
			}
		}
		return fileHandle;
	}

	private boolean isRubyMethod(IRubyElement e) {
		boolean retVal = false;
		if (e instanceof RubyMethod) {
			retVal = true;
		}
		return retVal;
	}

	private IRubyElement findRubyMethod(IRubyElement e) {
		if (isRubyMethod(e)) {
			return e;
		} else if (e == null) {
			return null;
		} else {
			return findRubyMethod(e.getParent());
		}
	}

	public String findAction(IRubyScript script, int offset)
			throws RubyModelException {
		String retVal = null;
		IRubyElement method = findRubyMethod(script.getElementAt(offset));
		if (method != null) {
			retVal = method.getElementName();
		} else {
			IParent parent = ((IParent) script.getChildren()[0]);
			if (parent.hasChildren()) {
				IRubyElement[] child = parent.getChildren();
				for (int j = 0; j < child.length; j++) {
					RubyMethod m = (RubyMethod) child[j];
					int methodStart = m.getSourceRange().getOffset();
					int methodEnd = (m.getSourceRange().getOffset() + m
							.getSourceRange().getLength());
					if ((offset >= methodStart) && (offset <= methodEnd)) {
						retVal = m.getElementName();
						break;
					}
				}
			}
		}
		return retVal;
	}

}
