/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.browser;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.browser.CloseWindowListener;
import org.eclipse.swt.browser.LocationAdapter;
import org.eclipse.swt.browser.LocationEvent;
import org.eclipse.swt.browser.OpenWindowListener;
import org.eclipse.swt.browser.TitleEvent;
import org.eclipse.swt.browser.TitleListener;
import org.eclipse.swt.browser.WindowEvent;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.radrails.rails.internal.ui.RailsUIPlugin;

/**
 * Internal web browser. This class is an <code>EditorPart</code> only so it
 * will show up in the editor view area. No actual editing can be done in the
 * browser.
 * 
 * @author Kyle
 * 
 */
public class BrowserEditor extends EditorPart {

	private Browser browser;

	private Text location;

	private Button backButton;

	private Button forwardButton;
	
	private Button refreshButton;

	/**
	 * @see org.eclipse.ui.ISaveablePart#doSave(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void doSave(IProgressMonitor monitor) {
		// Do nothing
	}

	/**
	 * @see org.eclipse.ui.ISaveablePart#doSaveAs()
	 */
	public void doSaveAs() {
		// Do nothing
	}

	/**
	 * @see org.eclipse.ui.IEditorPart#init(org.eclipse.ui.IEditorSite,
	 *      org.eclipse.ui.IEditorInput)
	 */
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		setSite(site);
		setInput(input);
	}

	/**
	 * @see org.eclipse.ui.ISaveablePart#isDirty()
	 */
	public boolean isDirty() {
		// Do nothing
		return false;
	}

	/**
	 * @see org.eclipse.ui.ISaveablePart#isSaveAsAllowed()
	 */
	public boolean isSaveAsAllowed() {
		// Do nothing
		return false;
	}

	/**
	 * @see org.eclipse.ui.IWorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 5;
		parent.setLayout(gridLayout);
		createBrowser(parent);
		BrowserInput input = (BrowserInput) getEditorInput();
		browser.setUrl(input.getURL());
	}

	/**
	 * Create all widgets and build the browser.
	 * 
	 * @param parent
	 *            the parent composite
	 * @return a <code>Browser</code> widget
	 */
	private Browser createBrowser(Composite parent) {

		backButton = new Button(parent, SWT.PUSH);
		ImageDescriptor backImage = AbstractUIPlugin.imageDescriptorFromPlugin(
				RailsUIPlugin.getInstance().getBundle().getSymbolicName(),
				"icons/nav_backward.gif");
		backButton.setImage(backImage.createImage());
		backButton.addSelectionListener(new SelectionListener() {

			public void widgetSelected(SelectionEvent e) {
				browser.back();
			}

			public void widgetDefaultSelected(SelectionEvent e) {

			}

		});
		backButton.setEnabled(false);

		forwardButton = new Button(parent, SWT.PUSH);
		ImageDescriptor forwardImage = AbstractUIPlugin
				.imageDescriptorFromPlugin(RailsUIPlugin.getInstance()
						.getBundle().getSymbolicName(), "icons/nav_forward.gif");
		forwardButton.setImage(forwardImage.createImage());
		forwardButton.addSelectionListener(new SelectionListener() {

			public void widgetSelected(SelectionEvent e) {
				browser.forward();
			}

			public void widgetDefaultSelected(SelectionEvent e) {

			}

		});
		forwardButton.setEnabled(false);
		
		refreshButton = new Button(parent, SWT.PUSH);
		ImageDescriptor refreshImage = AbstractUIPlugin
				.imageDescriptorFromPlugin(RailsUIPlugin.getInstance()
						.getBundle().getSymbolicName(), "icons/nav_refresh.gif");
		refreshButton.setImage(refreshImage.createImage());
		refreshButton.addSelectionListener(new SelectionListener() {

			public void widgetSelected(SelectionEvent e) {
				browser.refresh();
			}

			public void widgetDefaultSelected(SelectionEvent e) {

			}

		});
		refreshButton.setEnabled(true);

		location = new Text(parent, SWT.BORDER);
		location.addKeyListener(new KeyListener() {

			public void keyPressed(KeyEvent e) {
				// If Enter was pressed
				if (e.keyCode == 13) {
					go(location.getText());
				}
			}

			public void keyReleased(KeyEvent e) {
				// Do nothing
			}
		});
		GridData data = new GridData();
		data = new GridData();
		data.horizontalAlignment = GridData.FILL;
		data.grabExcessHorizontalSpace = true;
		location.setLayoutData(data);

		Button goButton = new Button(parent, SWT.PUSH);
		ImageDescriptor goImage = AbstractUIPlugin
		.imageDescriptorFromPlugin(RailsUIPlugin.getInstance()
				.getBundle().getSymbolicName(), "icons/nav_go.gif");
		goButton.setImage(goImage.createImage());
		goButton.addSelectionListener(new SelectionListener() {

			public void widgetSelected(SelectionEvent e) {
				go(location.getText());
			}

			public void widgetDefaultSelected(SelectionEvent e) {
				// Do nothing
			}
		});

		browser = new Browser(parent, SWT.NONE);
		data = new GridData();
		data.horizontalAlignment = GridData.FILL;
		data.verticalAlignment = GridData.FILL;
		data.horizontalSpan = 5;
		data.grabExcessHorizontalSpace = true;
		data.grabExcessVerticalSpace = true;
		browser.setLayoutData(data);

		browser.addLocationListener(new LocationAdapter() {
			public void changed(LocationEvent event) {
				if (event.top)
					location.setText(event.location);
				backButton.setEnabled(browser.isBackEnabled());
				forwardButton.setEnabled(browser.isForwardEnabled());
			}
		});
		browser.addTitleListener(new TitleListener() {
			public void changed(TitleEvent event) {
				setPartName(event.title);
			}
		});
		browser.addOpenWindowListener(new OpenWindowListener() {
			public void open(WindowEvent event) {
				openWindow(event);
			}
		});
		// TODO: should handle VisibilityWindowListener.show and .hide events
		browser.addCloseWindowListener(new CloseWindowListener() {
			public void close(WindowEvent event) {
			}
		});
		location.addSelectionListener(new SelectionAdapter() {
			public void widgetDefaultSelected(SelectionEvent e) {
				browser.setUrl(location.getText());
			}
		});

		return browser;

	}

	/**
	 * @see org.eclipse.ui.IWorkbenchPart#setFocus()
	 */
	public void setFocus() {
		if (browser != null && !browser.isDisposed()) {
			browser.setFocus();
		}
	}

	/**
	 * Opens a new browser window.
	 * 
	 * @param event
	 *            the open window event
	 */
	private void openWindow(WindowEvent event) {
			if (event.location != null)
				browser.setLocation(event.location);
			event.browser = browser;
	}

	/**
	 * Helper method to encapsulate the process of navigating to a URL.
	 * 
	 * @param url
	 *            the URL to go to
	 */
	private void go(String url) {
		browser.setUrl(url);
	}

}
