/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.graphics.Point;
import org.radrails.rails.internal.ui.RailsUILog;

/**
 * Closer for Ruby expression tags, e.g. <% %> and <%= %>.
 * 
 * @author mkent
 * 
 */
public class ExpressionCloser extends AbstractCloser {

	/**
	 * Constructor.
	 * 
	 * @param sourceViewer
	 *            the <code>SourceViewer</code> that the closer operates on
	 */
	public ExpressionCloser(SourceViewer sourceViewer) {
		super(sourceViewer);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.custom.VerifyKeyListener#verifyKey(org.eclipse.swt.events.VerifyEvent)
	 */
	public void verifyKey(VerifyEvent event) {
		if (!fEnabled) {
			return;
		}

		// Don't auto-close inside Ruby comments
		if(isCurrentLineRubyComment()) {
			return;
		}
		
		// Don't auto-close inside string literals
		if(isCursorInsideString()) {
			return;
		}
		
		IDocument document = fSourceViewer.getDocument();
		Point selection = fSourceViewer.getSelectedRange();
		int cursorOffset = selection.x;
		int selectionLength = selection.y;

		// If character is %, this is an open Ruby expression tag, complete it
		if (event.character == '%') {
			try {
				char prevChar = document.getChar(cursorOffset - 1);

				if (prevChar == '<') {
					StringBuffer sb = new StringBuffer();
					sb.append(event.character);
					sb.append('%');
					sb.append('>');

					document.replace(cursorOffset, selectionLength, sb
							.toString());

					event.doit = false;
					moveCursor(1);
				}

			} catch (BadLocationException e) {
				RailsUILog.logError("Bad location", e);
			}
		}
	}

}
