/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.ITextViewer;

/**
 * Used to scan and detect for Ruby keywords
 * 
 * @author mbaumbach
 * 
 * @version 0.4
 */
public class WordPartDetector {
	String wordPart = "";

	int docOffset;

	/**
	 * Method WordPartDetector.
	 * 
	 * @param viewer
	 *            is a text viewer
	 * @param documentOffset
	 *            into the SQL document
	 */
	public WordPartDetector(ITextViewer viewer, int documentOffset) {
		docOffset = documentOffset - 1;
		try {
			while (((docOffset) >= viewer.getTopIndexStartOffset())
					&& (Character.isLetterOrDigit(viewer.getDocument().getChar(
							docOffset)) || viewer.getDocument().getChar(docOffset) == '_')) {
				docOffset--;
			}
			// we've been one step too far : increase the offset
			docOffset++;
			wordPart = viewer.getDocument().get(docOffset,
					documentOffset - docOffset);
		} catch (BadLocationException e) {
			// do nothing
		}
	}

	/**
	 * Method getString.
	 * 
	 * @return String
	 */
	public String getString() {
		return wordPart;
	}

	/**
	 * Get the document offset.
	 * 
	 * @return	The document offset.
	 */
	public int getOffset() {
		return docOffset;
	}

} // WordPartDetector