/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.css;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.MultiLineRule;
import org.eclipse.jface.text.rules.WhitespaceRule;
import org.eclipse.jface.text.rules.WordRule;
import org.eclipse.ui.editors.text.EditorsUI;
import org.eclipse.ui.texteditor.AbstractTextEditor;
import org.eclipse.ui.texteditor.ChainedPreferenceStore;
import org.radrails.rails.internal.ui.RailsUIPlugin;
import org.radrails.rails.internal.ui.editors.AbstractManagedRuleBasedScanner;

/**
 * Scans the CSS code for matching rules and highlights correctly matched rules.
 * 
 * @author	mbaumbach
 * 
 * @version	0.4.1
 */
public class CSSCodeScanner extends AbstractManagedRuleBasedScanner {

	private static final String[] COLOR_KEYS = {ICSSEditorColorConstants.COMMENT, ICSSEditorColorConstants.PROPERTY,
		AbstractTextEditor.PREFERENCE_COLOR_FOREGROUND, ICSSEditorColorConstants.UNIT, ICSSEditorColorConstants.VALUE};
	
	/**
	 * Private constructor since this is a singleton. Use the
	 * getInstance() method.
	 */
	public CSSCodeScanner() {
		super();
		// Create color tokens
		IToken cssProperty = getToken(ICSSEditorColorConstants.PROPERTY);
		IToken cssValue = getToken(ICSSEditorColorConstants.VALUE);
		IToken cssUnit = getToken(ICSSEditorColorConstants.UNIT);
		IToken cssComments = getToken(ICSSEditorColorConstants.COMMENT);
		IToken def = getToken(AbstractTextEditor.PREFERENCE_COLOR_FOREGROUND);

		List rules = new ArrayList();
		
		// CSS property rule
		WordRule propertyRule = new WordRule(new CSSWordDetector(), def);
		String[] cssProperties = IHTMLCSSKeywordConstants.CSS_PROPERTIES;
		for (int i = 0; i < cssProperties.length; i++)
			propertyRule.addWord(cssProperties[i], cssProperty);
		rules.add(propertyRule);
		
		// Value rule
		rules.add(new MultiLineRule(":", ";", cssValue, (char) 0, false));
		
		// CSS unit rules
		WordRule unitRule = new WordRule(new CSSWordDetector(), def);
		String[] cssUnits = IHTMLCSSKeywordConstants.CSS_UNITS;
		for (int i = 0; i < cssUnits.length; i++)
			propertyRule.addWord(cssUnits[i], cssUnit);
		rules.add(unitRule);

		// Whitespace rule
		rules.add(new WhitespaceRule(new HTMLCSSWhitespaceDetector()));
		
		// Multiline comments
		rules.add(new MultiLineRule("/*", "*/", cssComments, (char) 0, true));
		
		// Add the rules to the scanner
		IRule[] result = new IRule[rules.size()];
		rules.toArray(result);
		setRules(result);
	}

	protected String[] getColorKeys() {
		return COLOR_KEYS;
	}

	protected IPreferenceStore getPreferenceStore() {
		IPreferenceStore[] prefStores = new IPreferenceStore[2];
		prefStores[0] = RailsUIPlugin.getInstance().getPreferenceStore();
		prefStores[1] = EditorsUI.getPreferenceStore();
		return new ChainedPreferenceStore(prefStores);
	}

} // CSSCodeScanner
