/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.css;

import java.util.ResourceBundle;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.texteditor.ITextEditor;
import org.eclipse.ui.texteditor.TextEditorAction;
import org.radrails.rails.internal.ui.RailsUILog;
import org.radrails.rails.internal.ui.RailsUIPlugin;
import org.radrails.rails.internal.ui.editors.IndentCodeFormatter;
import org.radrails.rails.internal.ui.preferences.IRailsPreferenceConstants;

/**
 * Action for formatting the editor.
 * 
 * @author	mbaumbach
 *
 * @version	0.5.3
 */
public class CSSFormatAction extends TextEditorAction {

	public CSSFormatAction(ResourceBundle bundle, String prefix, ITextEditor editor) {
		super(bundle, prefix, editor);
	}

	public void run() {
		IDocument doc = this.getTextEditor().getDocumentProvider().getDocument(this.getTextEditor().getEditorInput());

		try {
			ISelection selection = this.getTextEditor().getSelectionProvider().getSelection();
			if (selection instanceof TextSelection) {
				TextSelection textSelection = (TextSelection) selection;
				String text = textSelection.getText();
				if (text == null || text.length() == 0) {
					String allFormatted = IndentCodeFormatter.getInstance().format(doc.get(),
							Integer.parseInt(RailsUIPlugin.getInstance().getPreferenceStore().getString(IRailsPreferenceConstants.CSS_INDENTATION)),
							RailsUIPlugin.getInstance().getPreferenceStore().getBoolean(IRailsPreferenceConstants.CSS_FORMAT_USE_TAB));
					CSSEditor editor = (CSSEditor) this.getTextEditor();
					CSSEditor.CaretPosition cursorPos = editor.getCaretPosition();
					doc.set(allFormatted);
					editor.setCaretPosition(cursorPos);
				} else {
					// format always complete lines, otherwise the indentation
					// of the first line is lost
					int startPos = doc.getLineOffset(textSelection.getStartLine());
					int endLine = textSelection.getEndLine();
					int endPos = doc.getLineOffset(endLine) + doc.getLineLength(endLine);

					String formatted = IndentCodeFormatter.getInstance().format(doc.get(startPos, endPos - startPos),
							Integer.parseInt(RailsUIPlugin.getInstance().getPreferenceStore().getString(IRailsPreferenceConstants.CSS_INDENTATION)),
							RailsUIPlugin.getInstance().getPreferenceStore().getBoolean(IRailsPreferenceConstants.CSS_FORMAT_USE_TAB));
					doc.replace(startPos, endPos - startPos, formatted);
				}
			}

		} catch (BadLocationException e) {
			RailsUILog.logError(e.getMessage(), e);
		}

		super.run();
	}

}