/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.css;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.text.rules.IPredicateRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.MultiLineRule;
import org.eclipse.jface.text.rules.RuleBasedPartitionScanner;
import org.eclipse.jface.text.rules.Token;

/**
 * Partitions a CSS document into selector and default sections.
 * 
 * @author mbaumbach
 * 
 * @version 0.4.1
 */
public class CSSPartitionScanner extends RuleBasedPartitionScanner {

	private static CSSPartitionScanner instance;

	public final static String CSS_PROPERTIES = "__css_properties";

	public final static String CSS_COMMENTS = "__css_comments";

	public final static String[] CSS_PARTITION_TYPES = { CSS_PROPERTIES,
			CSS_COMMENTS };

	/**
	 * @return the singleton instance of the scanner
	 */
	public static CSSPartitionScanner getInstance() {
		if (instance == null) {
			instance = new CSSPartitionScanner();
		}
		return instance;
	}

	/**
	 * Constructor.
	 */
	private CSSPartitionScanner() {
		super();

		IToken cssProperties = new Token(CSS_PROPERTIES);
		IToken cssComments = new Token(CSS_COMMENTS);

		List rules = new ArrayList();
		rules.add(new MultiLineRule("{", "}", cssProperties, (char) 0, true));
		rules.add(new MultiLineRule("/*", "*/", cssComments, (char) 0, true));

		IPredicateRule[] result = new IPredicateRule[rules.size()];
		rules.toArray(result);
		setPredicateRules(result);
	}

} // CSSPartitionScanner
