/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.css;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.WhitespaceRule;
import org.eclipse.jface.text.rules.WordRule;
import org.eclipse.ui.editors.text.EditorsUI;
import org.eclipse.ui.texteditor.AbstractTextEditor;
import org.eclipse.ui.texteditor.ChainedPreferenceStore;
import org.radrails.rails.internal.ui.RailsUIPlugin;
import org.radrails.rails.internal.ui.editors.AbstractManagedRuleBasedScanner;

/**
 * CSS scanner for the selectors and CSS pseudo classes.
 * 
 * @author	mbaumbach
 *
 * @version	0.4.1
 */
public class CSSSelectorScanner extends AbstractManagedRuleBasedScanner {
	
	private static final String[] COLOR_KEYS = {ICSSEditorColorConstants.SELECTOR, AbstractTextEditor.PREFERENCE_COLOR_FOREGROUND};
	
	public CSSSelectorScanner() {
		super();
		// Create color tokens
		IToken cssSelector = getToken(ICSSEditorColorConstants.SELECTOR);
		IToken def = getToken(AbstractTextEditor.PREFERENCE_COLOR_FOREGROUND);

		List rules = new ArrayList();

		// Whitespace rule
		rules.add(new WhitespaceRule(new HTMLCSSWhitespaceDetector()));
		
		// Selector word rule
		WordRule selectorRule = new WordRule(new CSSWordDetector(), def);
		String[][] cssSelectors = IHTMLCSSKeywordConstants.HTML_KEYWORDS;
		for (int x = 0; x < cssSelectors.length; x++) {
			for (int i = 0; i < cssSelectors[x].length; i++) {
				selectorRule.addWord(cssSelectors[x][i], cssSelector);
			}
		}
		rules.add(selectorRule);
		
		// Add the rules to the scanner
		IRule[] result = new IRule[rules.size()];
		rules.toArray(result);
		setRules(result);
	}

	protected String[] getColorKeys() {
		return COLOR_KEYS;
	}

	protected IPreferenceStore getPreferenceStore() {
		IPreferenceStore[] prefStores = new IPreferenceStore[2];
		prefStores[0] = RailsUIPlugin.getInstance().getPreferenceStore();
		prefStores[1] = EditorsUI.getPreferenceStore();
		return new ChainedPreferenceStore(prefStores);
	}

} // CSSSelectorScanner