/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.js;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.WordRule;
import org.eclipse.ui.editors.text.EditorsUI;
import org.eclipse.ui.texteditor.AbstractTextEditor;
import org.eclipse.ui.texteditor.ChainedPreferenceStore;
import org.radrails.rails.internal.ui.RailsUIPlugin;
import org.radrails.rails.internal.ui.editors.AbstractManagedRuleBasedScanner;

public class JavascriptCodeScanner extends AbstractManagedRuleBasedScanner {

	private static final String[] COLOR_KEYS = {IJavascriptEditorColorConstants.EVENT,
		IJavascriptEditorColorConstants.KEYWORD, AbstractTextEditor.PREFERENCE_COLOR_FOREGROUND};
	
	public JavascriptCodeScanner() {
		super();

		// Create color tokens
		IToken event = getToken(IJavascriptEditorColorConstants.EVENT);
		IToken keyword = getToken(IJavascriptEditorColorConstants.KEYWORD);
		IToken def = getToken(AbstractTextEditor.PREFERENCE_COLOR_FOREGROUND);

		setDefaultReturnToken(def);

		List rules = new ArrayList();

		WordRule keywordRule = new WordRule(new JavascriptWordDetector(), def);
		for (int i = 0; i < IJavascriptConstants.JS_EVENTS.length; i++) {
			keywordRule.addWord(IJavascriptConstants.JS_EVENTS[i], event);
		}
		for (int i = 0; i < IJavascriptConstants.JS_KEYWORDS.length; i++) {
			keywordRule.addWord(IJavascriptConstants.JS_KEYWORDS[i], keyword);
		}
		rules.add(keywordRule);

		IRule[] result = new IRule[rules.size()];
		rules.toArray(result);
		setRules(result);
	}

	protected String[] getColorKeys() {
		return COLOR_KEYS;
	}
	
	protected IPreferenceStore getPreferenceStore() {
		IPreferenceStore[] prefStores = new IPreferenceStore[2];
		prefStores[0] = RailsUIPlugin.getInstance().getPreferenceStore();
		prefStores[1] = EditorsUI.getPreferenceStore();
		return new ChainedPreferenceStore(prefStores);
	}
}
