/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.js;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextViewerExtension;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.editors.text.EditorsUI;
import org.eclipse.ui.editors.text.TextEditor;
import org.eclipse.ui.texteditor.ChainedPreferenceStore;
import org.radrails.rails.internal.ui.RailsUIPlugin;
import org.radrails.rails.internal.ui.editors.BracketCloser;
import org.radrails.rails.internal.ui.editors.EditorMessages;
import org.radrails.rails.internal.ui.editors.IManagedSourceViewerConfiguration;
import org.radrails.rails.internal.ui.editors.QuoteCloser;

public class JavascriptEditor extends TextEditor {

	public static final String JAVASCRIPT_PARTITIONING = "__javascript_partitioning";

	protected void initializeEditor() {
		super.initializeEditor();

		// Add the rails ui plugin preferences to the editor
		IPreferenceStore pluginStore = RailsUIPlugin.getInstance()
				.getPreferenceStore();
		IPreferenceStore eclipseUIStore = EditorsUI.getPreferenceStore();
		setPreferenceStore(new ChainedPreferenceStore(new IPreferenceStore[] {
				pluginStore, eclipseUIStore }));

		setSourceViewerConfiguration(new JavascriptSourceViewerConfiguration());
	}

	protected void createActions() {
		super.createActions();

		IAction a = new JavascriptFormatAction(EditorMessages
				.getResourceBundle(), "Format.", this);
		a.setActionDefinitionId("org.radrails.rails.ui.editors.format");
		setAction("Format", a);
	}

	public void editorContextMenuAboutToShow(IMenuManager menu) {
		super.editorContextMenuAboutToShow(menu);
		addAction(menu, "Format");
	}

	protected ISourceViewer createSourceViewer(Composite parent,
			IVerticalRuler ruler, int styles) {
		fAnnotationAccess = createAnnotationAccess();
		fOverviewRuler = createOverviewRuler(getSharedColors());

		ISourceViewer viewer = new JavascriptSourceViewer(parent, ruler,
				getOverviewRuler(), isOverviewRulerVisible(), styles);
		// ensure decoration support has been created and configured.
		getSourceViewerDecorationSupport(viewer);
		return viewer;
	}

	public void createPartControl(Composite parent) {
		super.createPartControl(parent);

		ITextViewerExtension tve = (ITextViewerExtension) getSourceViewer();
		tve.appendVerifyKeyListener(new BracketCloser(
				(SourceViewer) getSourceViewer()));
		tve.appendVerifyKeyListener(new QuoteCloser(
				(SourceViewer) getSourceViewer()));
	}

	protected boolean affectsTextPresentation(PropertyChangeEvent event) {
		return ((IManagedSourceViewerConfiguration) getSourceViewerConfiguration()).affectsTextPresentation(event) ||
			super.affectsTextPresentation(event);
	}

	protected void handlePreferenceStoreChanged(PropertyChangeEvent event) {
		((IManagedSourceViewerConfiguration) getSourceViewerConfiguration()).handlePropertyChangeEvent(event);
		super.handlePreferenceStoreChanged(event);
	}

	public void setCaretPosition(CaretPosition pos) {

		try {
			int lineOffset = this.getSourceViewer().getDocument()
					.getLineOffset(pos.line);
			this.selectAndReveal(lineOffset + pos.column, 0);
		} catch (BadLocationException e) {
		}
	}

	public class CaretPosition {

		protected CaretPosition(int line, int column) {
			this.line = line;
			this.column = column;
		}

		protected int getColumn() {
			return column;
		}

		protected int getLine() {
			return line;
		}

		private int line;

		private int column;

	}

	public CaretPosition getCaretPosition() {
		// needed for positioning the cursor after formatting without selection

		StyledText styledText = this.getSourceViewer().getTextWidget();
		int caret = widgetOffset2ModelOffset(getSourceViewer(), styledText
				.getCaretOffset());
		IDocument document = getSourceViewer().getDocument();
		try {
			int line = document.getLineOfOffset(caret);
			int lineOffset = document.getLineOffset(line);
			return new CaretPosition(line, caret - lineOffset);
		} catch (BadLocationException e) {
			return new CaretPosition(0, 0);
		}
	}

}
