/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.js;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.text.rules.EndOfLineRule;
import org.eclipse.jface.text.rules.IPredicateRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.MultiLineRule;
import org.eclipse.jface.text.rules.RuleBasedPartitionScanner;
import org.eclipse.jface.text.rules.SingleLineRule;
import org.eclipse.jface.text.rules.Token;

public class JavascriptPartitionScanner extends RuleBasedPartitionScanner {

	private static JavascriptPartitionScanner instance;

	public static final String JAVASCRIPT_MULTILINE_COMMENT = "__javascript_multiline_comments";

	public static final String JAVASCRIPT_STRING = "__javascript_singleline_string";

	public static final String JAVASCRIPT_SINGLELINE_COMMENT = "__javascript_singleline_comment";

	public static final String[] JAVASCRIPT_PARTITION_TYPES = {
			JAVASCRIPT_MULTILINE_COMMENT, JAVASCRIPT_STRING,
			JAVASCRIPT_SINGLELINE_COMMENT };

	private JavascriptPartitionScanner() {
		super();

		IToken multiComment = new Token(JAVASCRIPT_MULTILINE_COMMENT);
		IToken string = new Token(JAVASCRIPT_STRING);
		IToken singleComment = new Token(JAVASCRIPT_SINGLELINE_COMMENT);

		List rules = new ArrayList();

		// Multiline comments
		rules.add(new MultiLineRule("/*", "*/", multiComment, '\\', true));

		// Singleline comments
		rules.add(new EndOfLineRule("//", singleComment, '\\'));

		// Strings
		rules.add(new SingleLineRule("\"", "\"", string, '\\', false));
		rules.add(new SingleLineRule("'", "'", string, '\\', false));

		IPredicateRule[] result = new IPredicateRule[rules.size()];
		rules.toArray(result);
		setPredicateRules(result);
	}

	public static JavascriptPartitionScanner getInstance() {
		if (instance == null) {
			instance = new JavascriptPartitionScanner();
		}
		return instance;
	}
}