/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.js;

import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.jface.text.rules.DefaultDamagerRepairer;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewerConfiguration;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.radrails.rails.internal.ui.RailsUIPlugin;
import org.radrails.rails.internal.ui.editors.IAbstractManagedScanner;
import org.radrails.rails.internal.ui.editors.IManagedSourceViewerConfiguration;
import org.radrails.rails.internal.ui.preferences.IRailsPreferenceConstants;

public class JavascriptSourceViewerConfiguration extends
		SourceViewerConfiguration implements IManagedSourceViewerConfiguration {

	private IAbstractManagedScanner fCodeScanner;
	private IAbstractManagedScanner fMultilineCommentScanner;
	private IAbstractManagedScanner fSinglelineCommentScanner;
	private IAbstractManagedScanner fStringScanner;
	
	public JavascriptSourceViewerConfiguration() {
		super();
		initScanners();
	}
	
	private void initScanners() {
		fCodeScanner = new JavascriptCodeScanner();
		fMultilineCommentScanner = new JavascriptMultilineCommentScanner();
		fSinglelineCommentScanner = new JavascriptSinglelineCommentScanner();
		fStringScanner = new JavascriptStringScanner();
	}
	
	public String getConfiguredDocumentPartitioning(ISourceViewer sourceViewer) {
		return JavascriptEditor.JAVASCRIPT_PARTITIONING;
	}

	public String[] getConfiguredContentTypes(ISourceViewer sourceViewer) {
		return new String[] { IDocument.DEFAULT_CONTENT_TYPE,
				JavascriptPartitionScanner.JAVASCRIPT_MULTILINE_COMMENT,
				JavascriptPartitionScanner.JAVASCRIPT_SINGLELINE_COMMENT,
				JavascriptPartitionScanner.JAVASCRIPT_STRING };
	}

	public IPresentationReconciler getPresentationReconciler(
			ISourceViewer sourceViewer) {
		PresentationReconciler reconciler = new PresentationReconciler();
		reconciler
				.setDocumentPartitioning(JavascriptEditor.JAVASCRIPT_PARTITIONING);

		DefaultDamagerRepairer dr = new DefaultDamagerRepairer(
				fCodeScanner);
		reconciler.setDamager(dr, IDocument.DEFAULT_CONTENT_TYPE);
		reconciler.setRepairer(dr, IDocument.DEFAULT_CONTENT_TYPE);

		dr = new DefaultDamagerRepairer(fMultilineCommentScanner);
		reconciler.setDamager(dr,
				JavascriptPartitionScanner.JAVASCRIPT_MULTILINE_COMMENT);
		reconciler.setRepairer(dr,
				JavascriptPartitionScanner.JAVASCRIPT_MULTILINE_COMMENT);

		dr = new DefaultDamagerRepairer(fSinglelineCommentScanner);
		reconciler.setDamager(dr,
				JavascriptPartitionScanner.JAVASCRIPT_SINGLELINE_COMMENT);
		reconciler.setRepairer(dr,
				JavascriptPartitionScanner.JAVASCRIPT_SINGLELINE_COMMENT);

		dr = new DefaultDamagerRepairer(fStringScanner);
		reconciler.setDamager(dr, JavascriptPartitionScanner.JAVASCRIPT_STRING);
		reconciler
				.setRepairer(dr, JavascriptPartitionScanner.JAVASCRIPT_STRING);

		return reconciler;
	}

	public int getTabWidth(ISourceViewer sourceViewer) {
		return RailsUIPlugin.getInstance().getPreferenceStore().getInt(
				IRailsPreferenceConstants.JAVASCRIPT_INDENTATION);
	}

	public boolean affectsTextPresentation(PropertyChangeEvent event) {
		return fCodeScanner.affectsBehavior(event) ||
			fSinglelineCommentScanner.affectsBehavior(event) ||
			fMultilineCommentScanner.affectsBehavior(event) ||
			fStringScanner.affectsBehavior(event);
	}

	public void handlePropertyChangeEvent(PropertyChangeEvent event) {
		if(fCodeScanner.affectsBehavior(event)) {
			fCodeScanner.adaptToPreferenceChange(event);
		}
		if(fSinglelineCommentScanner.affectsBehavior(event)) {
			fSinglelineCommentScanner.adaptToPreferenceChange(event);
		}
		if(fMultilineCommentScanner.affectsBehavior(event)) {
			fMultilineCommentScanner.adaptToPreferenceChange(event);
		}
		if(fStringScanner.affectsBehavior(event)) {
			fStringScanner.adaptToPreferenceChange(event);
		}
	}

}
