/*******************************************************************************
 * Copyright (c) 2006 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.rhtml;

import org.eclipse.jface.text.rules.ICharacterScanner;
import org.eclipse.jface.text.rules.IPredicateRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.Token;

/**
 * Predicate rule to detect an attribute partition in an HTML tag, such as
 * <code>width=</code> or <code>style=</code>
 * 
 * @author mkent
 * 
 */
public class AttributeRule implements IPredicateRule {

	private IToken fToken;

	public AttributeRule(IToken token) {
		fToken = token;
	}

	public IToken evaluate(ICharacterScanner scanner, boolean resume) {
		int ch = scanner.read();
		if (isWord(ch)) {

			do {
				ch = scanner.read();
			} while (isWord(ch));

			if (ch == '=') {
				return fToken;
			}
		}
		scanner.unread();
		return Token.UNDEFINED;
	}

	private boolean isWord(int ch) {
		return String.valueOf((char) ch).matches("\\w");
	}

	public IToken evaluate(ICharacterScanner scanner) {
		return evaluate(scanner, false);
	}

	public IToken getSuccessToken() {
		return fToken;
	}

}
