/*******************************************************************************
 * Copyright (c) 2006 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.rhtml;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.text.rules.IPredicateRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.RuleBasedPartitionScanner;
import org.eclipse.jface.text.rules.Token;

/**
 * Partition scanner for HTML attributes. Selects the equals sign and flags the
 * rest as default content.
 * 
 * @author mkent
 * 
 */
public class RHTMLAttributePartitionScanner extends RuleBasedPartitionScanner {
	private static RHTMLAttributePartitionScanner instance;

	public final static String RHTMLATT_EQUALS = "__rhtmlatt_equals";

	public final static String[] RHTMLTAG_PARTITION_TYPES = new String[] { RHTMLATT_EQUALS };

	/**
	 * @return the singleton instance of the scanner
	 */
	public static RHTMLAttributePartitionScanner getInstance() {
		if (instance == null) {
			instance = new RHTMLAttributePartitionScanner();
		}
		return instance;
	}

	/**
	 * Constructor.
	 */
	private RHTMLAttributePartitionScanner() {
		super();

		IToken equals = new Token(RHTMLATT_EQUALS);

		List rules = new ArrayList();
		rules.add(new EqualsRule(equals));

		IPredicateRule[] result = new IPredicateRule[rules.size()];
		rules.toArray(result);
		setPredicateRules(result);
	}
}
