/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.rhtml;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.TextPresentation;
import org.eclipse.jface.text.contentassist.CompletionProposal;
import org.eclipse.jface.text.contentassist.ContextInformation;
import org.eclipse.jface.text.contentassist.ICompletionProposal;
import org.eclipse.jface.text.contentassist.IContentAssistProcessor;
import org.eclipse.jface.text.contentassist.IContextInformation;
import org.eclipse.jface.text.contentassist.IContextInformationPresenter;
import org.eclipse.jface.text.contentassist.IContextInformationValidator;
import org.eclipse.jface.text.templates.Template;
import org.eclipse.jface.text.templates.TemplateCompletionProcessor;
import org.eclipse.jface.text.templates.TemplateContextType;
import org.eclipse.swt.graphics.Image;
import org.radrails.rails.internal.ui.RailsUILog;
import org.radrails.rails.internal.ui.RailsUIPlugin;
import org.radrails.rails.internal.ui.editors.WordPartDetector;
import org.radrails.rails.internal.ui.editors.templates.RhtmlContextType;
import org.radrails.rails.internal.ui.editors.templates.RhtmlTemplateManager;
import org.rubypeople.rdt.internal.ui.rubyeditor.RubyEditorMessages;

/**
 * Determines the code assist proposals.
 * 
 * @author mbaumbach
 * 
 * @version 0.4.1
 */
public class RHTMLCompletionProcessor extends TemplateCompletionProcessor
		implements IContentAssistProcessor {

	private static final Image templateImage = RailsUIPlugin
			.imageDescriptorFromPlugin("org.radrails.rails.ui",
					"icons/rails.gif").createImage();

	private static final char[] COMPLETION_CHARS = { '.' };

	protected Vector proposalList = new Vector();

	protected IContextInformationValidator validator = new Validator();

	private ITextViewer viewer;

	/**
	 * Simple content assist tip closer. The tip is valid in a range of 5
	 * characters around its popup location.
	 */
	protected static class Validator implements IContextInformationValidator,
			IContextInformationPresenter {

		protected int fInstallOffset;

		/**
		 * @see IContextInformationValidator#isContextInformationValid(int)
		 */
		public boolean isContextInformationValid(int offset) {
			return Math.abs(fInstallOffset - offset) < 5;
		}

		/**
		 * @see IContextInformationValidator#install(IContextInformation,
		 *      ITextViewer, int)
		 */
		public void install(IContextInformation info, ITextViewer viewer,
				int offset) {
			fInstallOffset = offset;
		}

		/**
		 * @see org.eclipse.jface.text.contentassist.IContextInformationPresenter#updatePresentation(int,
		 *      TextPresentation)
		 */
		public boolean updatePresentation(int documentPosition,
				TextPresentation presentation) {
			return false;
		}
	}

	/**
	 * @see org.eclipse.jface.text.contentassist.IContentAssistProcessor#computeCompletionProposals(org.eclipse.jface.text.ITextViewer,
	 *      int)
	 */
	public ICompletionProposal[] computeCompletionProposals(ITextViewer viewer,
			int documentOffset) {
		this.viewer = viewer;
		WordPartDetector wordPart = new WordPartDetector(viewer, documentOffset);

		return turnProposalVectorIntoAdaptedArray(wordPart, viewer,
				documentOffset);
	}

	/**
	 * Turns the vector into an Array of ICompletionProposal objects
	 * 
	 * @return An array of ICompletionProposals based on the list.
	 */
	protected ICompletionProposal[] turnProposalVectorIntoAdaptedArray(
			WordPartDetector word, ITextViewer viewer, int documentOffset) {
		ICompletionProposal[] templates = determineTemplateProposals(viewer,
				documentOffset);
		ICompletionProposal[] tags = determineTagProposals(viewer,
				documentOffset);
		
		ICompletionProposal[] props = new ICompletionProposal[templates.length + tags.length];
		System.arraycopy(tags, 0, props, 0, tags.length);
		System.arraycopy(templates, 0, props, tags.length, templates.length);
		return props;
	}

	/**
	 * @see org.eclipse.jface.text.contentassist.IContentAssistProcessor#computeContextInformation(org.eclipse.jface.text.ITextViewer,
	 *      int)
	 */
	public IContextInformation[] computeContextInformation(ITextViewer viewer,
			int offset) {
		IContextInformation[] result = new IContextInformation[5];
		for (int i = 0; i < result.length; i++)
			result[i] = new ContextInformation(
					MessageFormat
							.format(
									RubyEditorMessages
											.getString("CompletionProcessor.ContextInfo.display.pattern"), new Object[] { new Integer(i), new Integer(offset) }), //$NON-NLS-1$
					MessageFormat
							.format(
									RubyEditorMessages
											.getString("CompletionProcessor.ContextInfo.value.pattern"), new Object[] { new Integer(i), new Integer(offset - 5), new Integer(offset + 5) })); //$NON-NLS-1$
		return result;
	}

	/**
	 * @see org.eclipse.jface.text.contentassist.IContentAssistProcessor#getCompletionProposalAutoActivationCharacters()
	 */
	public char[] getCompletionProposalAutoActivationCharacters() {
		return COMPLETION_CHARS;
	}

	/**
	 * @see org.eclipse.jface.text.contentassist.IContentAssistProcessor#getContextInformationAutoActivationCharacters()
	 */
	public char[] getContextInformationAutoActivationCharacters() {
		return null;
	}

	/**
	 * @see org.eclipse.jface.text.contentassist.IContentAssistProcessor#getErrorMessage()
	 */
	public String getErrorMessage() {
		return null;
	}

	/**
	 * @see org.eclipse.jface.text.contentassist.IContentAssistProcessor#getContextInformationValidator()
	 */
	public IContextInformationValidator getContextInformationValidator() {
		return validator;
	}
	
	protected String getPrefix(IDocument doc, int offset) {
		int length = 0;
		String prefix = null;
		try {
			while((offset - length > 0) && Pattern.matches("\\w", doc.get(offset - length - 1, 1))) {
				length++;
			}
			prefix = doc.get(offset - length, length);
		} catch (BadLocationException e) {
			RailsUILog.logError("Error gathering proposal prefix", e);
		}
		return prefix;
	}

	private ICompletionProposal[] determineTemplateProposals(
			ITextViewer refViewer, int documentOffset) {
		String prefix = getPrefix(viewer.getDocument(),
				documentOffset);
		ICompletionProposal[] matchingTemplateProposals;
		if (prefix.length() == 0) {
			matchingTemplateProposals = super.computeCompletionProposals(
					refViewer, documentOffset);
		} else {
			ICompletionProposal[] templateProposals = super
					.computeCompletionProposals(refViewer, documentOffset);
			List templateProposalList = new ArrayList(templateProposals.length);
			for (int i = 0; i < templateProposals.length; i++) {
				if (templateProposals[i].getDisplayString().toLowerCase()
						.startsWith(prefix)) {
					templateProposalList.add(templateProposals[i]);
				}
			}
			matchingTemplateProposals = (ICompletionProposal[]) templateProposalList
					.toArray(new ICompletionProposal[templateProposalList
							.size()]);
		}
		
		return matchingTemplateProposals;
	}

	private ICompletionProposal[] determineTagProposals(ITextViewer viewer,
			int offset) {
		List tagList = new ArrayList(0);
		String tag = getTagFromPrefix(viewer, offset);
		if (tag != null) {
			tagList.add(new CompletionProposal(tag, offset, 0, 0));
		}

		return (ICompletionProposal[]) tagList.toArray(new ICompletionProposal[tagList.size()]);
	}

	private String getTagFromPrefix(ITextViewer viewer, int offset) {
		String tag = null;
		IDocument doc = viewer.getDocument();
		try {
			
			if((offset > 0) && doc.get(offset - 1, 1).equals(">")) {
				
				int tokLength = 0;
				while ((offset - tokLength > 0)
						&& !doc.get(offset - tokLength - 1, 1).equals("<")) {
					tokLength++;
				}
				String fullTag = doc.get(offset - tokLength, tokLength);
				String tagEx = "^(\\w+)(\\s+\\w+=\"\\w+\")*\\s*>";
				Matcher mat = Pattern.compile(tagEx).matcher(fullTag);
				if(mat.matches()) {
					tag = "</" + mat.group(1) + ">";
				}
			}
		} catch (BadLocationException e) {
			RailsUILog.logError("Error constructing tag completion", e);
		}
		
		return tag;
	}

	/**
	 * @see org.eclipse.jface.text.templates.TemplateCompletionProcessor#getTemplates(java.lang.String)
	 */
	protected Template[] getTemplates(String contextTypeId) {
		return RhtmlTemplateManager.getDefault().getTemplateStore()
				.getTemplates();
	}

	/**
	 * @see org.eclipse.jface.text.templates.TemplateCompletionProcessor#getContextType(org.eclipse.jface.text.ITextViewer,
	 *      org.eclipse.jface.text.IRegion)
	 */
	protected TemplateContextType getContextType(ITextViewer viewer,
			IRegion region) {
		return RhtmlTemplateManager.getDefault().getContextTypeRegistry()
				.getContextType(RhtmlContextType.ID);
	}

	/**
	 * @see org.eclipse.jface.text.templates.TemplateCompletionProcessor#getImage(org.eclipse.jface.text.templates.Template)
	 */
	protected Image getImage(Template template) {
		return templateImage;
	}

} // RHTMLCompletionProcessor
