/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.rhtml;

import java.util.ResourceBundle;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextViewerExtension;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.editors.text.EditorsUI;
import org.eclipse.ui.editors.text.ITextEditorHelpContextIds;
import org.eclipse.ui.editors.text.TextEditor;
import org.eclipse.ui.editors.text.TextFileDocumentProvider;
import org.eclipse.ui.ide.IDEActionFactory;
import org.eclipse.ui.texteditor.AbstractTextEditor;
import org.eclipse.ui.texteditor.AddTaskAction;
import org.eclipse.ui.texteditor.ChainedPreferenceStore;
import org.eclipse.ui.texteditor.DefaultRangeIndicator;
import org.eclipse.ui.texteditor.ITextEditorActionDefinitionIds;
import org.eclipse.ui.texteditor.ResourceAction;
import org.eclipse.ui.texteditor.TextOperationAction;
import org.radrails.rails.internal.ui.RailsUIPlugin;
import org.radrails.rails.internal.ui.editors.EditorMessages;
import org.radrails.rails.internal.ui.editors.ExpressionCloser;
import org.radrails.rails.internal.ui.editors.IManagedSourceViewerConfiguration;
import org.rubypeople.rdt.internal.ui.RubyPlugin;

/**
 * RHTML Editor
 * 
 * @author mbaumbach
 * 
 * @version 0.4
 */
public class RHTMLEditor extends TextEditor {

	/**
	 * Creates a new RHTMLEditor.
	 */
	public RHTMLEditor() {
		super();
		setRangeIndicator(new DefaultRangeIndicator());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.texteditor.AbstractDecoratedTextEditor#initializeEditor()
	 */
	protected void initializeEditor() {
		super.initializeEditor();

		// Add the preferences for the rails ui plugin to the editor
		IPreferenceStore pluginStore = RailsUIPlugin.getInstance()
				.getPreferenceStore();
		IPreferenceStore eclipseUIStore = EditorsUI.getPreferenceStore();
		IPreferenceStore rdtStore = RubyPlugin.getDefault()
				.getPreferenceStore();
		setPreferenceStore(new ChainedPreferenceStore(new IPreferenceStore[] {
				pluginStore, eclipseUIStore, rdtStore }));

		setSourceViewerConfiguration(new RHTMLEditorSourceViewerConfiguration());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {
		super.createPartControl(parent);

		ITextViewerExtension tve = (ITextViewerExtension) getSourceViewer();
		tve.appendVerifyKeyListener(new ExpressionCloser(
				(SourceViewer) getSourceViewer()));
	}

	protected ISourceViewer createSourceViewer(Composite parent,
			IVerticalRuler ruler, int styles) {
		fAnnotationAccess = createAnnotationAccess();
		fOverviewRuler = createOverviewRuler(getSharedColors());

		ISourceViewer viewer = new RHTMLSourceViewer(parent, ruler,
				getOverviewRuler(), isOverviewRulerVisible(), styles);
		// ensure decoration support has been created and configured.
		getSourceViewerDecorationSupport(viewer);
		return viewer;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.texteditor.AbstractTextEditor#affectsTextPresentation(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	protected boolean affectsTextPresentation(PropertyChangeEvent event) {
		return ((IManagedSourceViewerConfiguration) getSourceViewerConfiguration()).affectsTextPresentation(event) ||
			super.affectsTextPresentation(event);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.texteditor.AbstractDecoratedTextEditor#handlePreferenceStoreChanged(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	protected void handlePreferenceStoreChanged(PropertyChangeEvent event) {
		((IManagedSourceViewerConfiguration) getSourceViewerConfiguration()).handlePropertyChangeEvent(event);
		super.handlePreferenceStoreChanged(event);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.texteditor.AbstractTextEditor#createActions()
	 */
	protected void createActions() {
		super.createActions();

		ResourceBundle bundle = EditorMessages.getResourceBundle();
		IAction a = new TextOperationAction(bundle, "ContentAssistProposal.",
				this, ISourceViewer.CONTENTASSIST_PROPOSALS);

		// CTRL+Space key doesn't work without making this call
		a
				.setActionDefinitionId(ITextEditorActionDefinitionIds.CONTENT_ASSIST_PROPOSALS);
		setAction("ContentAssistProposal", a);
		a = new TextOperationAction(bundle, "ContentAssistTip.", this,
				ISourceViewer.CONTENTASSIST_CONTEXT_INFORMATION);
		a
				.setActionDefinitionId(ITextEditorActionDefinitionIds.CONTENT_ASSIST_CONTEXT_INFORMATION);
		setAction("ContentAssistTip", a);

		// Add the task action to the Edit pulldown menu (bookmark action is
		// 'free')
		ResourceAction ra = new AddTaskAction(bundle, "AddTask.", this);
		ra.setHelpContextId(ITextEditorHelpContextIds.ADD_TASK_ACTION);
		ra.setActionDefinitionId(ITextEditorActionDefinitionIds.ADD_TASK);
		setAction(IDEActionFactory.ADD_TASK.getId(), ra);

		a = new RHTMLFormatAction(bundle, "Format.", this);
		a.setActionDefinitionId("org.radrails.rails.ui.editors.format");
		setAction("Format", a);
	}

	/**
	 * @see AbstractTextEditor#editorContextMenuAboutToShow(org.eclipse.jface.action.IMenuManager)
	 */
	public void editorContextMenuAboutToShow(IMenuManager menu) {
		super.editorContextMenuAboutToShow(menu);
		addAction(menu, "ContentAssistProposal");
		addAction(menu, "ContentAssistTip");
		addAction(menu, "Format");
	}

	/**
	 * @see org.eclipse.ui.IEditorPart#init(org.eclipse.ui.IEditorSite,
	 *      org.eclipse.ui.IEditorInput)
	 */
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		setDocumentProvider(new TextFileDocumentProvider());
		super.init(site, input);
	}

	public void setCaretPosition(CaretPosition pos) {

		try {
			int lineOffset = this.getSourceViewer().getDocument()
					.getLineOffset(pos.line);
			this.selectAndReveal(lineOffset + pos.column, 0);
		} catch (BadLocationException e) {
		}
	}

	public class CaretPosition {

		protected CaretPosition(int line, int column) {
			this.line = line;
			this.column = column;
		}

		protected int getColumn() {
			return column;
		}

		protected int getLine() {
			return line;
		}

		private int line;

		private int column;

	}

	public CaretPosition getCaretPosition() {
		// needed for positioning the cursor after formatting without selection

		StyledText styledText = this.getSourceViewer().getTextWidget();
		int caret = widgetOffset2ModelOffset(getSourceViewer(), styledText
				.getCaretOffset());
		IDocument document = getSourceViewer().getDocument();
		try {
			int line = document.getLineOfOffset(caret);
			int lineOffset = document.getLineOffset(line);
			return new CaretPosition(line, caret - lineOffset);
		} catch (BadLocationException e) {
			return new CaretPosition(0, 0);
		}
	}

} // RHTMLEditor
