/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.rhtml;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.text.rules.IPredicateRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.MultiLineRule;
import org.eclipse.jface.text.rules.RuleBasedPartitionScanner;
import org.eclipse.jface.text.rules.Token;

/**
 * Partition scanner for the RHTML editor. The scanner has rules for the
 * following:
 * <ul>
 * <li>Doctype</li>
 * <li>Comments</li>
 * <li>Ruby expressions</li>
 * <li>Tags</li>
 * </ul>
 * 
 * @author kyle
 * 
 */
public class RHTMLPartitionScanner extends RuleBasedPartitionScanner {

	private static RHTMLPartitionScanner instance;

	public final static String RHTML_MULTILINE_COMMENT = "__rhtml_multiline_comment";

	public final static String RHTML_DOCTYPE = "__rhtml_doctype";

	public final static String RHTML_RUBY_EXPRESSION = "__rhtml_ruby_expression";

	public final static String RHTML_TAG = "__rhtml_tag";

	public final static String[] RHTML_PARTITION_TYPES = new String[] {
			RHTML_MULTILINE_COMMENT, RHTML_RUBY_EXPRESSION, RHTML_TAG,
			RHTML_DOCTYPE };

	/**
	 * @return the singleton instance of the scanner
	 */
	public static RHTMLPartitionScanner getInstance() {
		if (instance == null) {
			instance = new RHTMLPartitionScanner();
		}
		return instance;
	}

	/**
	 * Constructor.
	 */
	private RHTMLPartitionScanner() {
		super();

		IToken comment = new Token(RHTML_MULTILINE_COMMENT);
		IToken doctype = new Token(RHTML_DOCTYPE);
		IToken ruby = new Token(RHTML_RUBY_EXPRESSION);
		IToken tag = new Token(RHTML_TAG);

		List rules = new ArrayList();
		rules.add(new MultiLineRule("<!DOCTYPE", ">", doctype, (char) 0, true));
		rules.add(new MultiLineRule("<!--", "-->", comment, (char) 0, true));
		rules.add(new MultiLineRule("<%", "%>", ruby, (char) 0, true));
		rules.add(new TagRule(tag));

		IPredicateRule[] result = new IPredicateRule[rules.size()];
		rules.toArray(result);
		setPredicateRules(result);
	}

}
