/*******************************************************************************
 * Copyright (c) 2006 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.rhtml;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.text.rules.IPredicateRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.MultiLineRule;
import org.eclipse.jface.text.rules.RuleBasedPartitionScanner;
import org.eclipse.jface.text.rules.Token;

/**
 * Partitions an HTML tag into attributes, strings, and possibly ERb
 * expressions.
 * 
 * @author mkent
 * 
 */
public class RHTMLTagPartitionScanner extends RuleBasedPartitionScanner {

	public final static String RHTMLTAG_ATTRIBUTE = "__rhtmltag_attribute";

	public final static String RHTMLTAG_STRING = "__rhtmltag_string";

	public final static String RHTMLTAG_RUBY_EXPRESSION = "__rhtmltag_ruby_expression";

	public final static String[] RHTMLTAG_PARTITION_TYPES = new String[] {
			RHTMLTAG_ATTRIBUTE, RHTMLTAG_STRING, RHTMLTAG_RUBY_EXPRESSION };

	/**
	 * Constructor.
	 */
	public RHTMLTagPartitionScanner() {
		super();

		IToken att = new Token(RHTMLTAG_ATTRIBUTE);
		IToken string = new Token(RHTMLTAG_STRING);
		IToken ruby = new Token(RHTMLTAG_RUBY_EXPRESSION);

		List rules = new ArrayList();
		rules.add(new MultiLineRule("<%", "%>", ruby, (char) 0, true));
		rules.add(new MultiLineRule("\"", "\"", string, (char) 0, true));
		rules.add(new MultiLineRule("'", "'", string, (char) 0, true));
		rules.add(new AttributeRule(att));

		IPredicateRule[] result = new IPredicateRule[rules.size()];
		rules.toArray(result);
		setPredicateRules(result);
	}
}
