/*******************************************************************************
 * Copyright (c) 2006 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.rhtml;

import java.util.Map;

import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.rules.ITokenScanner;
import org.radrails.rails.internal.ui.editors.CompositeTokenScanner;

/**
 * Scans and colors the partitions of an HTML tag in an RHTML document.
 * 
 * @author mkent
 * 
 */
public class RHTMLTagScanner extends CompositeTokenScanner {

	public RHTMLTagScanner() {
		super(new RHTMLTagPartitionScanner());
	}

	protected Map createTokenScanners() {
		Map tokenScanners = super.createTokenScanners();

		ITokenScanner tagScanner = new DefaultTagScanner();
		ITokenScanner attributeScanner = new RHTMLAttributeScanner();
		ITokenScanner stringScanner = new DefaultStringScanner();
		ITokenScanner rubyScanner = new RubyExpressionScanner();

		tokenScanners.put(IDocument.DEFAULT_CONTENT_TYPE, tagScanner);
		tokenScanners.put(RHTMLTagPartitionScanner.RHTMLTAG_ATTRIBUTE,
				attributeScanner);
		tokenScanners.put(RHTMLTagPartitionScanner.RHTMLTAG_STRING,
				stringScanner);
		tokenScanners.put(RHTMLTagPartitionScanner.RHTMLTAG_RUBY_EXPRESSION,
				rubyScanner);

		return tokenScanners;
	}
}
