/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.templates;

import java.io.IOException;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.templates.ContextTypeRegistry;
import org.eclipse.jface.text.templates.persistence.TemplateStore;
import org.eclipse.ui.editors.text.templates.ContributionContextTypeRegistry;
import org.eclipse.ui.editors.text.templates.ContributionTemplateStore;
import org.radrails.rails.internal.ui.RailsUILog;
import org.radrails.rails.internal.ui.RailsUIPlugin;

/**
 * Manages templates for the editors.
 *
 * @author	mbaumbach
 *
 * @version	0.3.1
 */
public class RailsTemplateManager {
	/** Key to store custom templates. */
	private static final String CUSTOM_TEMPLATES_KEY= "org.radrails.rails.internal.ui.customtemplates"; //$NON-NLS-1$
	
	/** The shared instance. */
	private static RailsTemplateManager fgInstance;
	
	/** The template store. */
	private TemplateStore fStore;
	
	/** The context type registry. */
	private ContributionContextTypeRegistry fRegistry;
	
	private RailsTemplateManager() {}

	/**
	 * Returns the shared instance.
	 * 
	 * @return the shared instance
	 */
	public static RailsTemplateManager getInstance() {
		if (fgInstance == null) {
			fgInstance= new RailsTemplateManager();
		}
		return fgInstance;
	}

	/**
	 * Returns this plug-in's template store.
	 * 
	 * @return the template store of this plug-in instance
	 */
	public TemplateStore getTemplateStore() {
		if (fStore == null) {
			fStore= new ContributionTemplateStore(getContextTypeRegistry(),RailsUIPlugin.getInstance().getPreferenceStore(), CUSTOM_TEMPLATES_KEY);
			try {
				fStore.load();
			} catch (IOException e) {
				RailsUILog.logError("Unable to load template store", e);
			}
		}
		return fStore;
	}

	/**
	 * Returns this plug-in's context type registry.
	 * 
	 * @return the context type registry for this plug-in instance
	 */
	public ContextTypeRegistry getContextTypeRegistry() {
		if (fRegistry == null) {
			// create and configure the contexts available in the template editor
			fRegistry= new ContributionContextTypeRegistry();
//			fRegistry.addContextType(RailsFileContextType.RAILSFILE_CONTEXT_TYPE);
			fRegistry.addContextType("Ruby");
		}
		return fRegistry;
	}

	/**
	 * Gets the preference store.
	 * 
	 * @return	The preference store.
	 */
	public IPreferenceStore getPreferenceStore() {	    
		return RailsUIPlugin.getInstance().getPreferenceStore();
	}

	/**
	 * Saves the plugin preferences.
	 */
	public void savePluginPreferences() {
		RailsUIPlugin.getInstance().savePluginPreferences();
	}
}
