/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.yml;

import org.eclipse.jface.text.rules.ICharacterScanner;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.Token;

/**
 * Rule for detecting element names in a YML file.
 * 
 * @author mkent
 * 
 */
public class NameRule implements IRule {

	private static final int UNDEFINED = -1;

	private IToken token;

	private int columnConstraint;

	/**
	 * Constructor. Column constraint defaults to undefined.
	 * 
	 * @param token
	 *            the token to return on success
	 */
	public NameRule(IToken token) {
		this(token, UNDEFINED);
	}

	/**
	 * Constructor.
	 * 
	 * @param token
	 *            the token to return on success
	 * @param columnConstraint
	 *            the column that the name must start in
	 */
	public NameRule(IToken token, int columnConstraint) {
		this.token = token;
		this.columnConstraint = columnConstraint;
	}

	/**
	 * Helper method to determine if the given character is a word character.
	 * 
	 * @param ch
	 *            the character to check
	 * @return true if the character is a word character, false otherwise
	 */
	private boolean isWordCharacter(int ch) {
		return String.valueOf((char) ch).matches("\\w");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.text.rules.IRule#evaluate(org.eclipse.jface.text.rules.ICharacterScanner)
	 */
	public IToken evaluate(ICharacterScanner scanner) {
		// Check column constraint
		if ((columnConstraint != UNDEFINED)
				&& (scanner.getColumn() != columnConstraint)) {
			return Token.UNDEFINED;
		}

		int ch = scanner.read();

		// Names must start with a word character
		if (isWordCharacter(ch)) {

			// Read more word characters
			int count = 0;
			do {
				ch = scanner.read();
				count++;
			} while (isWordCharacter(ch));

			// Names must end with a colon
			if (ch == ':') {
				return token;
			}

			for (int i = 0; i < count; i++) {
				scanner.unread();
			}
		}

		scanner.unread();
		return Token.UNDEFINED;
	}

}
