/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.yml;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.editors.text.EditorsUI;
import org.eclipse.ui.editors.text.TextEditor;
import org.eclipse.ui.texteditor.ChainedPreferenceStore;
import org.radrails.rails.internal.ui.RailsUIPlugin;

/**
 * Source editor for YML files.
 * 
 * @author mkent
 * 
 */
public class YMLEditor extends TextEditor {

	public static final String YML_PARITIONING = "__yml_partitioning";

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.texteditor.AbstractDecoratedTextEditor#initializeEditor()
	 */
	protected void initializeEditor() {
		super.initializeEditor();

		// Add the db ui plugin preferences to the editor
		IPreferenceStore pluginStore = RailsUIPlugin.getInstance()
				.getPreferenceStore();
		IPreferenceStore eclipseUIStore = EditorsUI.getPreferenceStore();
		setPreferenceStore(new ChainedPreferenceStore(new IPreferenceStore[] {
				pluginStore, eclipseUIStore }));

		setSourceViewerConfiguration(new YMLSourceViewerConfiguration());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.texteditor.AbstractTextEditor#affectsTextPresentation(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	protected boolean affectsTextPresentation(PropertyChangeEvent event) {
		if (event.getProperty().equals(IYMLEditorColorConstants.STRING)
				|| event.getProperty().equals(
						IYMLEditorColorConstants.NODE_NAME)
				|| event.getProperty().equals(
						IYMLEditorColorConstants.MAPPING_KEY)
				|| event.getProperty().equals(
						IYMLEditorColorConstants.YML_COMMENT)) {
			YMLScanner.getInstance().applyPropertyChange(event);
			return true;
		}

		return false;
	}

	protected ISourceViewer createSourceViewer(Composite parent,
			IVerticalRuler ruler, int styles) {
		fAnnotationAccess = createAnnotationAccess();
		fOverviewRuler = createOverviewRuler(getSharedColors());

		ISourceViewer viewer = new YMLSourceViewer(parent, ruler,
				getOverviewRuler(), isOverviewRulerVisible(), styles);
		// ensure decoration support has been created and configured.
		getSourceViewerDecorationSupport(viewer);
		return viewer;
	}
}
