/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.yml;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.rules.BufferedRuleBasedScanner;
import org.eclipse.jface.text.rules.EndOfLineRule;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.SingleLineRule;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.radrails.rails.internal.ui.RailsUIColorProvider;

/**
 * Token scanner for the YML editor. Returns tokens for node names, mapping
 * keys, and comments.
 * 
 * @author mkent
 * 
 */
public class YMLScanner extends BufferedRuleBasedScanner {

	private static YMLScanner instance;

	private Map tokenMap;

	private YMLScanner() {
		loadTokenMap();

		// Create color tokens
		IToken string = (IToken) tokenMap.get(IYMLEditorColorConstants.STRING);
		IToken nodeName = (IToken) tokenMap
				.get(IYMLEditorColorConstants.NODE_NAME);
		IToken mapKey = (IToken) tokenMap
				.get(IYMLEditorColorConstants.MAPPING_KEY);
		IToken comment = (IToken) tokenMap
				.get(IYMLEditorColorConstants.YML_COMMENT);

		List rules = new ArrayList();

		// String rules
		rules.add(new SingleLineRule("\"", "\"", string, '\\', false));
		rules.add(new SingleLineRule("'", "'", string, '\\', false));

		// Node name rule
		rules.add(new NameRule(nodeName, 0));

		// Mapping key rule
		rules.add(new NameRule(mapKey, 2));

		// Comment rule
		rules.add(new EndOfLineRule("#", comment));

		// Add the rules to the scanner
		IRule[] result = new IRule[rules.size()];
		rules.toArray(result);
		setRules(result);
	}

	public static YMLScanner getInstance() {
		if (instance == null) {
			instance = new YMLScanner();
		}
		return instance;
	}

	public void applyPropertyChange(PropertyChangeEvent event) {
		Color newColor = null;
		if (event.getNewValue() instanceof String) {
			newColor = RailsUIColorProvider.getInstance().getColor(
					(String) event.getNewValue());
		} else {
			newColor = RailsUIColorProvider.getInstance().getColor(
					(RGB) event.getNewValue());
		}

		Token tok = (Token) tokenMap.get(event.getProperty());
		TextAttribute oldAtt = (TextAttribute) tok.getData();
		tok.setData(new TextAttribute(newColor, oldAtt.getBackground(), oldAtt
				.getStyle()));
	}

	private void loadTokenMap() {
		tokenMap = new HashMap();

		tokenMap.put(IYMLEditorColorConstants.STRING, new Token(
				new TextAttribute(
						RailsUIColorProvider.getInstance()
								.getColorFromPreference(
										IYMLEditorColorConstants.STRING))));
		tokenMap.put(IYMLEditorColorConstants.NODE_NAME, new Token(
				new TextAttribute(RailsUIColorProvider.getInstance()
						.getColorFromPreference(
								IYMLEditorColorConstants.NODE_NAME), null,
						SWT.BOLD)));
		tokenMap.put(IYMLEditorColorConstants.MAPPING_KEY, new Token(
				new TextAttribute(RailsUIColorProvider.getInstance()
						.getColorFromPreference(
								IYMLEditorColorConstants.MAPPING_KEY))));
		tokenMap.put(IYMLEditorColorConstants.YML_COMMENT, new Token(
				new TextAttribute(RailsUIColorProvider.getInstance()
						.getColorFromPreference(
								IYMLEditorColorConstants.YML_COMMENT), null,
						SWT.ITALIC)));
	}
}
