/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.generators;

import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.TreeMap;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.ViewPart;
import org.radrails.internal.core.launching.IRailsLaunchConfigAttributes;
import org.radrails.rails.core.RailsLog;
import org.radrails.rails.internal.core.RailsPlugin;
import org.radrails.rails.internal.core.RailsRuntime;
import org.radrails.rails.internal.generators.Generator;
import org.radrails.rails.internal.generators.GeneratorLocatorsManager;
import org.radrails.rails.internal.ui.IRailsUIConstants;

/**
 * The Generators view provides an interface to the Rails generate script and
 * the following generators.
 * <ul>
 * <li>controller</li>
 * <li>mailer</li>
 * <li>migration</li>
 * <li>model</li>
 * <li>plugin</li>
 * <li>scaffold</li>
 * <li>web_service</li>
 * 
 * @author mkent
 * 
 */
public class GeneratorsView extends ViewPart implements Observer {

	private static final int MAX_INPUT_HISTORY_SIZE = 25;
	
	private Combo genCombo;

	private Button createButton;

	private Button destroyButton;

	private Button pretendButton;

	private Button skipButton;

	private Button forceButton;

	private Button quietButton;

	private Button backtraceButton;

	private Button helpButton;

	private Button svnButton;

	private Combo paramText;

	private Map generatorsMap;

	private Composite parent;

	private StackLayout layout;

	private Composite generateView;

	private Label selectProjectView;;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IViewPart#init(org.eclipse.ui.IViewSite)
	 */
	public void init(IViewSite site) throws PartInitException {
		super.init(site);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {

		this.parent = parent;
		layout = new StackLayout();
		parent.setLayout(layout);

		generateView = new Composite(parent, SWT.NONE);
		generateView.setLayout(new GridLayout());
		createGeneratorControls(generateView);

		selectProjectView = new Label(parent, SWT.NONE);
		selectProjectView
				.setText(IRailsUIConstants.SELECT_RAILS_PROJECT_STRING);

		layout.topControl = selectProjectView;
		parent.layout();

		GeneratorLocatorsManager.getInstance().addObserver(this);
	}

	/**
	 * Helper method to create the necessary widgets.
	 * 
	 * @param parent
	 *            the parent composite
	 */
	private void createGeneratorControls(Composite parent) {
		Composite comp = new Composite(parent, SWT.NULL);

		// Create the layout for the view
		GridLayout compLayout = new GridLayout();
		compLayout.numColumns = 3;
		comp.setLayout(compLayout);
		GridData compLayoutData = new GridData(GridData.FILL_HORIZONTAL);
		comp.setLayoutData(compLayoutData);

		// Create the combo box of generators
		genCombo = new Combo(comp, SWT.DROP_DOWN);
		GridData genComboData = new GridData();
		genComboData.widthHint = 200;
		genCombo.setLayoutData(genComboData);

		// Create the parameters text field
		paramText = new Combo(comp, SWT.BORDER);
		GridData paramTextData = new GridData(GridData.FILL_HORIZONTAL);
		paramText.setLayoutData(paramTextData);

		paramText.addKeyListener(new KeyListener() {
			public void keyPressed(KeyEvent e) {
				// Do nothing
			}

			public void keyReleased(KeyEvent e) {
				// Take action if Enter was pressed
				if (e.character == SWT.CR) {
					executeCommand();
				}
			}
		});

		// Create the Go button
		Button genButton = new Button(comp, SWT.PUSH);
		genButton.setText("Go");

		// Run the command when the button is clicked
		genButton.addSelectionListener(new SelectionListener() {

			public void widgetSelected(SelectionEvent e) {
				executeCommand();
			}

			public void widgetDefaultSelected(SelectionEvent e) {
				// Do nothing
			}

		});

		// Create the radio buttons for create/destroy
		Composite modes = new Composite(comp, SWT.NULL);
		modes.setLayout(new GridLayout(2, false));

		createButton = new Button(modes, SWT.RADIO);
		createButton.setText("Create");
		createButton.setSelection(true);
		destroyButton = new Button(modes, SWT.RADIO);
		destroyButton.setText("Destroy");

		// Create the text field for the options information
		Group optionsGroup = new Group(comp, SWT.NULL);
		optionsGroup.setLayout(new GridLayout(7, false));
		optionsGroup.setText("Options");
		optionsGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		pretendButton = new Button(optionsGroup, SWT.CHECK);
		pretendButton.setText("Pretend");
		forceButton = new Button(optionsGroup, SWT.CHECK);
		forceButton.setText("Force");
		skipButton = new Button(optionsGroup, SWT.CHECK);
		skipButton.setText("Skip");
		quietButton = new Button(optionsGroup, SWT.CHECK);
		quietButton.setText("Quiet");
		backtraceButton = new Button(optionsGroup, SWT.CHECK);
		backtraceButton.setText("Backtrace");
		svnButton = new Button(optionsGroup, SWT.CHECK);
		svnButton.setText("Use SVN");

		// Blank
		helpButton = new Button(comp, SWT.PUSH);
		helpButton.setText("Help");
		// Run the command when the button is clicked
		helpButton.addSelectionListener(new SelectionListener() {

			public void widgetSelected(SelectionEvent e) {
				String genName;
				if (!(genName = genCombo.getText()).equals("")) {
					String command = "script/generate " + genName
							+ " --help";
					launchTerminal("generate " + genName + " --help", command);
				}
			}

			public void widgetDefaultSelected(SelectionEvent e) {
				// Do nothing
			}

		});

		new Label(comp, SWT.NULL);

		// Populate the usage field when an item is selected from the combo
		genCombo.addSelectionListener(new SelectionListener() {

			public void widgetSelected(SelectionEvent e) {
				// Combo c = (Combo) e.widget;
				// TODO Use this for drop down history in the future.
			}

			public void widgetDefaultSelected(SelectionEvent e) {
				// Do nothing
			}

		});

	}

	public void setFocus() {
		// Do nothing
	}

	/**
	 * Runs the command specified in the view.
	 */
	private void executeCommand() {
		String script = null;

		// Check create or destroy
		if (createButton.getSelection()) {
			script = "script/generate ";
		} else if (destroyButton.getSelection()) {
			script = "script/destroy ";
		}

		// Check the options boxes
		String options = "";
		if (pretendButton.getSelection()) {
			options += "p";
		}
		if (forceButton.getSelection()) {
			options += "f";
		}
		if (skipButton.getSelection()) {
			options += "s";
		}
		if (quietButton.getSelection()) {
			options += "q";
		}
		if (backtraceButton.getSelection()) {
			options += "t";
		}
		if (svnButton.getSelection()) {
			options += "c";
		}
		if (options.length() > 0) {
			options = " -" + options;
		}

		// Prepare the full command
		String command = script + genCombo.getText() + " "
				+ paramText.getText() + options;

		IProject project = RailsPlugin.getInstance()
				.getSelectedRailsOrRubyProject();

		if (project != null) {
			// Update the input field history
			LinkedList items = new LinkedList(Arrays.asList(paramText.getItems()));
			items.addFirst(paramText.getText());
			if (items.size() > MAX_INPUT_HISTORY_SIZE) {
				items.removeLast();
			}
			paramText.setItems((String[]) items.toArray(new String[0]));
			
			launchTerminal(script.substring(script.indexOf("/")) + genCombo.getText(), command);
		} else {
			layout.topControl = selectProjectView;
			parent.layout();
		}
	}

	private void launchTerminal(String title, String command) {
		IProject project = RailsPlugin.getInstance()
				.getSelectedRailsOrRubyProject();
		
		// launch the generator command
		try {
			ILaunchConfigurationWorkingCopy wc = RailsRuntime
					.createInternalLaunch(IRailsLaunchConfigAttributes.SCRIPT_RUBY, title);
			wc.setAttribute(IRailsLaunchConfigAttributes.PROGRAM_ARGS, command);
			wc.setAttribute(IRailsLaunchConfigAttributes.WORKING_DIR, project
					.getLocation().toOSString());
			wc.setAttribute(IRailsLaunchConfigAttributes.SCRIPT_TYPE,
					IRailsLaunchConfigAttributes.SCRIPT_RUBY);
			// refresh just to be safe, even though not every task requires it
			wc.setAttribute(IRailsLaunchConfigAttributes.REQUIRES_REFRESH, "true");
			wc.setAttribute(IRailsLaunchConfigAttributes.PROJECT_NAME, project.getName());

			ILaunchConfiguration config = wc.doSave();
			config.launch(ILaunchManager.RUN_MODE, null);
		} catch (CoreException e) {
			RailsLog.logError("Error running generator", e);
		}
	}

	/**
	 * Reload the list of generators in the combo box from the generators
	 * currently discovered in the GeneratorsLocatorManager without doing any
	 * reparsing,
	 * 
	 */
	private void reloadGeneratorsPullDown() {

		genCombo.removeAll();

		Collection generators = generatorsMap.values();

		Iterator i = generators.iterator();
		while (i.hasNext()) {
			Generator gen = (Generator) i.next();
			genCombo.add(gen.getName());
		}

		genCombo.update();
	}

	/**
	 * Force a relocation of generators.
	 * 
	 */
	private void refreshGeneratorsList() {
		GeneratorLocatorsManager.getInstance().refreshAllGenerators();
	}

	private void reloadGeneratorsMap() {

		generatorsMap = new TreeMap();
		List generators = GeneratorLocatorsManager.getInstance()
				.getAllGenerators();
		Iterator genIt = generators.iterator();
		while (genIt.hasNext()) {
			Generator gen = (Generator) genIt.next();
			generatorsMap.put(gen.getName(), gen);
		}
	}

	protected void refreshGenerators() {
		refreshGeneratorsList();
		reloadGeneratorsMap();
		reloadGeneratorsPullDown();
	}

	public void update(Observable genManger, Object project) {
		if (!parent.isDisposed()) {
			if (project == null) {
				layout.topControl = selectProjectView;
				parent.layout();
			} else {
				reloadGeneratorsMap();
				reloadGeneratorsPullDown();
				layout.topControl = generateView;
				parent.layout();
			}
		}
	}

}
