/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.preferences;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.radrails.rails.internal.ui.RailsUIPlugin;
import org.radrails.rails.internal.ui.editors.css.ICSSEditorColorConstants;

/**
 * Preference page for the CSS editor.
 * 
 * @author	mbaumbach
 * 
 * @version	0.4.1
 */
public class CSSEditorPreferencePage extends SourceEditorPreferencePage
		implements IWorkbenchPreferencePage {

	/**
	 * Constructor.
	 */
	public CSSEditorPreferencePage() {
		setPreferenceStore(RailsUIPlugin.getInstance().getPreferenceStore());
		setDescription("CSS editor preferences");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.FieldEditorPreferencePage#createFieldEditors()
	 */
	protected void createFieldEditors() {
		Composite parent = getFieldEditorParent();
		parent.setLayout(new GridLayout());
		createSyntaxEditor(parent, ICSSEditorColorConstants.COMMENT, "Comment");
		createSyntaxEditor(parent, ICSSEditorColorConstants.SELECTOR, "Selector");
		createSyntaxEditor(parent, ICSSEditorColorConstants.PROPERTY, "Property");
		createSyntaxEditor(parent, ICSSEditorColorConstants.VALUE, "Value");
		addField(new StringFieldEditor(IRailsPreferenceConstants.CSS_INDENTATION, "Character(s) per indentation level", 3, getFieldEditorParent()));
		addField(new BooleanFieldEditor(IRailsPreferenceConstants.CSS_FORMAT_USE_TAB, "Use Tabs", getFieldEditorParent()));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
		// Do nothing
	}

} // CSSEditorPreferencePage