package org.radrails.rails.internal.ui.preferences;

import org.eclipse.core.runtime.Preferences;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.FileFieldEditor;
import org.eclipse.jface.preference.PreferenceStore;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.radrails.rails.core.IRailsConstants;
import org.radrails.rails.internal.core.RailsPlugin;

/**
 * @author matt
 *
 */
public class ConfigurationPreferencePage extends FieldEditorPreferencePage
		implements IWorkbenchPreferencePage {

	public ConfigurationPreferencePage() {
		super(GRID);
		setPreferenceStore(new AdaptedPreferences(RailsPlugin.getInstance().getPluginPreferences()));
		StringBuffer desc = new StringBuffer();
		desc.append("Set the locations of the scripts that RadRails depends on.\n");
		desc.append("Example (win32): C:\\ruby\\bin\\rails\n");
		desc.append("Example (linux): /usr/local/bin/rails\n");
		desc.append("Example (os x): /usr/bin/rails\n");
		setDescription(desc.toString());
	}
	
	protected void createFieldEditors() {
		addField(new FileFieldEditor(IRailsConstants.PREF_RAILS_PATH, "Rails path", getFieldEditorParent()));
		addField(new FileFieldEditor(IRailsConstants.PREF_RAKE_PATH, "Rake path", getFieldEditorParent()));
		addField(new FileFieldEditor(IRailsConstants.PREF_MONGREL_PATH, "Mongrel path", getFieldEditorParent()));
	}

	public void init(IWorkbench workbench) {

	}

	private class AdaptedPreferences extends PreferenceStore {
		
		private Preferences fPrefs;
		
		public AdaptedPreferences(Preferences prefs) {
			fPrefs = prefs;
		}

		public String getString(String name) {
			return fPrefs.getString(name);
		}

		public void setValue(String name, String value) {
			fPrefs.setValue(name, value);
		}

		public String getDefaultString(String name) {
			return fPrefs.getDefaultString(name);
		}
		
	}
}
