/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.preferences;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbench;
import org.radrails.rails.internal.ui.RailsUIPlugin;
import org.radrails.rails.internal.ui.editors.rhtml.IRHTMLEditorColorConstants;

/**
 * Preference page for the RHTML editor.
 * 
 * @author mkent
 * 
 */
public class RHTMLEditorPreferencePage extends SourceEditorPreferencePage {

	/**
	 * Constructor.
	 */
	public RHTMLEditorPreferencePage() {
		setPreferenceStore(RailsUIPlugin.getInstance().getPreferenceStore());
		setDescription("RHTML editor preferences");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.FieldEditorPreferencePage#createFieldEditors()
	 */
	protected void createFieldEditors() {
		Composite parent = getFieldEditorParent();
		parent.setLayout(new GridLayout());
		
		createSyntaxEditor(parent, IRHTMLEditorColorConstants.COMMENT, "Comment");
		createSyntaxEditor(parent, IRHTMLEditorColorConstants.TAG, "Tag");
		createSyntaxEditor(parent, IRHTMLEditorColorConstants.ATTRIBUTE, "Attribute");
		createSyntaxEditor(parent, IRHTMLEditorColorConstants.STRING, "String");
		createSyntaxEditor(parent, IRHTMLEditorColorConstants.DOCTYPE, "Doctype");
		createSyntaxEditor(parent, IRHTMLEditorColorConstants.RUBY_EXPRESSION_BACKGROUND,
				"Ruby expression");
		addField(new StringFieldEditor(IRailsPreferenceConstants.RHTML_INDENTATION, "Character(s) per indentation level", 3, getFieldEditorParent()));
		addField(new BooleanFieldEditor(IRailsPreferenceConstants.RHTML_FORMAT_USE_TAB, "Use Tabs", getFieldEditorParent()));
		addField(new BooleanFieldEditor(IRailsPreferenceConstants.RHTML_AUTOINSERT, "Content assist auto-insert", getFieldEditorParent()));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
		// Do nothing
	}

}
