/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.railsnavigator;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.DecoratingLabelProvider;
import org.eclipse.jface.viewers.ILabelDecorator;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.radrails.rails.internal.ui.IRailsUIConstants;
import org.radrails.rails.internal.ui.RailsUILog;
import org.radrails.rails.internal.ui.RailsUIPlugin;

/**
 * A class to take control of decorating labels and images on the
 * {@link RailsNavigator RailsNavigator}.
 * 
 * @author Kyle
 */
public class RailsDecoratingLabelProvider extends DecoratingLabelProvider {

	/**
	 * Constructor
	 * 
	 * @param provider
	 * @param decorator
	 */
	public RailsDecoratingLabelProvider(ILabelProvider provider,
			ILabelDecorator decorator) {
		super(provider, decorator);
	}

	/**
	 * @see org.eclipse.jface.viewers.ILabelProvider#getImage(java.lang.Object)
	 */
	public Image getImage(Object o) {
		if (o instanceof IProject) {
			IProject p = (IProject) o;
			ImageDescriptor image = null;
			try {
				if (p.isOpen()) {
					if (p.hasNature(IRailsUIConstants.RAILS_PROJECT_NATURE)) {
						image = AbstractUIPlugin.imageDescriptorFromPlugin(
								RailsUIPlugin.getInstance().getBundle()
										.getSymbolicName(),
								"icons/nav_railsproj.gif");
					} else {
						image = AbstractUIPlugin.imageDescriptorFromPlugin(
								RailsUIPlugin.getInstance().getBundle()
										.getSymbolicName(),
								"icons/nav_proj.gif");
					}
				} else {
					image = AbstractUIPlugin.imageDescriptorFromPlugin(
							RailsUIPlugin.getInstance().getBundle()
									.getSymbolicName(),
							"icons/nav_closeproj.gif");
				}
			} catch (CoreException e) {
				RailsUILog.logError("Error retrieving project icon", e);
			}
			if (image != null) {
				return image.createImage();
			}
		} 
		return super.getImage(o);
	}

}
