/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.railsnavigator;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.ui.model.WorkbenchLabelProvider;
import org.eclipse.ui.views.navigator.ResourceNavigator;
import org.radrails.rails.internal.core.RailsPlugin;

/**
 * Rails resource navigator.
 * 
 * @author Kyle
 * 
 */
public class RailsNavigator extends ResourceNavigator implements
		ISelectionChangedListener {

	private IProject selectedProject = null;

	/**
	 * Default constructor.
	 */
	public RailsNavigator() {
		super();
	}

	/**
	 * @see org.eclipse.ui.views.navigator.ResourceNavigator#initLabelProvider(org.eclipse.jface.viewers.TreeViewer)
	 */
	protected void initLabelProvider(TreeViewer viewer) {
		viewer.setLabelProvider(new RailsDecoratingLabelProvider(
				new WorkbenchLabelProvider(), getPlugin().getWorkbench()
						.getDecoratorManager().getLabelDecorator()));
		viewer.addSelectionChangedListener(this);

	}

	public void selectionChanged(SelectionChangedEvent event) {

		// Any number of the project selection stuff could go wrong and return
		// null,
		// so catch NullPointers rather than slowly checking everything safely.
		try {
			IStructuredSelection selection = (IStructuredSelection) event
					.getSelection();
			IProject inProject = ((IResource) selection.getFirstElement())
					.getProject();

			if (!inProject.isOpen()) {
				selectedProject = null;
				RailsPlugin.getInstance().setSelectedProject(null);
			} else if (selectedProject != inProject) {
				selectedProject = inProject;
				RailsPlugin.getInstance().setSelectedProject(selectedProject);
			}
		} catch (NullPointerException e) {
			// Something went wrong, or nothing selected at all, so no project
			// can be selected.
			selectedProject = null;
			RailsPlugin.getInstance().setSelectedProject(null);
		}
	}

}
