/*******************************************************************************
 * Copyright (c) 2006 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.railsplugins;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.core.ILaunchManager;
import org.radrails.internal.core.launching.IRailsLaunchConfigAttributes;
import org.radrails.rails.core.RailsLog;
import org.radrails.rails.core.railsplugins.RailsPluginDescriptor;
import org.radrails.rails.internal.core.RailsRuntime;

/**
 * Helper methods for RailsPluginsView.
 * 
 * @author mkent
 * 
 */
public class RailsPluginsHelper {

	/**
	 * Install a Rails plugin from an SVN repository. Information about the
	 * plugin is provided in a RailsPluginDescriptor.
	 * 
	 * @param project
	 *            the project to install the plugin into
	 * @param plugin
	 *            the plugin to install
	 * @param externals
	 *            true if an svn:externals entry is to be added, false otherwise
	 */
	public static void installPlugin(IProject project,
			RailsPluginDescriptor plugin, boolean externals, boolean checkout) {

		// Prepare plugin info
		String repos = (String) plugin.getProperty("repository");

		// Prepare the command
		String command = null;
		if (externals || checkout) {
			String ext_str = externals ? "x" : "";
			String check_str = checkout ? "o" : "";
			command = "script/plugin install -" + ext_str + check_str + " "
					+ repos;
		} else {
			command = "script/plugin install " + repos;
		}

		try {
			ILaunchConfigurationWorkingCopy wc = RailsRuntime
					.createInternalLaunch(
							IRailsLaunchConfigAttributes.SCRIPT_RUBY,
							"plugin install");
			wc.setAttribute(IRailsLaunchConfigAttributes.PROGRAM_ARGS, command);
			wc.setAttribute(IRailsLaunchConfigAttributes.WORKING_DIR, project
					.getLocation().toOSString());
			wc.setAttribute(IRailsLaunchConfigAttributes.SCRIPT_TYPE,
					IRailsLaunchConfigAttributes.SCRIPT_RUBY);
			wc.setAttribute(IRailsLaunchConfigAttributes.REQUIRES_REFRESH, "true");
			wc.setAttribute(IRailsLaunchConfigAttributes.PROJECT_NAME, project.getName());

			ILaunchConfiguration config = wc.doSave();
			config.launch(ILaunchManager.RUN_MODE, null);
		} catch (CoreException e) {
			RailsLog.logError("Error running rake task", e);
		}
	}

	/**
	 * Removes a plugin from the given project.
	 * 
	 * @param project
	 *            the project to remove the plugin from
	 * @param plugin
	 *            the plugin to remove
	 */
	public static void removePlugin(IProject project,
			RailsPluginDescriptor plugin) {

		// Prepare the plugin info
		String repos = (String) plugin.getProperty("repository");

		// Prepare the command
		String command = "script/plugin remove " + repos;

		try {
			ILaunchConfigurationWorkingCopy wc = RailsRuntime
					.createInternalLaunch(
							IRailsLaunchConfigAttributes.SCRIPT_RUBY,
							"plugin remove");
			wc.setAttribute(IRailsLaunchConfigAttributes.PROGRAM_ARGS, command);
			wc.setAttribute(IRailsLaunchConfigAttributes.WORKING_DIR, project
					.getLocation().toOSString());
			wc.setAttribute(IRailsLaunchConfigAttributes.SCRIPT_TYPE,
					IRailsLaunchConfigAttributes.SCRIPT_RUBY);
			wc.setAttribute(IRailsLaunchConfigAttributes.REQUIRES_REFRESH, "true");
			wc.setAttribute(IRailsLaunchConfigAttributes.PROJECT_NAME, project.getName());

			ILaunchConfiguration config = wc.doSave();
			config.launch(ILaunchManager.RUN_MODE, null);
		} catch (CoreException e) {
			RailsLog.logError("Error running rake task", e);
		}
	}
}
