/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/
package org.radrails.rails.internal.ui.railsplugins;

import java.util.Observable;
import java.util.Observer;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.progress.UIJob;
import org.radrails.rails.core.railsplugins.RailsPluginDescriptor;
import org.radrails.rails.core.railsplugins.RailsPluginsManager;
import org.radrails.rails.internal.core.RailsPlugin;
import org.radrails.rails.internal.ui.IRailsUIConstants;
import org.radrails.rails.internal.ui.RailsUILog;

/**
 * This view provides a list of currently available Rails plugins online. The
 * Refresh button will update the list.
 * 
 * @author mkent
 * 
 */
public class RailsPluginsView extends ViewPart implements Observer {

	private ListViewer fPluginsList;

	private Text fPluginText;

	private Button fSvnextButton;

	private Button fSvnCheckoutButton;

	private Button fInstallButton;

	private Button fRemoveButton;

	private Button fGoButton;

	private StackLayout fViewLayout;

	private Composite fPluginsView;

	private Label fSelectProjectView;

	private Composite fParent;

	public void createPartControl(Composite parent) {
		fParent = parent;
		fViewLayout = new StackLayout();
		parent.setLayout(fViewLayout);

		fPluginsView = new Composite(parent, SWT.NULL);
		fPluginsView.setLayout(new GridLayout(2, false));
		fPluginsView.setLayoutData(new GridData(GridData.FILL_BOTH));

		createListViewer(fPluginsView);
		createListControls(fPluginsView);

		fSelectProjectView = new Label(parent, SWT.NULL);
		fSelectProjectView
				.setText(IRailsUIConstants.SELECT_RAILS_PROJECT_STRING);

		Job j = new UIJob("Load plugins") {
			public IStatus runInUIThread(IProgressMonitor monitor) {
				try {
					fPluginsList.setInput(RailsPluginsManager.getInstance()
							.getPlugins());
				} catch (Exception e) {
					RailsUILog.logError("Error loading Rails plugin list", e);
					MessageDialog.openError(getSite().getShell(),
							"Error retrieving plugin list", e.getMessage());
				}
				return Status.OK_STATUS;
			}
		};
		j.setUser(true);
		j.schedule();

		fViewLayout.topControl = fSelectProjectView;
		parent.layout();

		RailsPlugin.getInstance().addProjectObserver(this);
	}

	protected void createListViewer(Composite parent) {
		fPluginsList = new ListViewer(parent, SWT.SINGLE | SWT.BORDER
				| SWT.V_SCROLL | SWT.H_SCROLL);
		fPluginsList.setContentProvider(new ArrayContentProvider());
		fPluginsList.setLabelProvider(new RailsPluginsLabelProvider());
		GridData pluginsListData = new GridData(GridData.FILL_VERTICAL);
		pluginsListData.widthHint = 125;
		fPluginsList.getList().setLayoutData(pluginsListData);

		fPluginsList
				.addSelectionChangedListener(new ISelectionChangedListener() {

					public void selectionChanged(SelectionChangedEvent event) {
						IStructuredSelection sel = (IStructuredSelection) event
								.getSelection();
						RailsPluginDescriptor plg = (RailsPluginDescriptor) sel
								.getFirstElement();

						if (plg != null) {
							fPluginText.setText(plg.getProperty("description"));
						}
					}

				});
	}

	protected void createListControls(Composite parent) {
		Composite ctrlComp = new Composite(parent, SWT.NULL);
		ctrlComp.setLayout(new GridLayout(1, false));
		ctrlComp.setLayoutData(new GridData(GridData.FILL_BOTH));

		// Create plugin description field
		fPluginText = new Text(ctrlComp, SWT.MULTI | SWT.BORDER | SWT.WRAP
				| SWT.V_SCROLL);
		fPluginText.setLayoutData(new GridData(GridData.FILL_BOTH));

		Composite optsComp = new Composite(ctrlComp, SWT.NULL);
		optsComp.setLayout(new GridLayout(4, false));
		optsComp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		// Blank label to fill space
		Label blank = new Label(optsComp, SWT.NULL);
		blank.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		// Options for plugin install
		Group optsGroup = new Group(optsComp, SWT.NULL);
		optsGroup.setText("Options");
		optsGroup.setLayout(new GridLayout(2, false));
		fSvnextButton = new Button(optsGroup, SWT.CHECK);
		fSvnextButton.setText("svn:externals");
		fSvnCheckoutButton = new Button(optsGroup, SWT.CHECK);
		fSvnCheckoutButton.setText("svn:checkout");

		// Install and remove controls
		Group btnGroup = new Group(optsComp, SWT.NULL);
		btnGroup.setText("Configuration");
		btnGroup.setLayout(new GridLayout(2, false));
		fInstallButton = new Button(btnGroup, SWT.RADIO);
		fInstallButton.setText("Install");
		fInstallButton.setSelection(true);
		fRemoveButton = new Button(btnGroup, SWT.RADIO);
		fRemoveButton.setText("Remove");
		fGoButton = new Button(optsComp, SWT.PUSH);
		fGoButton.setText("Go");
		GridData goBtnData = new GridData();
		goBtnData.widthHint = 50;
		fGoButton.setLayoutData(goBtnData);

		// Action listeners
		fGoButton.addSelectionListener(new SelectionListener() {

			public void widgetSelected(SelectionEvent e) {
				if (fInstallButton.getSelection()) {
					executeInstallCommand(fSvnextButton.getSelection(),
							fSvnCheckoutButton.getSelection());
				} else {
					executeRemoveCommand();
				}
			}

			public void widgetDefaultSelected(SelectionEvent e) {

			}

		});
	}

	/**
	 * Installs the selected plugin
	 * 
	 * @param externals
	 *            true if an svn:externals entry is to be added, false otherwise
	 */
	private void executeInstallCommand(boolean externals, boolean checkout) {
		IStructuredSelection sel = (IStructuredSelection) fPluginsList
				.getSelection();
		if (sel != null) {
			RailsPluginDescriptor plugin = (RailsPluginDescriptor) sel
					.getFirstElement();
			final IProject project = RailsPlugin.getInstance()
					.getSelectedRailsOrRubyProject();
			if (project != null) {
				RailsPluginsHelper.installPlugin(project, plugin, externals,
						checkout);
			}
		}
	}

	/**
	 * Removes the selected plugin
	 */
	private void executeRemoveCommand() {
		IStructuredSelection sel = (IStructuredSelection) fPluginsList
				.getSelection();
		if (sel != null) {
			RailsPluginDescriptor plugin = (RailsPluginDescriptor) sel
					.getFirstElement();
			IProject project = RailsPlugin.getInstance()
					.getSelectedRailsOrRubyProject();

			if (project != null) {
				RailsPluginsHelper.removePlugin(project, plugin);
			}
		}
	}

	/**
	 * Refreshes the list of plugins.
	 */
	protected void refreshPlugins() {
		Job j = new UIJob("Refresh plugins") {
			public IStatus runInUIThread(IProgressMonitor monitor) {
				try {
					RailsPluginsManager.getInstance().updatePlugins(monitor);
					fPluginsList.setInput(RailsPluginsManager.getInstance()
							.getPlugins());
				} catch (Exception e) {
					RailsUILog.logError("Error loading Rails plugin list", e);
					MessageDialog.openError(getSite().getShell(),
							"Error refreshing plugin list", e.getMessage());
				}
				return Status.OK_STATUS;
			}
		};
		j.setUser(true);
		j.schedule();
	}

	public void setFocus() {

	}

	public void update(Observable o, Object arg) {
		if (!fParent.isDisposed()) {
			if (arg == null) {
				fViewLayout.topControl = fSelectProjectView;
				fParent.layout();
			} else {
				fViewLayout.topControl = fPluginsView;
				fParent.layout();
			}
		}
	}

}
