/*******************************************************************************
 * Copyright (c) 2006 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.raketasks;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.core.IStreamListener;
import org.eclipse.debug.core.Launch;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.debug.core.model.IStreamMonitor;
import org.radrails.internal.core.launching.IRailsLaunchConfigAttributes;
import org.radrails.rails.core.RailsLog;
import org.radrails.rails.internal.core.RailsPlugin;
import org.radrails.rails.internal.core.RailsRuntime;
import org.radrails.rails.internal.ui.RailsUILog;

/**
 * @author mkent
 * 
 */
public class RakeTasksHelper {

	public static void runRakeTask(IProject project, String task,
			String parameters) {
		String command = task + " " + parameters;
		try {
			ILaunchConfigurationWorkingCopy wc = RailsRuntime
					.createInternalLaunch(IRailsLaunchConfigAttributes.SCRIPT_RAKE, task);
			wc.setAttribute(IRailsLaunchConfigAttributes.PROGRAM_ARGS, command);
			wc.setAttribute(IRailsLaunchConfigAttributes.WORKING_DIR, project
					.getLocation().toOSString());
			wc.setAttribute(IRailsLaunchConfigAttributes.SCRIPT_TYPE,
					IRailsLaunchConfigAttributes.SCRIPT_RAKE);
			// refresh just to be safe, even though not every task requires it
			wc.setAttribute(IRailsLaunchConfigAttributes.REQUIRES_REFRESH, "true");
			wc.setAttribute(IRailsLaunchConfigAttributes.PROJECT_NAME, project.getName());

			ILaunchConfiguration config = wc.doSave();
			config.launch(ILaunchManager.RUN_MODE, null);
		} catch (CoreException e) {
			RailsLog.logError("Error running rake task", e);
		}
	}

	public static Map getTasks() {
		Map tasks = new HashMap();

		try {
			BufferedReader bufReader = new BufferedReader(new StringReader(
					getTasksText()));

			Pattern pat = Pattern.compile("^rake\\s+([\\w:]+)\\s+#\\s+(.+)$");
			String line = null;
			while ((line = bufReader.readLine()) != null) {
				Matcher mat = pat.matcher(line);
				if (mat.matches()) {
					tasks.put(mat.group(1), mat.group(2));
				}
			}
		} catch (IOException e) {
			RailsUILog.logError("Error parsing rake tasks", e);
		}

		return tasks;
	}

	private static String getTasksText() {
		final StringBuffer buf = new StringBuffer();

		try {
			IProject proj = RailsPlugin.getInstance().getSelectedRailsProject();
			String rakePath = RailsPlugin.getInstance().getRakePath();

			if (proj != null && rakePath != null && !rakePath.equals("")) {
				Process proc = RailsRuntime.rubyExec(new String[] { rakePath,
						"--tasks" }, proj.getLocation().toFile());
				ILaunch launch = new Launch(null, ILaunchManager.RUN_MODE, null);
				IProcess iproc = DebugPlugin.newProcess(launch, proc,
						"raketasks");

				IStreamMonitor stdOut = iproc.getStreamsProxy()
						.getOutputStreamMonitor();

				stdOut.addListener(new IStreamListener() {

					public void streamAppended(final String text,
							IStreamMonitor monitor) {
						buf.append(text);
					}

				});
				proc.waitFor();
			}

		} catch (CoreException e) {
			RailsUILog.logError("Error listing rake tasks", e);
		} catch (InterruptedException e) {
			RailsUILog.logError("Error waiting for rake process", e);
		}
		return buf.toString();
	}
}
