/*******************************************************************************
 * Copyright (c) 2006 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.raketasks;

import java.util.Collection;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.TreeSet;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.Preferences.IPropertyChangeListener;
import org.eclipse.core.runtime.Preferences.PropertyChangeEvent;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;
import org.radrails.rails.core.IRailsConstants;
import org.radrails.rails.internal.core.RailsPlugin;
import org.radrails.rails.internal.ui.IRailsUIConstants;

public class RakeTasksView extends ViewPart implements Observer,
		IPropertyChangeListener {

	private StackLayout fViewLayout;

	private Composite fRakeTasksView;

	private Label fSelectGemView;

	private Label fSelectRailsProjectView;

	private Composite fParent;

	private Combo fTasksCombo;

	private Text fParamText;

	private Text fDescripText;

	private Map fTasks;

	public RakeTasksView() {
		super();
	}

	public void createPartControl(Composite parent) {
		fParent = parent;
		fViewLayout = new StackLayout();
		parent.setLayout(fViewLayout);

		fRakeTasksView = new Composite(parent, SWT.NULL);
		fRakeTasksView.setLayout(new GridLayout(2, false));
		fRakeTasksView.setLayoutData(new GridData(GridData.FILL_BOTH));

		createRakeControls(fRakeTasksView);

		fSelectGemView = new Label(parent, SWT.NULL);
		fSelectGemView.setText(IRailsUIConstants.SET_RAKE_PATH_MESSAGE);
		fSelectRailsProjectView = new Label(parent, SWT.NULL);
		fSelectRailsProjectView
				.setText(IRailsUIConstants.SELECT_RAILS_PROJECT_STRING);
		
		if(RailsPlugin.getInstance().getRakePath() == null ||
			RailsPlugin.getInstance().getRakePath().equals("")) {
			fViewLayout.topControl = fSelectGemView;
		}
		else {
			fViewLayout.topControl = fSelectRailsProjectView;
		}
		parent.layout();

		updateRakeTasks();
		RailsPlugin.getInstance().addProjectObserver(this);
		RailsPlugin.getInstance().getPluginPreferences().addPropertyChangeListener(this);
	}

	protected void createRakeControls(Composite parent) {
		Composite comp = new Composite(parent, SWT.NULL);

		// Create the layout for the view
		GridLayout compLayout = new GridLayout();
		compLayout.numColumns = 3;
		comp.setLayout(compLayout);
		GridData compLayoutData = new GridData(GridData.FILL_BOTH);
		comp.setLayoutData(compLayoutData);

		// Create the combo box of tasks
		fTasksCombo = new Combo(comp, SWT.DROP_DOWN);
		GridData tasksComboData = new GridData();
		tasksComboData.widthHint = 200;
		fTasksCombo.setLayoutData(tasksComboData);

		fTasksCombo.addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent e) {
				// Do nothing
			}

			public void widgetSelected(SelectionEvent e) {
				Combo c = (Combo) e.widget;
				String descrip = (String) fTasks.get(c.getText());
				fDescripText.setText(descrip);
			}

		});

		// Create the parameters text field
		fParamText = new Text(comp, SWT.BORDER);
		GridData paramTextData = new GridData(GridData.FILL_HORIZONTAL);
		fParamText.setLayoutData(paramTextData);

		fParamText.addKeyListener(new KeyListener() {
			public void keyPressed(KeyEvent e) {
				// Do nothing
			}

			public void keyReleased(KeyEvent e) {
				// Take action if Enter was pressed
				if (e.character == SWT.CR) {
					runRakeTask();
				}
			}
		});

		// Create the Go button
		Button genButton = new Button(comp, SWT.PUSH);
		genButton.setText("Go");

		// Run the task when the button is clicked
		genButton.addSelectionListener(new SelectionListener() {

			public void widgetSelected(SelectionEvent e) {
				runRakeTask();
			}

			public void widgetDefaultSelected(SelectionEvent e) {
				// Do nothing
			}

		});

		// Blank
		new Label(comp, SWT.NULL);

		// Create the text area for the task descriptions
		fDescripText = new Text(comp, SWT.WRAP | SWT.MULTI | SWT.V_SCROLL
				| SWT.BORDER);
		fDescripText.setLayoutData(new GridData(GridData.FILL_BOTH));
	}

	public void dispose() {
		super.dispose();
		RailsPlugin.getInstance().deleteProjectObserver(this);
		RailsPlugin.getInstance().getPluginPreferences().removePropertyChangeListener(this);
	}

	private void runRakeTask() {
		IProject project = RailsPlugin.getInstance().getSelectedRailsProject();
		if (project != null) {
			RakeTasksHelper.runRakeTask(project, fTasksCombo.getText(),
					fParamText.getText());
		}
	}

	protected void updateRakeTasks() {
		Job j = new Job("Update rake tasks") {
			protected IStatus run(IProgressMonitor monitor) {
				monitor.beginTask("Loading rake tasks", 2);
				monitor.worked(1);
				fTasks = RakeTasksHelper.getTasks();
				PlatformUI.getWorkbench().getDisplay().syncExec(new Runnable() {
					public void run() {
						Collection sortedItems = new TreeSet(fTasks.keySet());
						fTasksCombo.setItems((String[]) sortedItems
						.toArray(new String[] {}));
					}
				});
				monitor.worked(1);
				monitor.done();
				return Status.OK_STATUS;
			}
		};
		j.schedule();
	}

	public void setFocus() {

	}

	public void update(Observable o, Object arg) {
		if (!fParent.isDisposed()) {
			if (arg == null) {
				fViewLayout.topControl = fSelectRailsProjectView;
				fParent.layout();
			} else {
				if(RailsPlugin.getInstance().getRakePath() == null ||
					RailsPlugin.getInstance().getRakePath().equals("")) {
					fViewLayout.topControl = fSelectGemView;
				}
				else {
					fViewLayout.topControl = fRakeTasksView;
				}
				fParent.layout();
			}
		}
	}

	public void propertyChange(PropertyChangeEvent event) {
		if(event.getProperty().equals(IRailsConstants.PREF_RAKE_PATH)) {
			if (!fParent.isDisposed()) {
				if (event.getNewValue() == null || event.getNewValue().equals("")) {
					fViewLayout.topControl = fSelectGemView;
					fParent.layout();
				} else {
					fViewLayout.topControl = fRakeTasksView;
					fParent.layout();
				}
			}
		}		
	}

}
