/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.tail.ANSI;

import java.util.LinkedList;
import java.util.List;

/**
 * Provides a method to parse a given String that may be ANSI encoded into tokens
 * which represent each ANSI formatted substring.
 * 
 * @author Johannes Holzer <Johannes.Holzer@gmail.com>
 *
 */
public class Parser {
	/** ASCII-Code of the ESC-Symbol marking the possible beginning of an ANSI Sequence */
	private final static byte ESC = 27;
	// states
	private final static byte TEXT = 0;
	/** the ANSI_ESC_START state is reached when an ESC-Symbol is detected */
	private final static byte ANSI_ESC_START = 1; 
	/** the ANSI state is reached when ESC was followed by [ */ 
	private final static byte ANSI = 2;
	
	public static List parse(String s) {
		if (s == null)
			return null;
		
		List tokens = new LinkedList();
		Token t = new Token();
		int state = TEXT;
		char[] c = s.toCharArray();
		StringBuffer ansiBuffer = new StringBuffer();
		/* 
		 * An ANSI-Sequence looks something like this:
		 * ESC[3;2mThis is textESC[0m
		 */
		for (int i = 0; i < c.length; i++) {
			switch (state) {
			case TEXT:
				if (c[i] == ESC) 
					state = ANSI_ESC_START;
				else
					t.add(c[i]);
				break;
			case ANSI_ESC_START:
				if (c[i] == '[') {
					state = ANSI;
					tokens.add(t);
					t = new Token();
				} else {
					state = TEXT;
				}
				break;
			case ANSI:
				if (c[i] == 'm')
					state = TEXT;
				if (c[i] == 'm' || c[i] == ';') {
					t.addProperty(Integer.parseInt(ansiBuffer.toString()));
					ansiBuffer = new StringBuffer();
				} else {
					ansiBuffer.append(c[i]);
				}
				break;
			}
			
		}
		// string didn't contain any ANSI-Sequence
		if (tokens.size() == 0)
			tokens.add(t);
		
		return tokens;
	}
	
	
}
