/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.terminal;

import java.io.File;
import java.io.PrintWriter;
import java.util.Arrays;
import java.util.LinkedList;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.core.IStreamListener;
import org.eclipse.debug.core.Launch;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.debug.core.model.IStreamMonitor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.EditorPart;
import org.radrails.rails.internal.core.RailsPlugin;
import org.radrails.rails.internal.core.RailsRuntime;
import org.radrails.rails.internal.ui.RailsUILog;

/**
 * The terminal session window.
 * 
 * @author mkent
 * @deprecated
 */
public class TerminalSession extends EditorPart {

	private static final int MAX_INPUT_HISTORY_SIZE = 25;

	private Text terminalText;

	// private Text inputField;
	private Combo inputField;

	private File projectRoot;

	private String initialCommand;

	private String terminalPrompt;

	private Process systemProcess;

	private IProcess currentProcess;

	private PrintWriter stdIn;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.ISaveablePart#doSave(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void doSave(IProgressMonitor monitor) {
		// Do nothing
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.ISaveablePart#doSaveAs()
	 */
	public void doSaveAs() {
		// Do nothing
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IEditorPart#init(org.eclipse.ui.IEditorSite,
	 *      org.eclipse.ui.IEditorInput)
	 */
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		setSite(site);
		setInput(input);

		// Save the editor input information
		TerminalSessionInput tsi = (TerminalSessionInput) getEditorInput();
		projectRoot = tsi.getProjectRoot();
		initialCommand = tsi.getCommand();

		terminalPrompt = projectRoot.getName() + "> ";
	}

	private void displayPrompt() {
		// int start = terminalText.getText().length();
		// int length = terminalPrompt.length();

		terminalText.append(terminalPrompt);

		// StyleRange bold = new StyleRange(start, length, null, null,
		// SWT.BOLD);
		// terminalText.setStyleRange(bold);
	}

	/**
	 * Sets up the terminal window after the widgets are created.
	 */
	private void initTerminal() {
		// Add a key listener to the input box
		inputField.addKeyListener(new KeyListener() {
			public void keyPressed(KeyEvent e) {
				// Do nothing
			}

			public void keyReleased(KeyEvent e) {
				// Take action if Enter was pressed
				if (e.character == 13) {

					// If there is no process running, run the input as a
					// command
					if ((currentProcess == null)
							|| currentProcess.isTerminated()) {
						runCommand(inputField.getText());
					} else {
						// Echo the input to the terminal view
						terminalText.append(inputField.getText() + "\n");

						// TODO using the write() method on the StreamsProxy
						// from IProcess doesn't work - why not?

						// Write the input to the process
						if (stdIn == null) {
							stdIn = new PrintWriter(systemProcess
									.getOutputStream());
						}
						stdIn.println(inputField.getText().trim());
						stdIn.flush();
					}

					// Clear the input box
					inputField.setText("");
				}
			}
		});

		displayPrompt();

		// Run the initial input command if there was one
		if ((initialCommand != null) && (!initialCommand.equals(""))) {
			runCommand(initialCommand);
		}
	}

	/**
	 * Run a command on the command line, in the current project directory.
	 * 
	 * @param command
	 *            the command to run
	 */
	private void runCommand(String command) {
		// Echo the command to the terminal window
		terminalText.append(command + "\n");

		// Update the input field history
		LinkedList items = new LinkedList(Arrays.asList(inputField.getItems()));
		items.addFirst(command);
		if (items.size() > MAX_INPUT_HISTORY_SIZE) {
			items.removeLast();
		}
		inputField.setItems((String[]) items.toArray(new String[0]));

		try {
			// Run the command
			if (command.startsWith("ruby ")) {
				// Strip off the "ruby" part
				command = command.substring(command.indexOf("ruby ") + 5);
				systemProcess = RailsRuntime.rubyExec(
						command.split("\\s+"), projectRoot);
			} else {
				systemProcess = RailsRuntime.exec(
						command.split("\\s+"), projectRoot);
			}

			// Wrap the process and prepare the stream listeners
			ILaunch launch = new Launch(null, ILaunchManager.RUN_MODE, null);
			currentProcess = DebugPlugin.newProcess(launch, systemProcess,
					"command");
			final IStreamMonitor stdOut = currentProcess.getStreamsProxy()
					.getOutputStreamMonitor();
			final IStreamMonitor stdErr = currentProcess.getStreamsProxy()
					.getErrorStreamMonitor();

			// Write stdout to the terminal window
			stdOut.addListener(new IStreamListener() {
				public void streamAppended(final String text,
						IStreamMonitor monitor) {
					PlatformUI.getWorkbench().getDisplay().syncExec(
							new Runnable() {
								public void run() {
									terminalText.append(text);
								}
							});
				}
			});

			// Write stderr to the terminal window
			stdErr.addListener(new IStreamListener() {
				public void streamAppended(final String text,
						IStreamMonitor monitor) {
					PlatformUI.getWorkbench().getDisplay().syncExec(
							new Runnable() {
								public void run() {
									terminalText.append(text);
								}
							});
				}
			});

			final IProject project = RailsPlugin.getInstance().getSelectedRailsOrRubyProject();
			
			if ( project != null) {

				Thread refreshThread = new Thread(new Runnable() {
					public void run() {
						try {
							systemProcess.waitFor();

							// Refresh the project contents
							project.refreshLocal(IResource.DEPTH_INFINITE,  null);

							// Close the input stream
							if (stdIn != null) {
								stdIn.close();
								stdIn = null;
							}

							// Reset the prompt
							PlatformUI.getWorkbench().getDisplay().syncExec(
									new Runnable() {
										public void run() {
											displayPrompt();
										}
									});

						} catch (CoreException e) {
							RailsUILog.logError("Error refreshing project", e);
						} catch (InterruptedException e) {
							RailsUILog.logError(
									"Error waiting for command process to end",
									e);
						}
					}
				});
				refreshThread.start();
			}
		} catch (CoreException e) {
			RailsUILog.logError("Error running the command", e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.ISaveablePart#isDirty()
	 */
	public boolean isDirty() {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.ISaveablePart#isSaveAsAllowed()
	 */
	public boolean isSaveAsAllowed() {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {
		// Set layout
		parent.setLayout(new GridLayout());

		// Create terminal text area
		terminalText = new Text(parent, SWT.BORDER | SWT.MULTI | SWT.READ_ONLY
				| SWT.V_SCROLL | SWT.H_SCROLL);
		GridData terminalTextData = new GridData(GridData.FILL_BOTH);
		terminalText.setLayoutData(terminalTextData);

		// Override the default background color of gray
		Color white = PlatformUI.getWorkbench().getDisplay().getSystemColor(
				SWT.COLOR_WHITE);
		terminalText.setBackground(white);

		// Create input controls
		Composite inputControls = new Composite(parent, SWT.NULL);
		inputControls.setLayout(new GridLayout(2, false));
		inputControls.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Label promptLabel = new Label(inputControls, SWT.NULL);
		promptLabel.setText(terminalPrompt);

		inputField = new Combo(inputControls, SWT.DROP_DOWN);
		GridData inputTextData = new GridData(GridData.FILL_HORIZONTAL);
		inputField.setLayoutData(inputTextData);

		initTerminal();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchPart#setFocus()
	 */
	public void setFocus() {
		inputField.setFocus();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchPart#dispose()
	 */
	public void dispose() {
		super.dispose();

		// Close the input stream for the process
		if (stdIn != null) {
			stdIn.close();
		}

		// Stop the process if necessary
		if ((currentProcess != null) && !currentProcess.isTerminated()) {
			try {
				currentProcess.terminate();
			} catch (DebugException e) {
				RailsUILog.logError("Error stopping terminal process", e);
			}
		}
	}

}
