/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.wizards.pages;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;
import org.radrails.rails.internal.ui.wizards.NewRailsProjectWizard;

/**
 * Main dialog page for {@link NewRailsProjectWizard NewRailsProjectWizard}.
 * 
 * @author mkent
 * 
 */
public class WizardNewRailsProjectPage extends WizardNewProjectCreationPage {

	private Button generateButton;

	private Button webrickButton;
	
	private Button mongrelButton;

	private Button disablePluralizationButton;

	/**
	 * Constructor.
	 * 
	 * @param pageName
	 *            the name of the wizard page
	 */
	public WizardNewRailsProjectPage(String pageName) {
		super(pageName);
	}

	/**
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		super.createControl(parent);

		Composite composite = (Composite) super.getControl();

		createOptionsGroup(composite);

		setControl(composite);
	}

	/**
	 * Helper method to create the options widgets.
	 * 
	 * @param parent
	 *            the parent composite
	 */
	private void createOptionsGroup(Composite parent) {
		// Create the group
		Group optionsGroup = new Group(parent, SWT.NONE);
		optionsGroup.setLayout(new GridLayout());
		optionsGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		optionsGroup.setText("Options");

		// Create the check box
		generateButton = new Button(optionsGroup, SWT.CHECK);
		generateButton.setText("Generate Rails application skeleton");
		generateButton.setSelection(true);

		// Create the check box
		webrickButton = new Button(optionsGroup, SWT.CHECK);
		webrickButton.setText("Create a WEBrick server");
		webrickButton.setSelection(true);
		
		mongrelButton = new Button(optionsGroup, SWT.CHECK);
		mongrelButton.setText("Create a Mongrel server");
		mongrelButton.setSelection(false);

		// Create the check box
		disablePluralizationButton = new Button(optionsGroup, SWT.CHECK);
		disablePluralizationButton.setText("Disable table pluralization");
		disablePluralizationButton.setSelection(false);
	}

	/**
	 * @return the selection status of the generate rails skeleton option button
	 */
	public boolean getGenerateButtonSelection() {
		return generateButton.getSelection();
	}

	/**
	 * @return the selection status of the create webrick server button
	 */
	public boolean getWebrickButtonSelection() {
		return webrickButton.getSelection();
	}
	
	public boolean getMongrelButtonSelection() {
		return mongrelButton.getSelection();
	}

	/**
	 * @return the enable status of the create webrick server button
	 */
	public boolean getWebrickButtonEnabled() {
		return webrickButton.getEnabled();
	}

	/**
	 * 
	 * @return the selection status of the disable pluralization button
	 */
	public boolean getPluralizationButtonSelection() {
		return disablePluralizationButton.getSelection();
	}

}