package org.radrails.rails.ui;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.radrails.rails.internal.ui.RailsUILog;
import org.radrails.rails.internal.ui.raketasks.RakeTasksHelper;

/**
 * @author matt
 *
 */
public class RunMigrationAction implements IObjectActionDelegate {

	private IWorkbenchPart fPart;
	
	public RunMigrationAction() {
	}

	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
		fPart = targetPart;
	}

	public void run(IAction action) {
		IStructuredSelection sel = (IStructuredSelection) fPart.getSite().getSelectionProvider().getSelection();
		if(sel != null) {
			IFile file = (IFile) sel.getFirstElement();
			String fileName = file.getName();
			// get the migration version number from the file name
			String version = fileName.substring(0, fileName.indexOf('_'));
			try {
				int versionNum = Integer.parseInt(version);
				String params = "VERSION=" + versionNum;
				RakeTasksHelper.runRakeTask(file.getProject(), "db:migrate", params);
			} catch (NumberFormatException e) {
				RailsUILog.logError("Invalid migration number", e);
				MessageDialog.openError(fPart.getSite().getShell(), "Error running migration", "Invalid migration number: " + version);
			}
		}
	}

	public void selectionChanged(IAction action, ISelection selection) {

	}

}
