package org.radrails.rails.ui.rake;

import java.io.File;
import java.util.ArrayList;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.core.IStreamListener;
import org.eclipse.debug.core.Launch;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.debug.core.model.IStreamMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.MessageConsole;
import org.eclipse.ui.console.MessageConsoleStream;
import org.radrails.rails.internal.core.RailsPlugin;
import org.radrails.rails.internal.core.RailsRuntime;
import org.radrails.rails.internal.ui.RailsUILog;

/**
 * Runs a Rake command.
 * 
 * @author Kyle
 * @deprecated
 */
public class RakeAction implements IActionDelegate {

	private MessageConsoleStream consoleStream;

	public RakeAction() {
		super();
	}

	public void run(IAction action) {

		IProject project = RailsPlugin.getInstance().getSelectedRailsOrRubyProject();

		if ( project != null) {
			File projectFile = project.getLocation().toFile();
			ArrayList commands = new ArrayList();
			commands.add(RailsPlugin.getInstance().getRubyScriptPath("run_task.rb"));
			commands.add(action.getText());
			try {
				Process proc = RailsRuntime.rubyExec(
						(String[])commands.toArray(new String[]{}), projectFile, false);
				ILaunch launch = new Launch(null, ILaunchManager.RUN_MODE, null);
				IProcess rakeProcess = DebugPlugin.newProcess(launch, proc,
				"rake_command");

				MessageConsole myConsole = new MessageConsole("rake "
						+ action.getText(), null);
				ConsolePlugin.getDefault().getConsoleManager().showConsoleView(
						myConsole);
				consoleStream = myConsole.newMessageStream();
				consoleStream.setColor(PlatformUI.getWorkbench().getDisplay()
						.getSystemColor(SWT.COLOR_BLUE));
				ConsolePlugin.getDefault().getConsoleManager().addConsoles(
						new IConsole[] { myConsole });

				// Prepare the stream listeners
				IStreamMonitor stdOut = rakeProcess.getStreamsProxy()
				.getOutputStreamMonitor();
				IStreamMonitor stdErr = rakeProcess.getStreamsProxy()
				.getErrorStreamMonitor();

				// Write stdout to the console
				stdOut.addListener(new IStreamListener() {
					public void streamAppended(final String text,
							IStreamMonitor monitor) {
						// Log the output to the console
						consoleStream.print(text);
					}
				});

				// Write stdout to the console
				stdErr.addListener(new IStreamListener() {
					public void streamAppended(final String text,
							IStreamMonitor monitor) {
						// Log the output to the console
						consoleStream.print(text);
					}
				});

				// Wait for process to finish and refresh
				proc.waitFor();
				project.refreshLocal(IResource.DEPTH_INFINITE, null);
			} catch (CoreException e) {
				RailsUILog.logError("Error running Rake task", e);
			} catch (InterruptedException e) {
				RailsUILog.logError("Error waiting for process to end", e);
			}
		}
	}

	public void selectionChanged(IAction action, ISelection selection) {
	}

}
