package org.radrails.rails.ui.test;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationType;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;
import org.radrails.rails.internal.core.RailsPlugin;
import org.radrails.rails.internal.core.RailsRuntime;
import org.radrails.rails.internal.ui.IRailsUIConstants;
import org.radrails.rails.internal.ui.RailsUILog;
import org.rubypeople.rdt.internal.launching.RubyLaunchConfigurationAttribute;
import org.rubypeople.rdt.internal.launching.RubyRuntime;
import org.rubypeople.rdt.testunit.launcher.TestUnitLaunchConfigurationDelegate;
import org.rubypeople.rdt.testunit.launcher.TestUnitLaunchShortcut;

/**
 * Launches a group of Test::Unit tests.
 * 
 * @author Kyle
 * 
 */
public class TestLauncher extends TestUnitLaunchShortcut {

	protected IProject fProject;

	public TestLauncher() {
		super();
	}

	public void goLaunch(IProject project, String mode, String testFile) {
		try {
			fProject = project;
			doLaunch(mode, testFile);
		} catch (CoreException e) {
			RailsUILog.logError("Error launching tests", e);
		}
	}

	protected void doLaunch(String mode, String testFile) throws CoreException {
		String rakePath = RailsPlugin.getInstance().getRakePath();
		if((rakePath == null) || rakePath.equals("")) {
			PlatformUI.getWorkbench().getDisplay().syncExec(new Runnable() {
				public void run() {
					Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
					MessageDialog.openError(shell,
							"Error running tests", IRailsUIConstants.SET_RAKE_PATH_MESSAGE);
				}
			});
			return;
		}
		
		try {
			Process p = RailsRuntime.rakeExec(
					new String[] { "db:test:prepare" }, fProject
							.getLocation().toFile());
			p.waitFor();
		} catch (InterruptedException e) {
			RailsUILog
					.logError("Error waiting for test database to prepare", e);
		}
		ILaunchConfiguration config = createConfiguration(testFile);
		if (config != null) {
			config.launch(mode, null);
		}
	}

	protected ILaunchConfiguration createConfiguration(String testFile) {
		if (RubyRuntime.getDefault().getSelectedInterpreter() == null) {
			PlatformUI.getWorkbench().getDisplay().syncExec(new Runnable() {
				public void run() {
					showNoInterpreterDialog();
				}
			});
			return null;
		}
		ILaunchConfiguration config = null;
		try {
			ILaunchConfigurationType configType = getRubyLaunchConfigType();
			ILaunchConfigurationWorkingCopy wc = configType
					.newInstance(null,
							getLaunchManager()
									.generateUniqueLaunchConfigurationNameFrom(
											testFile));
			wc.setAttribute(RubyLaunchConfigurationAttribute.PROJECT_NAME,
					fProject.getName());
			wc.setAttribute(RubyLaunchConfigurationAttribute.WORKING_DIRECTORY,
					fProject.getLocation().toOSString());
			wc
					.setAttribute(
							RubyLaunchConfigurationAttribute.SELECTED_INTERPRETER,
							RubyRuntime.getDefault().getSelectedInterpreter()
									.getName());
			String tfPath = RailsPlugin.getInstance().getRubyScriptPath(
					testFile);
			if (tfPath.indexOf(" ") > -1) {
				tfPath = "'" + tfPath + "'";
			}
			wc.setAttribute(
					TestUnitLaunchConfigurationDelegate.LAUNCH_CONTAINER_ATTR,
					tfPath);
			wc.setAttribute(TestUnitLaunchConfigurationDelegate.TESTNAME_ATTR,
					"");
			wc.setAttribute(TestUnitLaunchConfigurationDelegate.TESTTYPE_ATTR,
					"");
			wc.setAttribute(ILaunchConfiguration.ATTR_SOURCE_LOCATOR_ID,
					"org.rubypeople.rdt.debug.ui.rubySourceLocator");
			config = wc.doSave();
		} catch (CoreException e) {
			RailsUILog.logError(
					"Error retrieving launch configuration working copy", e);
		}
		return config;
	}

}
