/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/
package org.radrails.server.core;

import java.util.Observable;
import java.util.Observer;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.console.MessageConsole;

/**
 * Console page for servers.
 * 
 * @author Kyle
 * 
 */
public class ServerConsole extends MessageConsole implements Observer {

	private Server server;

	private boolean stopped;

	/**
	 * Constructor. Adds this console to the manager.
	 * 
	 * @param server
	 *            the server to monitor
	 * @param name
	 *            the name of the console
	 * @param imageDescriptor
	 *            the icon for the console
	 */
	public ServerConsole(Server server, String name,
			ImageDescriptor imageDescriptor) {
		super(name, imageDescriptor);

		ServerConsoleManager.getInstance().addConsole(this);
		ServerManager.getInstance().addServerObserver(this);

		this.server = server;
		stopped = false;
	}

	/**
	 * Stop the underlying server.
	 */
	public void stopServer() {
		server.stop();
	}

	/**
	 * "Stop" the console by no longer observing the underlying server.
	 */
	private void stopConsole() {
		ServerManager.getInstance().deleteServerObserver(this);
		stopped = true;
		setName("<terminated> " + getName());
	}

	/**
	 * @return true if the server is stopped, false otherwise
	 */
	public boolean isStopped() {
		return stopped;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.console.AbstractConsole#dispose()
	 */
	protected void dispose() {
		super.dispose();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
	 */
	public void update(Observable arg0, Object arg1) {
		String action = (String) arg1;
		if (action.equals(IServerConstants.UPDATE)) {
			Server s = (Server) arg0;
			if (s.isStopped()) {
				stopConsole();
			}
		}
	}

}
