/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/
package org.radrails.server.core;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;

/**
 * Manages the active Server consoles.
 * 
 * @author Kyle
 * 
 */
public class ServerConsoleManager {

	private static ServerConsoleManager instance;

	private List consoles;

	private ServerConsoleManager() {
		consoles = new ArrayList();
	}

	/**
	 * @return the singleton instance of the manager
	 */
	public static ServerConsoleManager getInstance() {
		if (instance == null) {
			instance = new ServerConsoleManager();
		}
		return instance;
	}

	/**
	 * Adds a console to the console manager.
	 * 
	 * @param console
	 *            the console to add
	 */
	public void addConsole(ServerConsole console) {
		consoles.add(console);
	}

	/**
	 * Removes all terminated server consoles from the ConsolePlugin console
	 * manager and from this manager.
	 */
	public void removeTerminatedConsoles() {
		Iterator i = consoles.iterator();
		while (i.hasNext()) {
			ServerConsole c = (ServerConsole) i.next();
			// Remove the server if it's stopped
			if (c.isStopped()) {
				ConsolePlugin.getDefault().getConsoleManager().removeConsoles(
						new IConsole[] { c });
				i.remove();
			}
		}
	}

}
