/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/
package org.radrails.server.core;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;

import org.radrails.server.core.lighttpd.LightTPDServer;
import org.radrails.server.core.mongrel.MongrelServer;
import org.radrails.server.core.webrick.WEBrickServer;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 * Content handler for the server manager configuration file. Creates a new
 * server object for each <code>server</code> element and stores it in a
 * Collection for retrieval. The <code>type</code> attribute in the
 * <code>server</code> element determines which subclass of Server will be
 * instantiated.
 * 
 * @author mkent
 * 
 */
public class ServerManagerContentHandler implements ContentHandler {

	private StringBuffer data;

	private Collection servers;

	private Server currentServer;

	public void endDocument() throws SAXException {
		// Do nothing
	}

	public void startDocument() throws SAXException {
		servers = new ArrayList();
	}

	public void characters(char[] ch, int start, int length)
			throws SAXException {
		for (int i = start; i < start + length; i++) {
			data.append(ch[i]);
		}
	}

	public void ignorableWhitespace(char[] ch, int start, int length)
			throws SAXException {
		// Do nothing
	}

	public void endPrefixMapping(String prefix) throws SAXException {
		// Do nothing
	}

	public void skippedEntity(String name) throws SAXException {
		// Do nothing
	}

	public void setDocumentLocator(Locator locator) {
		// Do nothing
	}

	public void processingInstruction(String target, String data)
			throws SAXException {
		// Do nothing
	}

	public void startPrefixMapping(String prefix, String uri)
			throws SAXException {
		// Do nothing
	}

	public void endElement(String namespaceURI, String localName, String qName)
			throws SAXException {
		if (qName.equals("project")) {
			File proj = new File(data.toString());
			currentServer.project = proj;
		} else if (qName.equals("name")) {
			currentServer.name = data.toString();
		} else if (qName.equals("port")) {
			currentServer.port = data.toString();
		} else if (qName.equals("environment")) {
			currentServer.environment = data.toString();
		} else if (qName.equals("server")) {
			servers.add(currentServer);
		}
	}

	public void startElement(String namespaceURI, String localName,
			String qName, Attributes atts) throws SAXException {
		data = new StringBuffer();
		String type = atts.getValue("type");
		if ((type != null) && type.equals(IServerConstants.TYPE_WEBRICK)) {
			currentServer = new WEBrickServer();
		}
		else if((type != null) && type.equals(IServerConstants.TYPE_LIGHTTPD)) {
			currentServer = new LightTPDServer();
		}
		else if((type != null) && type.equals(IServerConstants.TYPE_MONGREL)) {
			currentServer = new MongrelServer();
		}
	}

	public Collection getServers() {
		return servers;
	}

}
