/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.server.core;

import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;

import org.eclipse.core.runtime.Plugin;
import org.osgi.framework.BundleContext;

/**
 * Plugin class for the server.core plugin.
 * 
 * @author mkent
 * 
 */
public class ServerPlugin extends Plugin {

	private static ServerPlugin instance;

	/**
	 * Default constructor.
	 */
	public ServerPlugin() {
		super();
		instance = this;
	}

	/**
	 * @return the singleton instance of <code>ServerPlugin</code>
	 */
	public static ServerPlugin getInstance() {
		return instance;
	}

	/**
	 * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);

		// Restore the state of the ServerManager
		ServerManager.getInstance().loadServers();
	}

	/**
	 * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);

		// Save the state of the ServerManager
		ServerManager.getInstance().saveServers();
	}

	/**
	 * Checks if the given port is available on localhost.
	 * 
	 * @param port
	 *            the port to check
	 * @return true if the port is available, false otherwise
	 */
	public static boolean isPortAvailable(String port) {
		boolean available = true;

		Socket socket = null;
		try {
			// Get the int value of the port
			int intPort = Integer.parseInt(port);

			// Attempt to connect to the localhost on that port
			// If an error occurs, we weren't able to connect, which
			// means the port is available.
			socket = new Socket();
			socket.connect(new InetSocketAddress(InetAddress
					.getByName("localhost"), intPort));
			if (socket.isConnected()) {
				// Connection established, which means the port
				// is in use
				available = false;
			}
		} catch (NumberFormatException e) {
			// The port couldn't be parsed
			available = false;
		} catch (IOException e) {
			// Could not connect, the port is available
			available = true;
		} finally {
			// Close the socket if necessary
			if (socket != null && !socket.isClosed()) {
				try {
					socket.close();
				} catch (IOException e) {
					// Log this, even though it shouldn't happen
					ServerLog.logError("Unable to close server socket.", e);
				}
			}
		}

		return available;
	}
}
