/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.server.internal.ui;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.radrails.server.core.IServerConstants;
import org.radrails.server.core.Server;

/**
 * Label provider for the server view.
 * 
 * @author mkent
 * 
 */
public class ServerLabelProvider extends LabelProvider implements
		ITableLabelProvider {

	/**
	 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object,
	 *      int)
	 */
	public Image getColumnImage(Object element, int columnIndex) {
		Server server = (Server) element;
		Image image = null;
		ImageDescriptor imageDesc = null;

		if (columnIndex == 0) {
			if (server.getType().equals(IServerConstants.TYPE_WEBRICK)) {
				imageDesc = AbstractUIPlugin.imageDescriptorFromPlugin(
						ServerUIPlugin.getInstance().getBundle()
								.getSymbolicName(), "icons/webrick.gif");
			}
			if (server.getType().equals(IServerConstants.TYPE_LIGHTTPD)) {
				imageDesc = AbstractUIPlugin.imageDescriptorFromPlugin(
						ServerUIPlugin.getInstance().getBundle()
								.getSymbolicName(), "icons/lighttpd.gif");
			}
			if (server.getType().equals(IServerConstants.TYPE_MONGREL)) {
				imageDesc = AbstractUIPlugin.imageDescriptorFromPlugin(
						ServerUIPlugin.getInstance().getBundle()
								.getSymbolicName(), "icons/mongrel.gif");
			}
		}
		if (imageDesc != null) {
			image = imageDesc.createImage();
		}
		return image;
	}

	/**
	 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object,
	 *      int)
	 */
	public String getColumnText(Object element, int columnIndex) {
		Server server = (Server) element;
		String label;

		switch (columnIndex) {
		case 1:
			label = server.getName();
			break;
		case 2:
			label = server.getProject().getName();
			break;
		case 3:
			label = server.getStatus();
			break;
		case 4:
			label = server.getPort();
			break;
		case 5:
			label = server.getEnvironment();
			break;
		default:
			label = "";
		}
		return label;
	}
}
