/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.server.internal.ui;

import java.util.Observable;
import java.util.Observer;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.ViewPart;
import org.radrails.server.core.IServerConstants;
import org.radrails.server.core.ServerManager;

/**
 * This view displays the existing web servers in the workspace. Users can
 * start, stop and edit the servers from this view.
 * 
 * @author mkent
 * 
 */
public class ServersView extends ViewPart implements Observer {

	private TableViewer serverViewer;

	/**
	 * Constructor.
	 */
	public ServersView() {
		super();
	}

	/**
	 * @see org.eclipse.ui.IViewPart#init(org.eclipse.ui.IViewSite)
	 */
	public void init(IViewSite site) throws PartInitException {
		super.init(site);
	}

	/**
	 * @see org.eclipse.ui.IWorkbenchPart#dispose()
	 */
	public void dispose() {
		super.dispose();

		ServerManager.getInstance().deleteServerObserver(this);
	}

	public ISelection getSelection() {
		return serverViewer.getSelection();
	}

	/**
	 * @see org.eclipse.ui.IWorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {
		parent.setLayout(new GridLayout());

		createTableViewer(parent);

		ServerManager.getInstance().addServerObserver(this);
		serverViewer.add(ServerManager.getInstance().getServers().toArray());
	}

	/**
	 * Helper method to create the table viewer.
	 * 
	 * @param parent
	 */
	private void createTableViewer(Composite parent) {
		serverViewer = new TableViewer(parent, SWT.SINGLE | SWT.FULL_SELECTION);
		Table serverTable = serverViewer.getTable();
		serverTable.setHeaderVisible(true);
		serverTable.setLinesVisible(false);
		serverTable.setLayoutData(new GridData(GridData.FILL_BOTH));

		TableColumn imageColumn = new TableColumn(serverTable, SWT.LEFT);
		imageColumn.setWidth(20);

		TableColumn nameColumn = new TableColumn(serverTable, SWT.LEFT);
		nameColumn.setText("Name");
		nameColumn.setWidth(100);

		TableColumn projectColumn = new TableColumn(serverTable, SWT.LEFT);
		projectColumn.setText("Project");
		projectColumn.setWidth(100);

		TableColumn statusColumn = new TableColumn(serverTable, SWT.LEFT);
		statusColumn.setText("Status");
		statusColumn.setWidth(125);

		TableColumn portColumn = new TableColumn(serverTable, SWT.LEFT);
		portColumn.setText("Port");
		portColumn.setWidth(50);

		TableColumn environmentColumn = new TableColumn(serverTable, SWT.LEFT);
		environmentColumn.setText("Environment");
		environmentColumn.setWidth(200);

		serverViewer.setLabelProvider(new ServerLabelProvider());
		serverViewer.setContentProvider(new ArrayContentProvider());
		getSite().setSelectionProvider(serverViewer);
	}

	/**
	 * @see org.eclipse.ui.IWorkbenchPart#setFocus()
	 */
	public void setFocus() {
		// Do nothing
	}

	/**
	 * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
	 */
	public void update(Observable arg0, Object arg1) {
		String action = (String) arg1;
		if (action.equals(IServerConstants.ADD)) {
			serverViewer.add(arg0);
		} else if (action.equals(IServerConstants.REMOVE)) {
			serverViewer.remove(arg0);
		} else if (action.equals(IServerConstants.UPDATE)) {
			serverViewer.update(arg0, null);
			getSite().getSelectionProvider().setSelection(serverViewer.getSelection());
		}
	}

}
