/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.server.internal.ui.actions;

import java.util.Arrays;
import java.util.List;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IViewActionDelegate;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPart;
import org.radrails.server.core.IServerConstants;
import org.radrails.server.core.Server;
import org.radrails.server.internal.ui.ServersView;
import org.radrails.server.internal.ui.dialogs.EditServerDialog;

/**
 * Edit server properties from Server view
 * 
 * @author mkent
 */
public class EditServerActionDelegate implements IObjectActionDelegate,
		IViewActionDelegate {

	private static final List ENABLED_FOR = Arrays.asList(new String[] { IServerConstants.STOPPED });
	
	private IViewPart view;

	/**
	 * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.action.IAction,
	 *      org.eclipse.ui.IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {

	}

	/**
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	public void run(IAction action) {
		IStructuredSelection sel = (IStructuredSelection) ((ServersView) view)
				.getSelection();
		if (sel.getFirstElement() != null) {
			Server server = (Server) sel.getFirstElement();

			if (server.isStopped()) {
				// Display a dialog to edit server options: name, port,
				// environment
				EditServerDialog dialog = new EditServerDialog(view
						.getViewSite().getShell(), server.getName(), server
						.getPort(), server.getEnvironment());
				int returnCode = dialog.open();

				if (returnCode == Window.OK) {
					server.updateName(dialog.getName());
					server.updatePort(dialog.getPort());
					server.updateEnvironment(dialog.getEnvironment());
				}
			}
		}
	}

	/**
	 * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction,
	 *      org.eclipse.jface.viewers.ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		ServerActionHelper.enableAction(action, selection, ENABLED_FOR);
	}

	/**
	 * @see org.eclipse.ui.IViewActionDelegate#init(org.eclipse.ui.IViewPart)
	 */
	public void init(IViewPart view) {
		this.view = view;
	}

}
