/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.server.internal.ui.actions;

import java.util.Arrays;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IViewActionDelegate;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.radrails.rails.internal.ui.browser.BrowserInput;
import org.radrails.server.core.IServerConstants;
import org.radrails.server.core.Server;
import org.radrails.server.internal.ui.ServersView;

/**
 * Launches browser from Servers view
 * 
 * @author Kyle Shank
 */
public class ViewBrowserActionDelegate implements Observer,
		IObjectActionDelegate, IViewActionDelegate {

	private static final List ENABLED_FOR = Arrays.asList(new String[] { IServerConstants.STARTED, IServerConstants.STOPPED });
	
	private IViewPart view;

	/**
	 * @see org.eclipse.ui.IViewActionDelegate#init(org.eclipse.ui.IViewPart)
	 */
	public void init(IViewPart view) {
		this.view = view;
	}

	/**
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	public void run(IAction action) {
		IStructuredSelection sel = (IStructuredSelection) ((ServersView) view)
				.getSelection();
		if (sel.getFirstElement() != null) {
			final Server server = (Server) sel.getFirstElement();

			boolean startServer = false;
			if (server.isStopped()) {
				startServer = MessageDialog
						.openQuestion(
								view.getViewSite().getShell(),
								"Server Error",
								"RadRails has noticed that your server is currently stopped. Do you want to start the server?");
			}

			if (startServer) {
				server.addObserver(this);
				ServerActionHelper.startServer(view, server);
			} else {
				update(server, null);
			}
		}
	}

	/**
	 * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction,
	 *      org.eclipse.jface.viewers.ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		ServerActionHelper.enableAction(action, selection, ENABLED_FOR);
	}

	/**
	 * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.action.IAction,
	 *      org.eclipse.ui.IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
		// Do nothing
	}

	/**
	 * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
	 */
	public void update(Observable arg0, Object arg1) {
		final Server server = (Server) arg0;
		if (server.isStarted()) {
			Display display = PlatformUI.getWorkbench().getDisplay();
			display.asyncExec(new Runnable() {

				public void run() {
					try {
						String port = String.valueOf(server.getPort());
						IEditorInput editorInput = new BrowserInput(
								"http://localhost:" + port);
						PlatformUI
								.getWorkbench()
								.getActiveWorkbenchWindow()
								.getActivePage()
								.openEditor(editorInput,
										"org.radrails.rails.ui.browser.BrowserEditor");
					} catch (Exception e) {
						e.printStackTrace();
					}
				}

			});
			arg0.deleteObserver(this);
		}
	}

}
