/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.server.internal.ui.console;

import org.eclipse.ui.IActionBars;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.IConsoleConstants;
import org.eclipse.ui.console.IConsolePageParticipant;
import org.eclipse.ui.part.IPageBookViewPage;
import org.radrails.server.core.ServerConsole;

/**
 * Page participant for the ServerConsole. Adds a remove all terminated action
 * and a stop action.
 * 
 * @author Kyle
 * 
 */
public class ServerConsolePageParticipant implements IConsolePageParticipant {

	private ServerRemoveAction consoleRemoveAction;

	private ServerStopAction consoleStopAction;

	private ServerConsole console;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.console.IConsolePageParticipant#init(org.eclipse.ui.part.IPageBookViewPage,
	 *      org.eclipse.ui.console.IConsole)
	 */
	public void init(IPageBookViewPage page, IConsole console) {
		this.console = (ServerConsole) console;
		this.consoleRemoveAction = new ServerRemoveAction();
		this.consoleStopAction = new ServerStopAction(this.console);

		// Add the actions to the toolbar
		IActionBars bars = page.getSite().getActionBars();
		bars.getToolBarManager().appendToGroup(IConsoleConstants.LAUNCH_GROUP,
				consoleRemoveAction);
		bars.getToolBarManager().appendToGroup(IConsoleConstants.LAUNCH_GROUP,
				consoleStopAction);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.console.IConsolePageParticipant#dispose()
	 */
	public void dispose() {
		this.consoleRemoveAction = null;
		this.console = null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.console.IConsolePageParticipant#activated()
	 */
	public void activated() {
		// Do nothing
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.console.IConsolePageParticipant#deactivated()
	 */
	public void deactivated() {
		// Do nothing
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class adapter) {
		return null;
	}

}
