/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.server.internal.ui.dialogs;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.radrails.db.core.IDatabaseConstants;

/**
 * Dialog to display server properties for editing.
 * 
 * @author mkent
 */
public class EditServerDialog extends Dialog {

	private Text nameText;

	private Text portText;

	private Combo envCombo;

	private String currName;

	private String currPort;

	private String currEnvironment;

	private String name;

	private String port;

	private String environment;

	/**
	 * Constructor.
	 * 
	 * @param parentShell
	 *            parent shell of the dialog
	 * @param currName
	 *            current name of the server
	 * @param currPort
	 *            current port of the server
	 * @param currEnvironment
	 *            current runtime environment of the server
	 */
	public EditServerDialog(Shell parentShell, String currName,
			String currPort, String currEnvironment) {
		super(parentShell);

		this.currName = currName;
		this.currPort = String.valueOf(currPort);
		this.currEnvironment = currEnvironment;
	}

	/**
	 * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
	 */
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText("Server properties");
	}

	/**
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) {
		Composite control = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		control.setLayout(layout);

		Label nameLabel = new Label(control, SWT.LEFT);
		nameLabel.setText("Name:");

		nameText = new Text(control, SWT.BORDER);
		GridData nameTextData = new GridData();
		nameTextData.widthHint = 150;
		nameText.setLayoutData(nameTextData);
		nameText.setText(currName);

		Label portLabel = new Label(control, SWT.LEFT);
		portLabel.setText("Port:");

		portText = new Text(control, SWT.BORDER);
		GridData portTextData = new GridData(GridData.FILL_HORIZONTAL);
		portText.setLayoutData(portTextData);
		portText.setText(currPort);

		Label envLabel = new Label(control, SWT.LEFT);
		envLabel.setText("Environment:");

		envCombo = new Combo(control, SWT.DROP_DOWN);
		envCombo.add(IDatabaseConstants.ENV_DEVELOPMENT);
		envCombo.add(IDatabaseConstants.ENV_PRODUCTION);
		envCombo.add(IDatabaseConstants.ENV_TEST);
		GridData envComboData = new GridData(GridData.FILL_HORIZONTAL);
		envCombo.setLayoutData(envComboData);
		envCombo.setText(currEnvironment);

		return control;
	}

	/**
	 * @see org.eclipse.jface.dialogs.Dialog#buttonPressed(int)
	 */
	public void buttonPressed(int buttonId) {
		if (buttonId == IDialogConstants.OK_ID) {
			name = nameText.getText();
			port = portText.getText();
			environment = envCombo.getText();
			okPressed();
		} else if (buttonId == IDialogConstants.CANCEL_ID) {
			cancelPressed();
		}
	}

	/**
	 * Return server name
	 * 
	 * @return name the name of the server
	 */
	public String getName() {
		return name;
	}

	/**
	 * Return server port
	 * 
	 * @return port the port the server runs on
	 */
	public String getPort() {
		return port;
	}

	/**
	 * Return server environment
	 * 
	 * @return environment the runtime environment of the server
	 */
	public String getEnvironment() {
		return environment;
	}
}
